import {DOCUMENT} from "@angular/common";
import {
  Component,
  HostListener,
  Inject,
  Renderer2,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {
  Router,
  RouterStateSnapshot,
} from "@angular/router";
import {CartDialog} from "@app/components/dialogs/cart/cart.dialog";
import {UpdateVersionDialog} from "@app/components/dialogs/update-version/update-version.dialog";
import {UserDialog} from "@app/components/dialogs/user/user.dialog";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {ApplicationRoleEnum} from "@app/shared/enums/application-role.enum";
import {LanguagesEnum} from "@app/shared/enums/languages.enum";
import {
  RoutesEnum,
  urlSeparator,
} from "@app/shared/enums/routes.enum";
import {ThemeEnum} from "@app/shared/enums/theme.enum";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {AppAction} from "@app/stores/app.action";
import {AppCartArchiveState} from "@app/stores/cart/archive/app-cart-archive.state";
import {environment} from "@environments/environment";
import {HomeHelper} from "@home/helpers/home.helper";
import {User} from "@models";
import {TranslateService} from "@ngx-translate/core";
import {Navigate} from "@ngxs/router-plugin";
import {
  Select,
  Store,
} from "@ngxs/store";
import {ApplicationRolePermissionEnum} from "@shared/enums/application-role-permission.enum";
import {ChemicalMoleculeVisualizationEnum} from "@shared/enums/chemical-molecule-visualization.enum";
import {SessionStorageEnum} from "@shared/enums/session-storage.enum";
import {SessionStorageHelper} from "@shared/helpers/session-storage.helper";
import {UrlQueryParamHelper} from "@shared/helpers/url-query-param.helper";
import {AppStatusService} from "@shared/services/app-status.service";
import {BreakpointService} from "@shared/services/breakpoint.service";
import {PermissionUtil} from "@shared/utils/permission.util";
import {
  combineLatest,
  Observable,
} from "rxjs";
import {tap} from "rxjs/internal/operators/tap";
import {
  distinctUntilChanged,
  filter,
  map,
  take,
} from "rxjs/operators";
import {
  isFalse,
  isFalsy,
  isNotNullNorUndefined,
  isNullOrUndefined,
  isTrue,
  isUndefined,
  LoggingService,
  MARK_AS_TRANSLATABLE,
  NotificationService,
  ResourceState,
} from "solidify-frontend";

@Component({
  selector: "dlcm-root",
  templateUrl: "./app.component.html",
  styleUrls: ["./app.component.scss"],
})
export class AppComponent extends SharedAbstractPresentational {
  private readonly _ATTRIBUTE_SRC_DIFFER_KEY: string = "src-differ";
  private readonly _ATTRIBUTE_SRC: string = "src";
  private static readonly _themeAttributeName: string = "theme";
  currentModule: string;
  isHomePage: boolean;
  logo: string;

  @Select((state: LocalStateModel) => state.router.state) urlStateObs: Observable<RouterStateSnapshot>;
  @Select((state: LocalStateModel) => state.application.appLanguage) appLanguageObs: Observable<LanguagesEnum>;
  @Select((state: LocalStateModel) => state.application.isLoggedIn) isLoggedObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state.application.isApplicationInitialized) isApplicationInitializedObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state.application.theme) themeObs: Observable<ThemeEnum>;
  @Select((state: LocalStateModel) => state.application.userRoles) userRolesObs: Observable<ApplicationRoleEnum[]>;
  @Select((state: LocalStateModel) => state.application.application_user.current) currentUserObs: Observable<User>;

  numberArchiveInCartObs: Observable<number> = ResourceState.total(this.store, AppCartArchiveState);

  ignoreGrid: boolean = false;
  displayCart: boolean = false;

  @HostListener("window:beforeunload", ["$event"])
  leaveWithoutSaveNotification($event: Event): void {
    const preventExit = this.store.selectSnapshot((state: LocalStateModel) => state.application.preventExit);
    const ignorePreventExit = this.store.selectSnapshot((state: LocalStateModel) => state.application.ignorePreventLeavePopup);
    if (isFalse(ignorePreventExit) && isTrue(preventExit)) {
      $event.returnValue = true;
    }
  }

  get sessionStorageHelper(): typeof SessionStorageHelper {
    return SessionStorageHelper;
  }

  get sessionStorageEnum(): typeof SessionStorageEnum {
    return SessionStorageEnum;
  }

  get routesEnum(): typeof RoutesEnum {
    return RoutesEnum;
  }

  constructor(private store: Store,
              private router: Router,
              private translate: TranslateService,
              private _renderer: Renderer2,
              @Inject(DOCUMENT) private readonly _document: Document,
              private _notificationService: NotificationService,
              private _loggingService: LoggingService,
              public appStatusService: AppStatusService,
              public breakpointService: BreakpointService,
              private readonly _dialog: MatDialog) {
    super();
    this.store.dispatch(new AppAction.InitApplication());
    this._setOverrideCssStyleSheet();
    this._listenCurrentModule();
    this.subscribe(this._observeThemeChange());
    this.observeOfflineOnlineMode();
    this.themeChange(environment.theme);
    this.subscribe(this.observeUpdateAvailableForModal());
    this.subscribe(this.observeLoggedUserForCart());
    this.checkOrcidQueryParam();
    this._activeChemicalMoleculePreviewIfEnable();
    this.subscribe(this.listenLanguageChange());
    this.subscribe(this.listenAppReady());
  }

  private observeOfflineOnlineMode(): void {
    let previousStateIsOnline: boolean | undefined = undefined;
    this.subscribe(this.appStatusService.online.pipe(
      distinctUntilChanged(),
      tap((isOnline: boolean) => {
        if ((isUndefined(previousStateIsOnline) || isTrue(previousStateIsOnline)) && isFalsy(isOnline)) {
          this._notificationService.showWarning(MARK_AS_TRANSLATABLE("app.status.offline"));
          previousStateIsOnline = isOnline;
          return;
        }

        if (isFalse(previousStateIsOnline) && isTrue(isOnline)) {
          this._notificationService.showInformation(MARK_AS_TRANSLATABLE("app.status.backToOnline"), true);
          previousStateIsOnline = isOnline;
          return;
        }
        return;
      }),
    ));
  }

  private observeUpdateAvailableForModal(): Observable<string> {
    return this.appStatusService.updateAvailable.pipe(
      distinctUntilChanged(),
      filter(version => !isNullOrUndefined(version)),
      tap((version) => {
        this._dialog.open(UpdateVersionDialog);
      }),
    );
  }

  private observeLoggedUserForCart(): Observable<boolean> {
    return combineLatest([this.isLoggedObs, this.userRolesObs])
      .pipe(
        map(([isLogged, userRoles]) => this.computeDisplayCart(isLogged, userRoles)),
      );
  }

  private checkOrcidQueryParam(): void {
    if (UrlQueryParamHelper.contains(environment.orcidQueryParam)) {
      this.subscribe(this.currentUserObs.pipe(
        filter(user => isNotNullNorUndefined(user)),
        take(1),
        tap(user => {
          this._dialog.open(UserDialog, {
            data: user,
            width: "90%",
          });
          this._notificationService.showSuccess(MARK_AS_TRANSLATABLE("notification.orcid.associated"), {orcid: user?.person?.orcid});
        }),
      ));
    }
  }

  useLanguage(language: LanguagesEnum): void {
    this.store.dispatch(new AppAction.ChangeAppLanguage(language));
  }

  navigate(route: string): void {
    this.store.dispatch(new Navigate([route]));
  }

  private _setTitle(): void {
    setTimeout(() => {
      this._document.title = environment.appTitle;
      const description = this.translate.instant("app.meta.description");
      this._document.querySelector("meta[name=\"description\"]").setAttribute("content", description);
    }, 0);
  }

  private setTheme(): void {
    this._renderer.setAttribute(this._document.body, AppComponent._themeAttributeName, environment.theme);
  }

  private setFavicon(): void {
    const header = this._document.querySelector("head");
    const link = this._document.createElement("link");
    this._renderer.setAttribute(link, "type", "image/x-icon");
    this._renderer.setAttribute(link, "rel", "shortcut icon");
    this._renderer.setAttribute(link, "href", `./assets/themes/${environment.theme}/favicon.ico`);
    this._renderer.appendChild(header, link);
  }

  private setImageToolbar(): void {
    this.logo = `assets/themes/${environment.theme}/toolbar-header-image.svg`;
  }

  private _listenCurrentModule(): void {
    this.subscribe(this.urlStateObs
      .pipe(
        tap(urlState => {
          this.ignoreGrid = false;
          this.isHomePage = false;
          if (urlState) {
            const url = urlState.url;
            if (url.startsWith(urlSeparator + RoutesEnum.homePage)) {
              this.currentModule = RoutesEnum.homePage;
              this.isHomePage = url === (urlSeparator + RoutesEnum.homePage) || url.startsWith(urlSeparator + RoutesEnum.homePage + "?");
              this.ignoreGrid = true;
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.deposit)) {
              this.currentModule = RoutesEnum.deposit;
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.preservationSpace)) {
              this.currentModule = RoutesEnum.preservationSpace;
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.admin)) {
              this.currentModule = RoutesEnum.admin;
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.preservationPlanning)) {
              this.currentModule = RoutesEnum.preservationPlanning;
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.order)) {
              this.currentModule = RoutesEnum.order;
              return;
            }

            this.currentModule = null;
          }
        }),
      ),
    );
  }

  logout(): void {
    this.store.dispatch(new AppAction.Logout());
  }

  getTitle(): string {
    return this._document.title;
  }

  themeChange(theme: ThemeEnum): void {
    if (theme !== this.store.selectSnapshot((state: LocalStateModel) => state.application.theme)) {
      this.store.dispatch(new AppAction.ChangeAppTheme(theme));
    }
  }

  navigateToHome(): void {
    this.navigate(RoutesEnum.homePage);
  }

  private _observeThemeChange(): Observable<ThemeEnum> {
    return this.themeObs.pipe(
      distinctUntilChanged(),
      tap((theme: ThemeEnum) => {
        this.setTheme();
        this.setFavicon();
        this.setImageToolbar();
      }),
    );
  }

  private listenLanguageChange(): Observable<LanguagesEnum> {
    return this.appLanguageObs.pipe(
      tap((language: LanguagesEnum) => {
        this._setTitle();
      }),
    );
  }

  private listenAppReady(): Observable<boolean> {
    return this.isApplicationInitializedObs.pipe(
      tap((isInit: boolean) => {
        this._setTitle();
      }),
    );
  }

  openCart(): void {
    this._dialog.open(CartDialog, {
      width: "400px",
    });
  }

  private _setOverrideCssStyleSheet(): void {
    const header = this._document.querySelector("head");
    const link = this._document.createElement("link");
    this._renderer.setAttribute(link, "type", "text/css");
    this._renderer.setAttribute(link, "rel", "stylesheet");
    this._renderer.setAttribute(link, "href", `./assets/styles/override.css`);
    this._renderer.appendChild(header, link);
  }

  computeDisplayCart(isLogged: boolean, userRoles: ApplicationRoleEnum[]): boolean {
    this.displayCart = PermissionUtil.isUserHavePermission(isLogged, ApplicationRolePermissionEnum.userPermission, userRoles);
    return this.displayCart;
  }

  search(searchTerm: string): void {
    HomeHelper.navigateToSearch(this.store, searchTerm);
  }

  private _activeChemicalMoleculePreviewIfEnable(): void {
    if (environment.visualizationChemicalMoleculeMode === ChemicalMoleculeVisualizationEnum.disabled) {
      return;
    }
    const threeDimensionalLib = environment.visualizationChemicalMolecule3dLib;
    const twoDimensionalLibs = environment.visualizationChemicalMolecule2dLibs;
    const listScript = [];
    this._document.querySelectorAll(`script[${this._ATTRIBUTE_SRC_DIFFER_KEY}]`).forEach(script => {
      listScript.push(script);
    });

    if (environment.visualizationChemicalMoleculeMode === ChemicalMoleculeVisualizationEnum.threeDimensionalOnly
      || environment.visualizationChemicalMoleculeMode === ChemicalMoleculeVisualizationEnum.threeAndTwoDimensional) {
      const jsmolScript = this._getScriptWithValueEndWith(listScript, threeDimensionalLib);
      if (isNullOrUndefined(jsmolScript)) {
        this._loggingService.logWarning(`Unable to enable chemical molecule visualization, missing ${threeDimensionalLib}`);
        return;
      }
      this._activeScriptDiffer([jsmolScript]);
    }

    if (environment.visualizationChemicalMoleculeMode === ChemicalMoleculeVisualizationEnum.threeAndTwoDimensional) {
      const listScriptToTreat = [];
      twoDimensionalLibs.forEach(libNameNeeded => {
        const scriptLibNeeded = this._getScriptWithValueEndWith(listScript, libNameNeeded);
        if (isNullOrUndefined(scriptLibNeeded)) {
          this._loggingService.logWarning(`Unable to enable chemical molecule visualization 2D, missing ${libNameNeeded}`);
          return;
        }
        listScriptToTreat.push(scriptLibNeeded);
      });
      if (listScriptToTreat.length !== twoDimensionalLibs.length) {
        this._loggingService.logWarning(`Swith to chemical molecule visualization 3D`);
        environment.visualizationChemicalMoleculeMode = ChemicalMoleculeVisualizationEnum.threeDimensionalOnly;
        return;
      }
      // Need to differ lib JSME to avoid error at startup
      setTimeout(() => {
        this._activeScriptDiffer(listScriptToTreat);
      }, 1000);
    }
  }

  private _activeScriptDiffer(listScript: Element[]): void {
    listScript.forEach(script => {
      const value = this._getValueScriptDiffer(script);
      if (isNullOrUndefined(value)) {
        return;
      }
      this._renderer.setAttribute(script, this._ATTRIBUTE_SRC, value);
      this._renderer.removeAttribute(script, this._ATTRIBUTE_SRC_DIFFER_KEY);
    });
  }

  private _getValueScriptDiffer(script: Element): string | undefined {
    if (isNullOrUndefined(script) || isNullOrUndefined(script.attributes)) {
      return undefined;
    }
    const attribute = script.attributes[this._ATTRIBUTE_SRC_DIFFER_KEY];
    return isNullOrUndefined(attribute) ? undefined : attribute.value;
  }

  private _getScriptWithValueEndWith(listScript: Element[], threeDimensionalLib: string): Element | undefined {
    return listScript.find(script => {
      const value = this._getValueScriptDiffer(script);
      return isNotNullNorUndefined(value) && value.endsWith(threeDimensionalLib);
    });
  }
}
