import {
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {
  MatDialog,
  MatDialogRef,
} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {DepositCollectionAction} from "@deposit/stores/collection/deposit-collection.action";
import {DepositCollectionState} from "@deposit/stores/collection/deposit-collection.state";
import {DepositCollectionStatusHistoryAction} from "@deposit/stores/collection/status-history/deposit-collection-status-history.action";
import {DepositCollectionStatusHistoryState} from "@deposit/stores/collection/status-history/deposit-collection-status-history.state";
import {depositActionNameSpace} from "@deposit/stores/deposit.action";
import {
  DepositState,
  DepositStateModel,
} from "@deposit/stores/deposit.state";
import {environment} from "@environments/environment";
import {
  Aip,
  Deposit,
} from "@models";
import {
  Actions,
  ofActionCompleted,
  Store,
} from "@ngxs/store";
import {
  SharedAipDetailDialog,
  SharedAipDetailDialogData,
} from "@shared/components/dialogs/shared-aip-detail/shared-aip-detail.dialog";
import {SharedAbstractDetailEditRoutable} from "@shared/components/routables/shared-abstract-detail-edit/shared-abstract-detail-edit.routable";
import {ComplianceLevelEnumHelper} from "@shared/enums/business/compliance-level.enum";
import {PackageStatusEnumHelper} from "@shared/enums/business/package-status.enum";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {AppRoutesEnum} from "@shared/enums/routes.enum";
import {DataTableActions} from "@shared/models/data-table-actions.model";
import {DataTableColumns} from "@shared/models/data-table-columns.model";
import {StatusHistory} from "@shared/models/status-history.model";
import {SecurityService} from "@shared/services/security.service";
import {Observable} from "rxjs";
import {tap} from "rxjs/operators";
import {
  AssociationRemoteState,
  isNullOrUndefined,
  MARK_AS_TRANSLATABLE,
  MemoizedUtil,
  OrderEnum,
  QueryParameters,
  ResourceActionHelper,
  ResourceState,
} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-collection-container",
  templateUrl: "./deposit-collection.container.html",
  styleUrls: ["./deposit-collection.container.scss"],
})
export class DepositCollectionContainer extends SharedAbstractDetailEditRoutable<Deposit, DepositStateModel> implements OnInit {
  readonly KEY_PARAM_NAME: keyof Deposit & string = undefined;

  isLoadingCollectionObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, DepositCollectionState);
  listCollectionObs: Observable<Aip[]> = AssociationRemoteState.selected(this._store, DepositCollectionState);
  queryParametersObs: Observable<QueryParameters> = AssociationRemoteState.queryParameters(this._store, DepositCollectionState);

  canEditObs: Observable<boolean> = MemoizedUtil.select(this._store, DepositState, state => state.canEdit);

  dialogCollectionFileRef: MatDialogRef<SharedAipDetailDialog<Aip>>;

  isLoadingHistoryObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, DepositCollectionStatusHistoryState);
  collectionHistoryObs: Observable<StatusHistory[]> = MemoizedUtil.select(this._store, DepositCollectionStatusHistoryState, state => state.history);
  queryParametersHistoryObs: Observable<QueryParameters> = MemoizedUtil.select(this._store, DepositCollectionStatusHistoryState, state => state.queryParameters);

  columns: DataTableColumns<Aip>[];
  actions: DataTableActions<Aip>[] = [
    {
      logo: IconNameEnum.delete,
      callback: (aip: Aip) => this.deleteAip(this._resId, aip),
      placeholder: MARK_AS_TRANSLATABLE("crud.list.action.delete"),
      displayOnCondition: (aip: Aip) => MemoizedUtil.selectSnapshot(this._store, DepositState, state => state.canEdit),
    },
    {
      logo: IconNameEnum.navigate,
      callback: (aip: Aip) => this.goToAip(aip),
      placeholder: MARK_AS_TRANSLATABLE("crud.list.action.goToAip"),
      displayOnCondition: (aip: Aip) => this._securityService.isRootOrAdmin(),
    },
  ];

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _dialog: MatDialog,
              protected readonly _securityService: SecurityService) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.deposit, depositActionNameSpace);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.getCurrentModelOnParent();

    this.columns = [
      {
        field: "info.name" as any,
        header: MARK_AS_TRANSLATABLE("deposit.aip.table.aipName"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isSortable: true,
        isFilterable: false,
      },
      {
        field: "info.status" as any,
        header: MARK_AS_TRANSLATABLE("deposit.aip.table.status"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isSortable: false,
        isFilterable: false,
        translate: true,
        filterEnum: PackageStatusEnumHelper.getListKeyValue(),
      },
      {
        field: "info.complianceLevel" as any,
        header: MARK_AS_TRANSLATABLE("deposit.aip.table.complianceLevel"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isSortable: false,
        isFilterable: false,
        translate: true,
        component: DataTableComponentEnum.conformityLevelStar,
        filterEnum: ComplianceLevelEnumHelper.getListKeyValue(),
      },
    ];
  }

  private getCurrentModelOnParent(): void {
    this._resId = this._route.snapshot.parent.paramMap.get(AppRoutesEnum.paramIdWithoutPrefixParam);
    this.getDepositById(this._resId);
  }

  private getDepositById(id: string): void {
    const depositInState = ResourceState.currentSnapshot(this._store, DepositState);
    if (isNullOrUndefined(depositInState) || id !== depositInState.resId) {
      this._store.dispatch(ResourceActionHelper.getById(depositActionNameSpace, id, true));
    }
    this.getSubResourceWithParentId(id);
  }

  getSubResourceWithParentId(id: string): void {
    this._store.dispatch(new DepositCollectionAction.GetAll(id, null, true));
  }

  refresh(): void {
    this._store.dispatch(new DepositCollectionAction.Refresh(this._resId));
  }

  onQueryParametersEvent(queryParameters: QueryParameters): void {
    this._store.dispatch(new DepositCollectionAction.ChangeQueryParameters(this._resId, queryParameters, true));
  }

  showDetail(aip: Aip): void {
    this.dialogCollectionFileRef = this._dialog.open(SharedAipDetailDialog, {
      data: {
        aip: aip,
        resId: aip.resId,
        parentId: this._resId,
        isReadonly: !this.isEdit,
        isLoading: this.isLoadingHistoryObs,
        statusHistory: this.collectionHistoryObs,
        queryParametersObs: this.queryParametersHistoryObs,
        state: DepositCollectionStatusHistoryAction,
        name: aip.info.name,
        buttons: [
          {
            labelToTranslate: MARK_AS_TRANSLATABLE("crud.list.action.goToAip"),
            icon: IconNameEnum.navigate,
            color: "primary",
            callback: current => this.goToAip(current),
            displayCondition: current => this._securityService.isRootOrAdmin(),
            order: 0,
          },
          {
            labelToTranslate: MARK_AS_TRANSLATABLE("crud.list.action.delete"),
            icon: IconNameEnum.delete,
            color: "primary",
            callback: current => this.deleteAip(this._resId, current),
            displayCondition: current => this.canEditObs,
            order: 1,
          },
        ],
      } as SharedAipDetailDialogData<Aip>,
      width: environment.modalWidth,
    });
  }

  private goToAip(aip: Aip): void {
    this.subscribe(this._actions$.pipe(
      ofActionCompleted(DepositCollectionAction.GoToAip),
      tap(result => {
        if (!result.result.successful) {
          return;
        }
        if (this.dialogCollectionFileRef) {
          this.dialogCollectionFileRef.close();
        }
      }),
    ));
    this._store.dispatch(new DepositCollectionAction.GoToAip(aip));
  }

  deleteAip(_resId: string, aip: Aip): void {
    this.subscribe(this._actions$.pipe(
      ofActionCompleted(DepositCollectionAction.DeleteSuccess),
      tap((result) => {
        if (!result.result.successful) {
          return;
        }
        if (this.dialogCollectionFileRef) {
          this.dialogCollectionFileRef.close();
        }
        this._store.dispatch(new DepositCollectionAction.Refresh(this._resId));
      }),
    ));
    this._store.dispatch(new DepositCollectionAction.Delete(this._resId, aip.resId));
  }
}
