import {PersonRole} from "@admin/models/person-role.model";
import {
  ChangeDetectorRef,
  Component,
  Inject,
  OnInit,
  ViewChild,
} from "@angular/core";
import {
  MatDialog,
  MatDialogRef,
} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {AppState} from "@app/stores/app.state";
import {DepositDeleteFolderDialog} from "@deposit/components/dialogs/deposit-delete-folder/deposit-delete-folder.dialog";
import {
  DepositFileMoveDialog,
  DepositFileMoveDialogData,
} from "@deposit/components/dialogs/deposit-file-move/deposit-file-move.dialog";
import {DepositDataFileHelper} from "@deposit/helpers/deposit-data-file.helper";
import {DepositDataFile} from "@deposit/models/deposit-data-file.model";
import {DepositDataFileAction} from "@deposit/stores/data-file/deposit-data-file.action";
import {DepositDataFileState} from "@deposit/stores/data-file/deposit-data-file.state";
import {DepositDataFileStatusHistoryAction} from "@deposit/stores/data-file/status-history/deposit-data-file-status-history.action";
import {DepositDataFileStatusHistoryState} from "@deposit/stores/data-file/status-history/deposit-data-file-status-history.state";
import {
  DepositAction,
  depositActionNameSpace,
} from "@deposit/stores/deposit.action";
import {
  DepositState,
  DepositStateModel,
} from "@deposit/stores/deposit.state";
import {environment} from "@environments/environment";
import {Deposit} from "@models";
import {
  Actions,
  ofActionCompleted,
  Select,
  Store,
} from "@ngxs/store";
import {
  SharedConfirmDialog,
  SharedConfirmDialogData,
} from "@shared/components/dialogs/shared-confirm/shared-confirm.dialog";
import {
  SharedFileDetailDialog,
  SharedFileDetailDialogData,
} from "@shared/components/dialogs/shared-file-detail/shared-file-detail.dialog";
import {SharedHistoryDialog} from "@shared/components/dialogs/shared-history/shared-history.dialog";
import {SharedPreviewDialogData} from "@shared/components/dialogs/shared-preview/shared-preview-dialog-data.model";
import {SharedPreviewDialog} from "@shared/components/dialogs/shared-preview/shared-preview.dialog";
import {SharedDataTablePresentational} from "@shared/components/presentationals/shared-data-table/shared-data-table.presentational";
import {SharedAbstractDetailEditRoutable} from "@shared/components/routables/shared-abstract-detail-edit/shared-abstract-detail-edit.routable";
import {ComplianceLevelEnumHelper} from "@shared/enums/business/compliance-level.enum";
import {
  DataFileStatusEnum,
  DataFileStatusEnumHelper,
} from "@shared/enums/business/data-file-status.enum";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {FileViewModeEnum} from "@shared/enums/file-view-mode.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {AppRoutesEnum} from "@shared/enums/routes.enum";
import {FileVisualizerHelper} from "@shared/filevisualizer/helpers/file-visualizer.helper";
import {
  AbstractFileVisualizer,
  FILE_VISUALIZERS,
} from "@shared/filevisualizer/services/abstract-file-visualizer.service";
import {DataTableActions} from "@shared/models/data-table-actions.model";
import {DataTableBulkActions} from "@shared/models/data-table-bulk-actions.model";
import {DataTableColumns} from "@shared/models/data-table-columns.model";
import {DeleteDialog} from "@shared/models/delete-dialog.model";
import {StatusHistoryDialog} from "@shared/models/status-history-dialog.model";
import {StatusHistory} from "@shared/models/status-history.model";
import {SecurityService} from "@shared/services/security.service";
import {Observable} from "rxjs";
import {
  distinctUntilChanged,
  take,
  tap,
} from "rxjs/operators";
import {
  CompositionState,
  isNullOrUndefined,
  isTrue,
  isUndefined,
  MARK_AS_TRANSLATABLE,
  MemoizedUtil,
  OrderEnum,
  QueryParameters,
  ResourceActionHelper,
  ResourceState,
  StringUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-file-container",
  templateUrl: "./deposit-file.container.html",
  styleUrls: ["./deposit-file.container.scss"],
})
export class DepositFileContainer extends SharedAbstractDetailEditRoutable<Deposit, DepositStateModel> implements OnInit {
  isLoadingDataFileObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, DepositDataFileState);
  listDataFileObs: Observable<DepositDataFile[]> = CompositionState.list(this._store, DepositDataFileState);
  queryParametersObs: Observable<QueryParameters> = CompositionState.queryParameters(this._store, DepositDataFileState);

  foldersObs: Observable<string[]> = MemoizedUtil.select(this._store, DepositDataFileState, state => state.folders);
  intermediateFoldersObs: Observable<string[]> = MemoizedUtil.select(this._store, DepositDataFileState, state => state.intermediateFolders);
  foldersWithIntermediateFoldersObs: Observable<string[]> = MemoizedUtil.select(this._store, DepositDataFileState, state => state.foldersWithIntermediateFolders);
  currentFolderObs: Observable<string> = MemoizedUtil.select(this._store, DepositDataFileState, state => state.currentFolder);
  canEditObs: Observable<boolean> = MemoizedUtil.select(this._store, DepositState, state => state.canEdit).pipe(
    distinctUntilChanged(),
    tap(canEdit => this.actions = [...this.actions]), // Force method computeContext on datatable
  );

  isLoadingHistoryObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, DepositDataFileStatusHistoryState);
  dataFileHistoryObs: Observable<StatusHistory[]> = MemoizedUtil.select(this._store, DepositDataFileStatusHistoryState, state => state.history);
  queryParametersHistoryObs: Observable<QueryParameters> = MemoizedUtil.select(this._store, DepositDataFileStatusHistoryState, state => state.queryParameters);

  @Select(AppState.currentOrgUnitPerson) currentPersonObs: Observable<PersonRole>;

  @ViewChild("dataTablePresentational")
  readonly dataTablePresentational: SharedDataTablePresentational<DepositDataFile>;

  currentFileViewMode: FileViewModeEnum = FileViewModeEnum.FolderView;

  columns: DataTableColumns<DepositDataFile>[];

  actions: DataTableActions<DepositDataFile>[] = [
    {
      logo: IconNameEnum.notIgnore,
      callback: (depositDataFile: DepositDataFile) => this.validate(this._resId, depositDataFile),
      placeholder: MARK_AS_TRANSLATABLE("crud.list.action.validate"),
      displayOnCondition: (depositDataFile: DepositDataFile) => depositDataFile.status === DataFileStatusEnum.IGNORED_FILE,
    },
    {
      logo: IconNameEnum.download,
      callback: (depositDataFile: DepositDataFile) => this.downloadDataFile(this._resId, depositDataFile),
      placeholder: MARK_AS_TRANSLATABLE("crud.list.action.download"),
      displayOnCondition: (depositDataFile: DepositDataFile) => true,
    },
    {
      logo: IconNameEnum.resume,
      callback: (depositDataFile: DepositDataFile) => this.resumeDataFile(this._resId, depositDataFile),
      placeholder: MARK_AS_TRANSLATABLE("deposit.file.button.resume"),
      displayOnCondition: (depositDataFile: DepositDataFile) => depositDataFile.status === DataFileStatusEnum.IN_ERROR,
    },
    {
      logo: IconNameEnum.delete,
      callback: (depositDataFile: DepositDataFile) => this.deleteDataFile(this._resId, depositDataFile),
      placeholder: MARK_AS_TRANSLATABLE("crud.list.action.delete"),
      displayOnCondition: (depositDataFile: DepositDataFile) => MemoizedUtil.selectSnapshot(this._store, DepositState, state => state.canEdit),
    },
    {
      logo: IconNameEnum.preview,
      callback: (depositDataFile: DepositDataFile) => this.showPreview(depositDataFile),
      placeholder: MARK_AS_TRANSLATABLE("deposit.file.button.showPreview"),
      displayOnCondition: (depositDataFile: DepositDataFile) => FileVisualizerHelper.canHandle(this._fileVisualizers, {
        dataFile: depositDataFile,
        fileExtension: FileVisualizerHelper.getFileExtension(depositDataFile.fileName),
      }),
    },
    {
      logo: IconNameEnum.move,
      callback: (depositDataFile: DepositDataFile) => this.moveDataFileWithDialog(depositDataFile),
      placeholder: MARK_AS_TRANSLATABLE("crud.list.action.move"),
      displayOnCondition: (depositDataFile: DepositDataFile) => MemoizedUtil.selectSnapshot(this._store, DepositState, state => state.canEdit),
    },
    {
      logo: IconNameEnum.history,
      callback: (depositDataFile: DepositDataFile) => this.historyDataFile(this._resId, depositDataFile),
      placeholder: MARK_AS_TRANSLATABLE("crud.list.action.history"),
    },
  ];

  bulkActions: DataTableBulkActions[] = [
    {
      logo: IconNameEnum.delete,
      callback: (listId: string[]) => this.bulkDeleteDataFile(this._resId, listId),
      label: MARK_AS_TRANSLATABLE("crud.list.bulkAction.deleteSelection"),
      displayOnCondition: (listId: string[]) => this.canEditObs,
    },
  ];

  columnsToSkippedFilter: keyof DepositDataFile[] | string[] = [
    DepositDataFileHelper.RELATIVE_LOCATION,
    "status",
  ];

  dialogDataFileRef: MatDialogRef<SharedFileDetailDialog<DepositDataFile>>;

  get fileViewModeEnum(): typeof FileViewModeEnum {
    return FileViewModeEnum;
  }

  readonly KEY_PARAM_NAME: keyof Deposit & string = undefined;

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _dialog: MatDialog,
              protected readonly _securityService: SecurityService,
              @Inject(FILE_VISUALIZERS) private readonly _fileVisualizers: AbstractFileVisualizer[]) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.deposit, depositActionNameSpace);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.getCurrentModelOnParent();

    this.columns = [
      {
        field: "fileName",
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.fileName"),
        type: FieldTypeEnum.string,
        order: OrderEnum.ascending,
        filterableField: "dataFile.finalData" as any,
        sortableField: "dataFile.finalData" as any,
        isSortable: true,
        isFilterable: true,
      },
      {
        field: "creation.when" as any,
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.creation.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
      },
      {
        field: "status",
        header: StringUtil.stringEmpty,
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        sortableField: "dataFile.status" as any,
        filterableField: "dataFile.status" as any,
        isSortable: false,
        isFilterable: false,
        translate: false,
        component: DataTableComponentEnum.dataFileQuickStatus,
        width: "35px",
        // filterEnum: ComplianceLevelEnumHelper.getListKeyValue(),
      },
      {
        field: "status",
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.status"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        sortableField: "dataFile.status" as any,
        isSortable: true,
        isFilterable: true,
        translate: true,
        tooltip: (value) => DataFileStatusEnumHelper.getExplanation(value as DataFileStatusEnum),
        filterEnum: DataFileStatusEnumHelper.getListKeyValue(),
      },
      {
        field: "complianceLevel",
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.complianceLevel"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        sortableField: "dataFile.complianceLevel" as any,
        filterableField: "dataFile.complianceLevel" as any,
        isSortable: true,
        isFilterable: false,
        translate: true,
        component: DataTableComponentEnum.conformityLevelStar,
        filterEnum: ComplianceLevelEnumHelper.getListKeyValue(),
      },
      {
        field: "fileSize",
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.size"),
        type: FieldTypeEnum.size,
        order: OrderEnum.none,
        sortableField: "dataFile.fileSize" as any,
        filterableField: "dataFile.fileSize" as any,
        isSortable: true,
        isFilterable: false,
        translate: true,
      },
    ];
  }

  private getCurrentModelOnParent(): void {
    this._resId = this._route.snapshot.parent.paramMap.get(AppRoutesEnum.paramIdWithoutPrefixParam);
    this.getDepositById(this._resId);
  }

  private getDepositById(id: string): void {
    const depositInState = ResourceState.currentSnapshot(this._store, DepositState);
    if (isNullOrUndefined(depositInState) || id !== depositInState.resId) {
      this._store.dispatch(ResourceActionHelper.getById(depositActionNameSpace, id, true));
    }
    this.getSubResourceWithParentId(id);
  }

  getSubResourceWithParentId(id: string): void {
    this._store.dispatch(new DepositDataFileAction.GetAll(id, undefined, true));
    this._store.dispatch(new DepositDataFileAction.GetListFolder(id));
  }

  onQueryParametersEvent(queryParameters: QueryParameters, withRefresh: boolean = true): void {
    this._store.dispatch(new DepositDataFileAction.ChangeQueryParameters(this._resId, queryParameters, true, withRefresh));
    // this._changeDetector.detectChanges(); // Allow to display spinner the first time
  }

  refresh(): void {
    this._store.dispatch(new DepositDataFileAction.Refresh(this._resId));
  }

  download($event: DepositDataFile): void {
    this._store.dispatch(new DepositDataFileAction.Download(this._resId, $event));
  }

  selectFolder(folderFullName: string | undefined): void {
    if (isUndefined(folderFullName)) {
      this.currentFileViewMode = FileViewModeEnum.FlatView;
    } else {
      this.currentFileViewMode = FileViewModeEnum.FolderView;
    }
    this._store.dispatch(new DepositDataFileAction.ChangeCurrentFolder(folderFullName, true));
  }

  downloadFolder(folderName: string): void {
    this._store.dispatch(new DepositAction.Download(this._resId, folderName));
  }

  deleteFolder(folderName: string): void {
    this.subscribe(this._dialog.open(DepositDeleteFolderDialog, {
      minWidth: "500px",
      data: {
        name: folderName,
      } as DeleteDialog,
    }).afterClosed().pipe(
      tap((isConfirmed: boolean | undefined) => {
        if (isNullOrUndefined(isConfirmed)) {
          return;
        }
        if (isTrue(isConfirmed)) {
          this._store.dispatch(new DepositDataFileAction.DeleteFolder(this._resId, folderName));
        }
      }),
    ));
  }

  showDetail(depositDataFile: DepositDataFile): void {
    this.dialogDataFileRef = this._dialog.open(SharedFileDetailDialog, {
      data: {
        dataFile: depositDataFile,
        resId: depositDataFile.resId,
        parentId: this._resId,
        isReadonly: !this.isEdit,
        isLoading: this.isLoadingHistoryObs,
        statusHistory: this.dataFileHistoryObs,
        queryParametersObs: this.queryParametersHistoryObs,
        state: DepositDataFileStatusHistoryAction,
        name: depositDataFile.fileName,
        buttons: [
          {
            labelToTranslate: MARK_AS_TRANSLATABLE("deposit.file.button.validate"),
            color: "primary",
            icon: IconNameEnum.notIgnore,
            callback: current => this.validate(this._resId, current),
            displayCondition: current => current.status === DataFileStatusEnum.IGNORED_FILE,
            order: 0,
          },
          {
            labelToTranslate: MARK_AS_TRANSLATABLE("deposit.file.button.download"),
            color: "primary",
            icon: IconNameEnum.download,
            callback: current => this.downloadDataFile(this._resId, current),
            order: 1,
          },
          {
            labelToTranslate: MARK_AS_TRANSLATABLE("deposit.file.button.delete"),
            color: "primary",
            icon: IconNameEnum.delete,
            callback: (current) => this.deleteDataFile(this._resId, current),
            displayCondition: current => this.canEditObs,
            order: 2,
          },
          {
            labelToTranslate: MARK_AS_TRANSLATABLE("deposit.file.button.resume"),
            color: "primary",
            icon: IconNameEnum.resume,
            callback: current => this.resumeDataFile(this._resId, current),
            displayCondition: current => current.status === DataFileStatusEnum.IN_ERROR,
            order: 3,
          },
          {
            labelToTranslate: MARK_AS_TRANSLATABLE("deposit.file.button.move"),
            color: "primary",
            icon: IconNameEnum.move,
            callback: current => this.moveDataFileWithDialog(current),
            displayCondition: current => this.canEditObs,
            order: 4,
          },
        ],
      } as SharedFileDetailDialogData<DepositDataFile>,
      width: environment.modalWidth,
    });
  }

  bulkDeleteDataFile(parentId: string, listId: string[]): void {
    this._store.dispatch(new DepositDataFileAction.DeleteAll(parentId, listId));
  }

  deleteDataFile(parentId: string, dataFile: DepositDataFile): void {
    this._store.dispatch(new DepositDataFileAction.Delete(parentId, dataFile));
    this.subscribe(this._actions$.pipe(
      ofActionCompleted(DepositDataFileAction.DeleteSuccess),
      tap(result => {
        if (!result.result.successful) {
          return;
        }
        if (this.dialogDataFileRef) {
          this.dialogDataFileRef.close();
        }
      }),
    ));
  }

  moveDataFileWithDialog(dataFile: DepositDataFile): void {
    this._dialog.open(DepositFileMoveDialog, {
      data: {
        parentResId: this._resId,
        depositDataFile: dataFile,
      } as DepositFileMoveDialogData,
    });
    if (!isNullOrUndefined(this.dialogDataFileRef)) {
      this.subscribe(this._actions$.pipe(
        ofActionCompleted(DepositDataFileAction.MoveSuccess),
        take(1),
        tap((result) => {
          if (isTrue(result.result.successful)) {
            this.dialogDataFileRef.close();
          }
        }),
      ));
    }
  }

  showPreview(dataFile: DepositDataFile): void {
    const previewDialogRef = this._dialog.open(SharedPreviewDialog, {
      width: "min-content",
      height: "min-content",
      data: {
        fileInput: {
          dataFile: dataFile,
        },

      } as SharedPreviewDialogData,
    });
  }

  moveDataFile(dataFile: DepositDataFile): void {
    this._store.dispatch(new DepositDataFileAction.Move(this._resId, {
      resId: dataFile.resId,
      relativeLocation: dataFile.relativeLocation,
    }));
  }

  validate(parentId: string, dataFile: DepositDataFile): void {
    this._store.dispatch(new DepositDataFileAction.Validate(parentId, dataFile));
  }

  downloadDataFile(parentId: string, dataFile: DepositDataFile): void {
    this._store.dispatch(new DepositDataFileAction.Download(parentId, dataFile));
  }

  resumeDataFile(parentId: string, dataFile: DepositDataFile): void {
    this._store.dispatch(new DepositDataFileAction.ResumeDataFile(parentId, dataFile));
  }

  historyDataFile(parentId: string, dataFile: DepositDataFile): void {
    const dialogRef = this._dialog.open(SharedHistoryDialog, {
      width: environment.modalWidth,
      data: {
        parentId: parentId,
        resourceResId: dataFile.resId,
        name: LocalStateEnum.deposit,
        statusHistory: this.dataFileHistoryObs,
        isLoading: this.isLoadingHistoryObs,
        queryParametersObs: this.queryParametersHistoryObs,
        state: DepositDataFileStatusHistoryAction,
      } as StatusHistoryDialog,
    });
  }

  selectAndDelete(): void {
    this.subscribe(this._dialog.open(SharedConfirmDialog, {
      data: {
        titleToTranslate: MARK_AS_TRANSLATABLE("deposit.file.bulkDelete.title"),
        messageToTranslate: MARK_AS_TRANSLATABLE("deposit.file.bulkDelete.message"),
        confirmButtonToTranslate: MARK_AS_TRANSLATABLE("deposit.file.bulkDelete.button.confirm"),
        cancelButtonToTranslate: MARK_AS_TRANSLATABLE("deposit.file.bulkDelete.button.cancel"),
      } as SharedConfirmDialogData,
    }).afterClosed().pipe(
      tap((isConfirmed: boolean | undefined) => {
        if (isNullOrUndefined(isConfirmed)) {
          return;
        }
        if (isTrue(isConfirmed)) {
          this.subscribe(this.dataTablePresentational.selectAllResult().pipe(
            take(1),
            tap(selection => {
              this.bulkDeleteDataFile(this._resId, selection);
              this.dataTablePresentational.cleanSelection();
            }),
          ));
        }
      }),
    ));
  }

  get depositEnum(): typeof Deposit.StatusEnum {
    return Deposit.StatusEnum;
  }
}
