import {PersonExtended} from "@admin/models/person-extended.model";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Input,
  OnInit,
  ViewChild,
} from "@angular/core";
import {
  FormBuilder,
  FormGroup,
  Validators,
} from "@angular/forms";
import {MatDialog} from "@angular/material/dialog";
import {SharedAbstractFormPresentational} from "@app/shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {LocalModelAttributeEnum} from "@app/shared/enums/model-attribute.enum";
import {BaseFormDefinition} from "@app/shared/models/base-form-definition.model";
import {sharedLicenseActionNameSpace} from "@app/shared/stores/license/shared-license.action";
import {sharedPersonActionNameSpace} from "@app/shared/stores/person/shared-person.action";
import {
  DepositOrderAuthorDialog,
  DepositOrderAuthorDialogWrapper,
} from "@deposit/components/dialogs/deposit-order-author/deposit-order-author.dialog";
import {DepositPersonDialog} from "@deposit/components/dialogs/deposit-person/deposit-person.dialog";
import {environment} from "@environments/environment";
import {
  AdditionalFieldsForm,
  Deposit,
  Language,
  License,
  OrganizationalUnit,
  Person,
  PreservationPolicy,
  SubmissionPolicy,
} from "@models";
import {FormlyFieldConfig} from "@ngx-formly/core";
import {TranslateService} from "@ngx-translate/core";
import {SharedSearchableMultiSelectPresentational} from "@shared/components/presentationals/shared-searchable-multi-select/shared-searchable-multi-select.presentational";
import {ApplicationRoleEnum} from "@shared/enums/application-role.enum";
import {AccessLevelEnumHelper} from "@shared/enums/business/access-level-enum.helper";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {BreakpointService} from "@shared/services/breakpoint.service";
import {SharedLicenseState} from "@shared/stores/license/shared-license.state";
import {
  SharedPersonState,
  SharedPersonStateModel,
} from "@shared/stores/person/shared-person.state";
import _ from "lodash";
import {
  distinctUntilChanged,
  tap,
} from "rxjs/operators";
import {
  DateUtil,
  isEmptyArray,
  isNotNullNorUndefined,
  isNullOrUndefined,
  KeyValue,
  MappingObjectUtil,
  PropertyName,
  ResourceNameSpace,
  SolidifyValidator,
  urlSeparator,
} from "solidify-frontend";
import AccessEnum = Deposit.AccessEnum;
import {EmbargoAccessLevelEnumHelper} from "@app/shared/enums/business/embargo-access-level-enum.helper";
import {AipInfoEmbargo} from "@app/generated-api/model/aip-info-embargo.partial.model";

@Component({
  selector: "dlcm-deposit-form",
  templateUrl: "./deposit-form.presentational.html",
  styleUrls: ["./deposit-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositFormPresentational extends SharedAbstractFormPresentational<Deposit> implements OnInit {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  accessEnumValues: KeyValue[] = AccessLevelEnumHelper.getListKeyValue();
  embargoEnumValues: KeyValue[] = [];
  isRequiredEmbargo: boolean = false;

  additionalFieldsValues: Object;

  formFormly: FormGroup;

  addFieldsForm: AdditionalFieldsForm;

  get embargoEnum(): typeof AccessEnum {
    return AccessEnum;
  }

  @Input()
  set additionalFieldsForm(additionalFieldsForm: AdditionalFieldsForm) {
    if (isNotNullNorUndefined(additionalFieldsForm)) {
      this.addFieldsForm = additionalFieldsForm;
      this.fields = JSON.parse(additionalFieldsForm.description);
    }
  }

  fields: FormlyFieldConfig[];

  @Input()
  isReady: boolean = false;

  @Input()
  languages: Language[];

  @Input()
  listSubmissionPolicies: SubmissionPolicy[];

  @Input()
  listPreservationPolicies: PreservationPolicy[];

  @Input()
  selectedPersons: PersonExtended[];

  @Input()
  listPersons: PersonExtended[];

  @Input()
  personConnected: PersonExtended;

  @Input()
  currentApplicationRole: ApplicationRoleEnum[];

  @Input()
  orgUnit: OrganizationalUnit;

  @ViewChild("authorMultiSearchableSelect")
  authorMultiSearchableSelect: SharedSearchableMultiSelectPresentational<SharedPersonStateModel, Person>;

  readonly TIME_BEFORE_DISPLAY_TOOLTIP: number = environment.timeBeforeDisplayTooltipOnInput;

  get applicationRoleEnum(): typeof ApplicationRoleEnum {
    return ApplicationRoleEnum;
  }

  sharedPersonActionNameSpace: ResourceNameSpace = sharedPersonActionNameSpace;
  sharedPersonState: typeof SharedPersonState = SharedPersonState;

  sharedLicenseActionNameSpace: ResourceNameSpace = sharedLicenseActionNameSpace;
  sharedLicenseState: typeof SharedLicenseState = SharedLicenseState;

  licenceCallback: (value: License) => string = (value: License) => value.openLicenseId + " (" + value.title + ")";

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              private readonly _fb: FormBuilder,
              private readonly dialog: MatDialog,
              public readonly breakpointService: BreakpointService,
              private readonly _translateService: TranslateService) {
    super(_changeDetectorRef, _elementRef);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.validationLicenseRequiredWhenAccessPublic();
    this.validationEmbargo();
  }

  private validationLicenseRequiredWhenAccessPublic(): void {
    const licenseFormControl = this.form.get(this.formDefinition.licenseId);
    this.subscribe(this.form.get(this.formDefinition.access).valueChanges.pipe(
      distinctUntilChanged(),
      tap(accessLevel => {
        if (accessLevel === AccessEnum.PUBLIC) {
          licenseFormControl.setValidators([Validators.required]);
        } else {
          licenseFormControl.setValidators([]);
        }
        licenseFormControl.updateValueAndValidity();
      }),
    ));
  }

  protected initNewForm(): void {
    this.initEmbargoAccessLevel(AccessEnum.PUBLIC);
    let formDesc = {
      [this.formDefinition.organizationalUnitId]: [this.orgUnit.resId, [Validators.required, SolidifyValidator]],
      [this.formDefinition.title]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.description]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.languageId]: [""],
      [this.formDefinition.publicationDate]: [new Date(), [Validators.required, SolidifyValidator]],
      [this.formDefinition.collectionBegin]: [""],
      [this.formDefinition.collectionEnd]: [""],
      [this.formDefinition.access]: [AccessEnum.PUBLIC],
      [this.formDefinition.hasEmbargo]: [false],
      [this.formDefinition.licenseId]: [this.orgUnit?.defaultLicense?.resId, [SolidifyValidator]],
      [this.formDefinition.submissionPolicyId]: [this.orgUnit?.defaultSubmissionPolicy?.resId, [Validators.required, SolidifyValidator]],
      [this.formDefinition.preservationPolicyId]: [this.orgUnit?.defaultPreservationPolicy?.resId, [Validators.required, SolidifyValidator]],
      [this.formDefinition.authors]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.keywords]: [[...this.orgUnit?.keywords], [SolidifyValidator]],
      [this.formDefinition.doi]: [null, {disabled: true}],
      [this.formDefinition.embargoAccess]: [null],
      [this.formDefinition.embargoNumberMonths]: [""],
   };

    if (isNotNullNorUndefined(this.addFieldsForm)) {
      formDesc = {
        ...formDesc,
        [this.formDefinition.additionalFieldsFormId]: [this.addFieldsForm.resId, [Validators.required, SolidifyValidator]],
      };
    }

    this.form = this._fb.group(formDesc);
    this.formFormly = this._fb.group({});
  }

  protected bindFormTo(deposit: Deposit): void {
    this.initEmbargoAccessLevel(deposit.access);
    let formDesc = {
      [this.formDefinition.organizationalUnitId]: [deposit.organizationalUnitId, [Validators.required, SolidifyValidator]],
      [this.formDefinition.title]: [deposit.title, [Validators.required, SolidifyValidator]],
      [this.formDefinition.description]: [deposit.description, [Validators.required, SolidifyValidator]],
      [this.formDefinition.languageId]: [deposit.languageId],
      [this.formDefinition.publicationDate]: [deposit.publicationDate, [Validators.required, SolidifyValidator]],
      [this.formDefinition.collectionBegin]: [DateUtil.convertOffsetDateTimeIso8601ToDate(deposit.collectionBegin), [SolidifyValidator]],
      [this.formDefinition.collectionEnd]: [DateUtil.convertOffsetDateTimeIso8601ToDate(deposit.collectionEnd), [SolidifyValidator]],
      [this.formDefinition.access]: [deposit.access],
      [this.formDefinition.hasEmbargo]: [deposit.hasEmbargo],
      [this.formDefinition.licenseId]: [deposit.licenseId],
      [this.formDefinition.submissionPolicyId]: [deposit.submissionPolicyId, [Validators.required, SolidifyValidator]],
      [this.formDefinition.preservationPolicyId]: [deposit.preservationPolicyId, [Validators.required, SolidifyValidator]],
      [this.formDefinition.authors]: [_.map(this.selectedPersons, LocalModelAttributeEnum.resId), [Validators.required, SolidifyValidator]],
      [this.formDefinition.keywords]: [isNullOrUndefined(deposit.keywords) ? [] : [...deposit.keywords], [SolidifyValidator]],
      [this.formDefinition.doi]: [deposit.doi, {disabled: true}],
      [this.formDefinition.embargoAccess]: [isNullOrUndefined(deposit.embargo) ? "" : deposit.embargo.access],
      [this.formDefinition.embargoNumberMonths]: [isNullOrUndefined(deposit.embargo) ? "" : deposit.embargo.months],
    };

    if (isNotNullNorUndefined(deposit.additionalFieldsFormId)) {
      formDesc = {
        ...formDesc,
        [this.formDefinition.additionalFieldsFormId]: [deposit.additionalFieldsFormId, [Validators.required, SolidifyValidator]],
      };
    }

    this.form = this._fb.group(formDesc);

    this.formFormly = this._fb.group({}); // TODO INIT FORM WITH MODEL
    this.additionalFieldsValues = JSON.parse(deposit.additionalFieldsValues);

    this.isValidWhenDisable = this.form.valid;
  }

  protected treatmentBeforeSubmit(deposit: Deposit): Deposit {
    deposit.publicationDate = DateUtil.convertToLocalDateDateSimple(deposit.publicationDate);
    deposit.collectionBegin = DateUtil.convertToOffsetDateTimeIso8601(deposit.collectionBegin);
    deposit.collectionEnd = DateUtil.convertToOffsetDateTimeIso8601(deposit.collectionEnd);
    deposit.collectionEnd = DateUtil.convertToOffsetDateTimeIso8601(deposit.collectionEnd);
    if (isNotNullNorUndefined(this.formFormly.value) && MappingObjectUtil.size(this.formFormly.value) > 0) {
      deposit.additionalFieldsValues = JSON.stringify(this.formFormly.value);
    } else {
      deposit.additionalFieldsValues = null;
    }
    if (deposit.hasEmbargo) {
      const newEmbargo: AipInfoEmbargo = {
        access: this.form.get(this.formDefinition.embargoAccess).value === "" ? null : this.form.get(this.formDefinition.embargoAccess).value,
        months: this.form.get(this.formDefinition.embargoNumberMonths).value === "" ? null : this.form.get(this.formDefinition.embargoNumberMonths).value,
      };
      deposit.embargo = newEmbargo;
    }
    return deposit;
  }

  private validationEmbargo(): void {
    const embargoDurationFormControl = this.form.get(this.formDefinition.embargoNumberMonths);
    const embargoAccessFormControl =  this.form.get(this.formDefinition.embargoAccess);
    this.subscribe(this.form.get(this.formDefinition.hasEmbargo).valueChanges.pipe(
      distinctUntilChanged(),
      tap(hasEmbargo => {
        if (hasEmbargo === true) {
          embargoDurationFormControl.setValidators([Validators.required]);
          embargoAccessFormControl.setValidators([Validators.required]);
        } else {
          embargoDurationFormControl.setValidators([]);
          embargoAccessFormControl.setValidators([]);
        }
        embargoDurationFormControl.updateValueAndValidity();
        embargoAccessFormControl.updateValueAndValidity();
      }),
    ));
  }

  protected disableSpecificField(): void {
    this.form.get(this.formDefinition.doi).disable();
  }

  addPeople(): void {
    this.subscribe(
      this.dialog.open(DepositPersonDialog, {
        width: "90%",
      }).afterClosed().pipe(
        tap((person: PersonExtended) => {
          if (isNullOrUndefined(person)) {
            return;
          }
          this.listPersons = [...this.listPersons, person];
          this.addAuthor(person);
        }),
      ),
    );
  }

  addMeAsAuthor(): void {
    this.addAuthor(this.personConnected);
  }

  addAuthor(person: Person): void {
    this.authorMultiSearchableSelect.add(person);
  }

  personConnectedAlreadyInAuthor(): boolean {
    return this.form.get(this.formDefinition.authors).value.includes(this.personConnected.resId);
  }

  getSipPath(sipId: string): string {
    return "#" + urlSeparator + RoutesEnum.preservationSipDetail + urlSeparator + sipId;
  }

  navigateToPerson(person: PersonExtended): void {
    this.navigate([RoutesEnum.adminPersonDetail, person.resId]);
  }

  orderAuthors(listContributor: Person[]): void {
    this.subscribe(
      this.dialog.open(DepositOrderAuthorDialog, {
        width: "500px",
        data: {
          authorsId: this.form.get(this.formDefinition.authors).value,
          persons: listContributor,
        } as DepositOrderAuthorDialogWrapper,
      }).afterClosed().pipe(
        tap((authorsIdSorted: string[]) => {
          if (isNullOrUndefined(authorsIdSorted) || isEmptyArray(authorsIdSorted)) {
            return;
          }
          const formControl = this.form.get(this.formDefinition.authors);
          formControl.setValue(authorsIdSorted);
          formControl.markAsDirty();
        }),
      ),
    );
  }

  formlyChange(): void {
    if (this.form.dirty) {
      return;
    }
    this.form.markAsDirty();
    this._changeDetectorRef.detectChanges();
  }

  adaptEmbargoAccessLevel(value: AccessEnum): void {
    this.initEmbargoAccessLevel(value);
    this.form.get(this.formDefinition.embargoAccess).setValue(null);
  }

  initEmbargoAccessLevel(value: AccessEnum): void {
    this.embargoEnumValues =  [];
    this.embargoEnumValues = EmbargoAccessLevelEnumHelper.getListKeyValue(value);
  }

}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() organizationalUnitId: string;
  @PropertyName() title: string;
  @PropertyName() description: string;
  @PropertyName() languageId: string;
  @PropertyName() publicationDate: string;
  @PropertyName() collectionBegin: string;
  @PropertyName() collectionEnd: string;
  @PropertyName() authors: string;
  @PropertyName() access: string;
  @PropertyName() hasEmbargo: string;
  @PropertyName() embargoAccess: string;
  @PropertyName() embargoNumberMonths: string;
  @PropertyName() licenseId: string;
  @PropertyName() submissionPolicyId: string;
  @PropertyName() preservationPolicyId: string;
  @PropertyName() keywords: string;
  @PropertyName() doi: string;
  @PropertyName() additionalFieldsFormId: string;
}
