import {PersonRole} from "@admin/models/person-role.model";
import {
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {MatTabGroup} from "@angular/material/tabs";
import {
  ActivatedRoute,
  Router,
} from "@angular/router";
import {
  DepositAction,
  depositActionNameSpace,
} from "@app/features/deposit/stores/deposit.action";
import {
  DepositState,
  DepositStateModel,
} from "@app/features/deposit/stores/deposit.state";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {AppState} from "@app/stores/app.state";
import {AppOrgUnitPersonRoleAction} from "@app/stores/organizational-unit-person-role/app-organizational-unit-person-role.action";
import {ModeDepositTabEnum} from "@deposit/enums/mode-deposit-tab.enum";
import {DepositHelper} from "@deposit/helpers/deposit.helper";
import {DepositService} from "@deposit/services/deposit.service";
import {DepositCollectionAction} from "@deposit/stores/collection/deposit-collection.action";
import {DepositCollectionState} from "@deposit/stores/collection/deposit-collection.state";
import {DepositDataFileAction} from "@deposit/stores/data-file/deposit-data-file.action";
import {DepositDataFileState} from "@deposit/stores/data-file/deposit-data-file.state";
import {DepositPersonAction} from "@deposit/stores/people/deposit-person.action";
import {DepositStatusHistoryAction} from "@deposit/stores/status-history/deposit-status-history.action";
import {DepositStatusHistoryState} from "@deposit/stores/status-history/deposit-status-history.state";
import {environment} from "@environments/environment";
import {Deposit} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  ofActionCompleted,
  Select,
  Store,
} from "@ngxs/store";
import {Tab} from "@shared/components/containers/shared-tabs/shared-tabs.container";
import {SharedHistoryDialog} from "@shared/components/dialogs/shared-history/shared-history.dialog";
import {SharedAbstractDetailEditRoutable} from "@shared/components/routables/shared-abstract-detail-edit/shared-abstract-detail-edit.routable";
import {PackageStatusEnum} from "@shared/enums/business/package-status.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {
  AppRoutesEnum,
  DepositRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {PollingHelper} from "@shared/helpers/polling.helper";
import {ExtraButtonToolbar} from "@shared/models/extra-button-toolbar.model";
import {StatusHistoryDialog} from "@shared/models/status-history-dialog.model";
import {StatusHistory} from "@shared/models/status-history.model";
import {SecurityService} from "@shared/services/security.service";
import {
  BehaviorSubject,
  combineLatest,
  Observable,
} from "rxjs";
import {tap} from "rxjs/internal/operators/tap";

import {
  distinctUntilChanged,
  filter,
  flatMap,
  take,
} from "rxjs/operators";
import {
  isFalse,
  isNullOrUndefined,
  isTrue,
  MemoizedUtil,
  ObservableUtil,
  Override,
  QueryParameters,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-detail-edit-routable",
  templateUrl: "./deposit-detail-edit.routable.html",
  styleUrls: ["./deposit-detail-edit.routable.scss"],
})
export class DepositDetailEditRoutable extends SharedAbstractDetailEditRoutable<Deposit, DepositStateModel> implements OnInit, OnDestroy {
  historyObs: Observable<StatusHistory[]> = MemoizedUtil.select(this._store, DepositStatusHistoryState, state => state.history);
  isLoadingHistoryObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, DepositStatusHistoryState);
  isLoadingObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, DepositState);
  queryParametersObs: Observable<QueryParameters> = MemoizedUtil.select(this._store, DepositStatusHistoryState, state => state.queryParameters);
  numberFilesNotReady: Observable<number | undefined> = MemoizedUtil.select(this._store, DepositDataFileState, state => state.numberFilesNotReady);
  numberFilesObs: Observable<number | undefined> = MemoizedUtil.select(this._store, DepositDataFileState, state => state.numberFiles);
  numberCollectionObs: Observable<number | undefined> = MemoizedUtil.select(this._store, DepositCollectionState, state => state.numberCollections);
  @Select(AppState.currentOrgUnitPersonRoleResId) currentPersonRoleObs: Observable<string[]>;
  @Select(AppState.currentOrgUnitPerson) currentPersonObs: Observable<PersonRole>;
  @Select(AppState.currentUserApplicationRolesResId) currentUserApplicationRoleObs: Observable<string[]>;

  depositModeTabEnumObs: Observable<ModeDepositTabEnum> = MemoizedUtil.select(this._store, DepositState, state => state.depositModeTabEnum);

  @ViewChild("matTabGroup")
  readonly matTabGroup: MatTabGroup;

  private get rootUrl(): string[] {
    return [AppRoutesEnum.deposit, this._orgUnitResId, DepositRoutesEnum.detail, this._resId];
  }

  listTabs: Tab[] = [
    {
      id: TabEnum.METADATA,
      suffixUrl: DepositRoutesEnum.metadata,
      icon: IconNameEnum.metadata,
      titleToTranslate: MARK_AS_TRANSLATABLE("deposit.tab.details"),
      route: () => [...this.rootUrl, DepositRoutesEnum.metadata],
    },
    {
      id: TabEnum.FILES,
      suffixUrl: DepositRoutesEnum.data,
      icon: IconNameEnum.files,
      titleToTranslate: MARK_AS_TRANSLATABLE("deposit.tab.data"),
      route: () => [...this.rootUrl, DepositRoutesEnum.data],
    },
  ];

  messageReasonUnableSubmit: string | undefined;
  canDoValidatorActions: boolean = false;
  canDoAlterationActions: boolean = false;
  canSubmitAction: boolean = false;
  canReserveDoiAction: boolean = false;

  canDoAlterationActionsBS: BehaviorSubject<boolean | undefined> = new BehaviorSubject(undefined);
  readonly canDoAlterationActionsObs: Observable<boolean | undefined> = ObservableUtil.asObservable(this.canDoAlterationActionsBS);

  private _orgUnitResId: string | undefined = undefined;

  readonly KEY_PARAM_NAME: keyof Deposit & string = "title";

  listExtraButtons: ExtraButtonToolbar<Deposit>[] = [
    {
      color: "primary",
      typeButton: "flat-button",
      icon: IconNameEnum.save,
      labelToTranslate: MARK_AS_TRANSLATABLE("app.navigation.button.save"),
      order: 40,
      callback: () => this.depositService?.formPresentational?.onSubmit(),
      displayCondition: (resource) => this.isEdit,
      disableCondition: () => this.depositService?.formPresentational?.form?.pristine || this.depositService?.formPresentational?.form?.invalid || this.depositService?.formPresentational?.formFormly?.invalid,
    },
    {
      color: "primary",
      icon: IconNameEnum.download,
      displayCondition: current => !isNullOrUndefined(current) && !isNullOrUndefined(current.status) && current.status === PackageStatusEnum.COMPLETED,
      callback: () => this.download(),
      labelToTranslate: MARK_AS_TRANSLATABLE("preservation.sip.button.download"),
      order: 40,
    },
    {
      color: "primary",
      icon: IconNameEnum.edit,
      displayCondition: current => !this.isEdit && !isNullOrUndefined(current) && (current.status === Deposit.StatusEnum.INVALIDATION || current.status === Deposit.StatusEnum.REJECTED || current.status === Deposit.StatusEnum.INERROR),
      callback: () => this.backToRevision(),
      disableCondition: current => isNullOrUndefined(current),
      labelToTranslate: MARK_AS_TRANSLATABLE("deposit.backToEdit"),
      order: 21,
    },
    {
      color: "primary",
      icon: IconNameEnum.reserveDoi,
      displayCondition: current => this.displayReserveDOIButton(),
      callback: () => this.reserveDOI(),
      disableCondition: current => isNullOrUndefined(current),
      labelToTranslate: MARK_AS_TRANSLATABLE("deposit.reserveDOI"),
      tooltipToTranslate: MARK_AS_TRANSLATABLE("deposit.tooltips.doi"),
      order: 50,
    },
    {
      color: "primary",
      icon: IconNameEnum.submit,
      displayCondition: current => this.displaySubmitButton(),
      callback: () => this.submit(),
      disableCondition: current => isNullOrUndefined(current),
      labelToTranslate: MARK_AS_TRANSLATABLE("deposit.submit"),
      typeButton: "flat-button",
      order: 51,
    },
    {
      color: "success",
      icon: IconNameEnum.approve,
      displayCondition: current => !this.isEdit && this.canDoValidatorActions,
      callback: () => this.approve(),
      disableCondition: current => isNullOrUndefined(current),
      labelToTranslate: MARK_AS_TRANSLATABLE("deposit.approve"),
      typeButton: "flat-button",
      order: 52,
    },
    {
      color: "warn",
      icon: IconNameEnum.unapprove,
      displayCondition: current => !this.isEdit && this.canDoValidatorActions,
      callback: () => this.reject(),
      disableCondition: current => isNullOrUndefined(current),
      labelToTranslate: MARK_AS_TRANSLATABLE("deposit.reject"),
      typeButton: "flat-button",
      order: 53,
    },
  ];

  get depositHelper(): typeof DepositHelper {
    return DepositHelper;
  }

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _dialog: MatDialog,
              protected readonly _router: Router,
              private readonly _securityService: SecurityService,
              public readonly depositService: DepositService) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.deposit, depositActionNameSpace);
  }

  ngOnInit(): void {
    super.ngOnInit();

    this.retrieveResource();
    this.retrieveCurrentModelWithUrl();
    this._computeCurrentUserRight();
    this._computeEditableField();

    this.subscribe(this.depositModeTabEnumObs.pipe(distinctUntilChanged()), mode => {
      const tab = this.listTabs.find(t => t.id === TabEnum.FILES);
      if (mode === ModeDepositTabEnum.COLLECTION) {
        tab.icon = IconNameEnum.collection;
        tab.titleToTranslate = MARK_AS_TRANSLATABLE("deposit.tab.collection");
      } else if (mode === ModeDepositTabEnum.FILE) {
        tab.icon = IconNameEnum.files;
        tab.titleToTranslate = MARK_AS_TRANSLATABLE("deposit.tab.datafiles");
      } else {
        tab.icon = IconNameEnum.files;
        tab.titleToTranslate = MARK_AS_TRANSLATABLE("deposit.tab.data");
      }
      this.listTabs = [...this.listTabs];
    });

    this._createPollingListenerForFileReady();
  }

  private _createPollingListenerForFileReady(): void {
    this.subscribe(PollingHelper.startPollingObs({
      intervalRefreshInSecond: environment.refreshDepositSubmittedIntervalInSecond,
      doBeforeFilter: () => {
        const shouldContinuePollingBefore = this.shouldContinuePolling;
        this.computedShouldContinuePolling();
        if (shouldContinuePollingBefore && !this.shouldContinuePolling) {
          this._store.dispatch(new DepositDataFileAction.Refresh(this._resId));
        }
      },
      filter: () => this.shouldContinuePolling,
      actionToDo: () => {
        this._store.dispatch(new DepositDataFileAction.GetNumberFilesInError(this._resId));
        this._store.dispatch(new DepositDataFileAction.GetNumberFilesIgnored(this._resId));
        this._store.dispatch(new DepositDataFileAction.GetNumberFilesExcluded(this._resId));
        this._store.dispatch(new DepositDataFileAction.GetNumberFilesNotReady(this._resId));
      },
    }));
  }

  private shouldContinuePolling: boolean = false;

  private computedShouldContinuePolling(): void {
    this.shouldContinuePolling = MemoizedUtil.selectSnapshot(this._store, DepositDataFileState, state => state.numberFilesInError === 0)
      && MemoizedUtil.selectSnapshot(this._store, DepositDataFileState, state => state.numberFilesIgnored === 0)
      && MemoizedUtil.selectSnapshot(this._store, DepositDataFileState, state => state.numberFilesExcluded === 0)
      && MemoizedUtil.selectSnapshot(this._store, DepositDataFileState, state => state.numberFilesNotReady > 0);
  }

  ngOnDestroy(): void {
    super.ngOnDestroy();
    this.cleanState();
  }

  protected retrieveResIdFromUrl(): void {
    this._resId = this._route.snapshot.paramMap.get(AppRoutesEnum.paramIdWithoutPrefixParam);
    this._orgUnitResId = this._route.parent.snapshot.paramMap.get(AppRoutesEnum.paramIdOrgUnitWithoutPrefixParam);
  }

  private _computeEditableField(): void {
    this.subscribe(this.canDoAlterationActionsObs.pipe(
      distinctUntilChanged(),
      tap(isEdit => {
        this._store.dispatch(new DepositAction.ChangeEditProperty(isEdit));
      }),
    ));
  }

  private _computeCurrentUserRight(): void {
    this.subscribe(this.currentObs.pipe(
      filter(deposit => !isNullOrUndefined(deposit) && deposit.resId === this._resId && isFalse(this.isEdit) && deposit.status === Deposit.StatusEnum.INPROGRESS),
      tap(deposit => {
        this.canReserveDoiAction = isNullOrUndefined(deposit.doi);
      }),
    ));

    this.subscribe(this.currentObs.pipe(
      distinctUntilChanged(),
      filter(deposit => !isNullOrUndefined(deposit) && deposit.resId === this._resId),
      tap(deposit => {
        const canDoValidatorActions = this._securityService.depositInValidationStep(deposit);
        const canDoAlterationActions = this._securityService.depositInEditionStep(deposit);

        this.subscribe(combineLatest([this.numberCollectionObs, this.numberFilesObs, this.numberFilesNotReady])
          .pipe(
            distinctUntilChanged(),
            filter(([numberCollections, numberFiles, numberFilesNotReady]) =>
              !isNullOrUndefined(numberCollections) &&
              !isNullOrUndefined(numberFiles) &&
              !isNullOrUndefined(numberFilesNotReady)),
            tap(([numberCollections, numberFiles, numberFilesNotReady]) => {
              let canSubmitAction = deposit.status === Deposit.StatusEnum.INPROGRESS;
              this.messageReasonUnableSubmit = undefined;
              if (numberFiles === 0 && numberCollections === 0) {
                this.messageReasonUnableSubmit = MARK_AS_TRANSLATABLE("deposit.error.noFileOrCollection");
                canSubmitAction = false;
              }
              if (numberFilesNotReady > 0) {
                this.messageReasonUnableSubmit = MARK_AS_TRANSLATABLE("deposit.error.fileNotReady");
                canSubmitAction = false;
              }
              this.canSubmitAction = canSubmitAction;
            }),
          ));

        if (this._securityService.isRootOrAdmin()) {
          this.canDoValidatorActions = canDoValidatorActions;
          this.canDoAlterationActions = canDoAlterationActions;
          this.canDoAlterationActionsBS.next(this.canDoAlterationActions);
          return;
        }

        const currentPerson = this._store.selectSnapshot(AppState.currentPerson);
        const currentOrgUnitPerson = this._store.selectSnapshot(AppState.currentOrgUnitPerson);
        if (this.checkPersonRole(currentOrgUnitPerson, deposit)) {
          this.computeCanDoValidatorAndAlterationWithCurrentRole(currentOrgUnitPerson, canDoValidatorActions, canDoAlterationActions, deposit);
          return;
        }

        this._store.dispatch(new AppOrgUnitPersonRoleAction.GetById(deposit.organizationalUnitId, currentPerson.resId));
        this.subscribe(this._actions$.pipe(
          ofActionCompleted(AppOrgUnitPersonRoleAction.GetByIdSuccess),
          flatMap(() => combineLatest([this.currentPersonObs])
            .pipe(
              filter(([personRole]) => this.checkPersonRole(personRole, deposit)),
              distinctUntilChanged(),
              tap(([personRole]) => {
                this.computeCanDoValidatorAndAlterationWithCurrentRole(personRole, canDoValidatorActions, canDoAlterationActions, deposit);
              }),
            )),
          take(1),
        ));

      }),
    ));
  }

  private computeCanDoValidatorAndAlterationWithCurrentRole(personRole: PersonRole, canDoValidatorActions: boolean, canDoAlterationActions: boolean, deposit: Deposit): void {
    let roles = [];
    if (!isNullOrUndefined(personRole.roles)) {
      roles = personRole.roles.map(r => r.resId);
    }

    this.canDoValidatorActions = canDoValidatorActions;
    this.canDoAlterationActions = canDoAlterationActions;
    if (isTrue(this.canDoValidatorActions)) {
      this.canDoValidatorActions = this._securityService.canApproveOrRejectDeposit(deposit, roles);
    }
    if (isTrue(this.canDoAlterationActions)) {
      this.canDoAlterationActions = this._securityService.canEditDeposit(deposit, roles);
    }
    this.canDoAlterationActionsBS.next(this.canDoAlterationActions);
    this._changeDetector.detectChanges();
  }

  private checkPersonRole(personRole: PersonRole, deposit: Deposit): boolean {
    return !isNullOrUndefined(personRole) && !isNullOrUndefined(deposit) && personRole.orgUnitId === deposit.organizationalUnitId;
  }

  private displaySubmitButton(): boolean {
    return !this.isEdit && this.canDoAlterationActions && isFalse(MemoizedUtil.isLoadingSnapshot(this._store, DepositState)) && this.canSubmitAction;
  }

  private displayReserveDOIButton(): boolean {
    return !this.isEdit && this.canDoAlterationActions && this.canReserveDoiAction;
  }

  @Override()
  edit(): void {
    this._store.dispatch(new Navigate([RoutesEnum.deposit, this._orgUnitResId, DepositRoutesEnum.detail, this._resId, DepositRoutesEnum.metadata, DepositRoutesEnum.edit]));
  }

  getSubResourceWithParentId(id: string): void {
    this._store.dispatch(new DepositPersonAction.GetAll(id));
    this._store.dispatch(new DepositDataFileAction.Refresh(id));
    this._store.dispatch(new DepositCollectionAction.Refresh(id));
  }

  submit(): void {
    this.subscribe(this.currentObs.pipe(
      take(1),
      tap(deposit => this._store.dispatch(new DepositAction.Submit(deposit))),
    ));
  }

  reserveDOI(): void {
    this.subscribe(this.currentObs.pipe(
      take(1),
      tap(deposit => this._store.dispatch(new DepositAction.ReserveDOI(deposit))),
    ));
  }

  showHistory(): void {
    const dialogRef = this._dialog.open(SharedHistoryDialog, {
      width: environment.modalWidth,
      data: {
        parentId: null,
        resourceResId: this._resId,
        name: this._state,
        statusHistory: this.historyObs,
        isLoading: this.isLoadingHistoryObs,
        queryParametersObs: this.queryParametersObs,
        state: DepositStatusHistoryAction,
      } as StatusHistoryDialog,
    });
  }

  approve(): void {
    this.subscribe(this.currentObs.pipe(
      take(1),
      tap(deposit => this._store.dispatch(new DepositAction.Approve(deposit))),
    ));
  }

  reject(): void {
    this.subscribe(this.currentObs.pipe(
      take(1),
      tap(deposit => this._store.dispatch(new DepositAction.Reject(deposit))),
    ));
  }

  backToRevision(): void {
    this.subscribe(this.currentObs.pipe(
      take(1),
      tap(deposit => this._store.dispatch(new DepositAction.BackToEdit(deposit))),
    ));
  }

  backToList(): void {
    this._store.dispatch(new Navigate([AppRoutesEnum.deposit, this._orgUnitResId]));
  }

  @Override()
  backToDetail(): void {
    super.backToDetail([...this.rootUrl, DepositRoutesEnum.metadata]);
  }

  private currentTab: Tab;

  setCurrentTab($event: Tab): void {
    this.currentTab = $event;
  }

  download(): void {
    this._store.dispatch(new DepositAction.Download(this._resId));
  }
}

enum TabEnum {
  METADATA = "METADATA",
  FILES = "FILES",
}
