import {
  ChangeDetectorRef,
  Component,
  ElementRef,
  OnInit,
} from "@angular/core";
import {
  AbstractControl,
  FormBuilder,
  FormGroup,
  Validators,
} from "@angular/forms";
import {MatDialog} from "@angular/material/dialog";
import {MatTabChangeEvent} from "@angular/material/tabs";
import {
  ActivatedRoute,
  NavigationCancel,
  NavigationEnd,
  Router,
} from "@angular/router";
import {
  DepositAction,
  depositActionNameSpace,
} from "@app/features/deposit/stores/deposit.action";
import {
  DepositState,
  DepositStateModel,
} from "@app/features/deposit/stores/deposit.state";
import {SharedAbstractListRoutable} from "@app/shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@app/shared/enums/field-type.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {
  AppRoutesEnum,
  DepositRoutesEnum,
  RoutesEnum,
} from "@app/shared/enums/routes.enum";
import {
  DepositTabStatusEnum,
  DepositTabStatusName,
} from "@deposit/enums/deposit-tab-status.enum";
import {depositAuthorizedOrganizationalUnitNameSpace} from "@deposit/stores/authorized-organizational-unit/deposit-authorized-organizational-unit.action";
import {DepositAuthorizedOrganizationalUnitState} from "@deposit/stores/authorized-organizational-unit/deposit-authorized-organizational-unit.state";
import {environment} from "@environments/environment";
import {
  Deposit,
  OrganizationalUnit,
} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {DepositStatusEnumHelper} from "@shared/enums/business/deposit-status.enum";
import {PackageStatusEnum} from "@shared/enums/business/package-status.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {PollingHelper} from "@shared/helpers/polling.helper";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {DataTableActions} from "@shared/models/data-table-actions.model";
import {DataTableColumns} from "@shared/models/data-table-columns.model";
import {RouterExtService} from "@shared/services/router-ext.service";
import {UserPreferencesUtil} from "@shared/utils/user-preferences.util";
import {Observable} from "rxjs";
import {
  distinctUntilChanged,
  filter,
  tap,
} from "rxjs/operators";
import {
  FormValidationHelper,
  isNullOrUndefined,
  isUndefined,
  MappingObjectUtil,
  MemoizedUtil,
  OrderEnum,
  Override,
  PropertyName,
  QueryParameters,
  QueryParametersUtil,
  ResourceNameSpace,
  ResourceState,
  SolidifyValidator,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-list-routable",
  templateUrl: "./deposit-list.routable.html",
  styleUrls: ["./deposit-list.routable.scss"],
})
export class DepositListRoutable extends SharedAbstractListRoutable<Deposit, DepositStateModel> implements OnInit {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  readonly KEY_CREATE_BUTTON: string = MARK_AS_TRANSLATABLE("deposit.new");
  readonly KEY_REFRESH_BUTTON: string = MARK_AS_TRANSLATABLE("deposit.refresh");
  readonly KEY_BACK_BUTTON: string | undefined = undefined;
  readonly KEY_PARAM_NAME: keyof Deposit & string = "title";
  readonly KEY_ORGANIZATIONAL_UNIT: keyof Deposit & string = "organizationalUnitId";
  readonly KEY_STATUS: keyof Deposit & string = "status";

  depositAuthorizedOrganizationalUnitNameSpace: ResourceNameSpace = depositAuthorizedOrganizationalUnitNameSpace;
  depositAuthorizedOrganizationalUnitState: typeof DepositAuthorizedOrganizationalUnitState = DepositAuthorizedOrganizationalUnitState;

  form: FormGroup;

  private _orgUnitResId: string;
  private _isInitialized: boolean = false;
  listTabStatus: DepositTabStatus[] = [
    {
      tabEnum: DepositTabStatusEnum.inProgress,
      depositStatusEnum: Deposit.StatusEnum.INPROGRESS,
      labelToTranslate: DepositTabStatusName.inProgress,
      elementCount: MemoizedUtil.select(this._store, DepositState, s => s.counterTabInProgress),
    },
    {
      tabEnum: DepositTabStatusEnum.inValidation,
      depositStatusEnum: Deposit.StatusEnum.INVALIDATION,
      labelToTranslate: DepositTabStatusName.inValidation,
      elementCount: MemoizedUtil.select(this._store, DepositState, s => s.counterTabInValidation),
    },
    {
      tabEnum: DepositTabStatusEnum.completed,
      depositStatusEnum: Deposit.StatusEnum.COMPLETED,
      labelToTranslate: DepositTabStatusName.completed,
      // elementCount: MemoizedUtil.select(this._store, DepositState, s => s.counterTabCompleted),
    },
    {
      tabEnum: DepositTabStatusEnum.all,
      depositStatusEnum: undefined,
      labelToTranslate: DepositTabStatusName.all,
      // elementCount: MemoizedUtil.select(this._store, DepositState, s => s.counterTabAll),
    },
  ];
  currentTabStatus: DepositTabStatus | undefined;
  selectedTabIndex: number;

  columnsSkippedToClear: string[] = [this.KEY_ORGANIZATIONAL_UNIT, this.KEY_STATUS];

  columnStatus: DataTableColumns = {
    field: this.KEY_STATUS,
    header: MARK_AS_TRANSLATABLE("deposit.table.header.status"),
    type: FieldTypeEnum.singleSelect,
    order: OrderEnum.none,
    isFilterable: true,
    isSortable: true,
    translate: true,
    filterEnum: DepositStatusEnumHelper.getListKeyValue(),
  };
  private INTERVAL_REFRESH_IN_SECOND: number = environment.refreshTabStatusCounterIntervalInSecond;

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog,
              private readonly _fb: FormBuilder,
              protected readonly _router: Router) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, LocalStateEnum.deposit, depositActionNameSpace, {
      canCreate: MemoizedUtil.select(_store, DepositState, state => state.canCreate),
    });
  }

  ngOnInit(): void {
    super.ngOnInit();

    this.retrieveOrgUnitInUrl();
    this.updateCurrentTab();
    this.subscribe(this.observeRoutingUpdate());

    this.form = this._fb.group({
      [this.formDefinition.organizationalUnitId]: [this._orgUnitResId, [Validators.required, SolidifyValidator]],
    });

    this.subscribe(this._route.url.pipe(distinctUntilChanged()),
      value => {
        // TAB or ORGUNIT change
        this.retrieveOrgUnitInUrl();
        if (this._isInitialized === true) {
          let queryParameters = QueryParametersUtil.clone(ResourceState.queryParametersSnapshot(this._store, DepositState));
          queryParameters = QueryParametersUtil.resetToFirstPage(queryParameters);
          this.onQueryParametersEvent(queryParameters, false);
        }
      });

    this.subscribe(this.form.get(this.formDefinition.organizationalUnitId).valueChanges.pipe(
      distinctUntilChanged(),
      tap(resId => {
        // ORGUNIT CHANGE
        this._store.dispatch(new Navigate([RoutesEnum.deposit, resId, this.currentTabStatus.tabEnum]));
        this._store.dispatch(new DepositAction.SetOrganizationalUnit(resId));
        this.refreshCounter();
      }),
    ));

    this._isInitialized = true;

    this.refreshCounter();
    this.subscribe(PollingHelper.startPollingObs({
      intervalRefreshInSecond: this.INTERVAL_REFRESH_IN_SECOND,
      actionToDo: () => this.refreshCounter(),
    }));
  }

  private refreshCounter(): void {
    const listTabWithCounter = this.listTabStatus.filter(t => !isNullOrUndefined(t.elementCount)).map(tabStatus => tabStatus.depositStatusEnum);
    this._store.dispatch(new DepositAction.RefreshAllCounterStatusTab(listTabWithCounter));
  }

  getFormControl(key: string): AbstractControl {
    return FormValidationHelper.getFormControl(this.form, key);
  }

  private retrieveOrgUnitInUrl(): void {
    this._orgUnitResId = this._route.snapshot.parent.paramMap.get(AppRoutesEnum.paramIdOrgUnitWithoutPrefixParam);
  }

  get formValidationHelper(): typeof FormValidationHelper {
    return FormValidationHelper;
  }

  private observeRoutingUpdate(): Observable<any> {
    return this._router.events
      .pipe(
        filter(event => event instanceof NavigationCancel || event instanceof NavigationEnd),
        distinctUntilChanged(),
        tap(event => {
          this.updateCurrentTab();
        }),
      );
  }

  private updateCurrentTab(): void {
    const tab: DepositTabStatusEnum = this._route.snapshot.paramMap.get(DepositRoutesEnum.paramTabWithoutPrefix) as DepositTabStatusEnum;
    this.selectedTabIndex = this.listTabStatus.findIndex(t => t.tabEnum === tab);
    if (this.selectedTabIndex !== -1) {
      this.currentTabStatus = this.listTabStatus[this.selectedTabIndex];
    }
  }

  private download(deposit: Deposit): void {
    this._store.dispatch(new DepositAction.Download(deposit.resId));
  }

  onTabChanged(tabEvent: MatTabChangeEvent): void {
    const index = tabEvent.index;
    this.currentTabStatus = this.listTabStatus[index];
    this.selectedTabIndex = index;
    this._store.dispatch(new Navigate([AppRoutesEnum.deposit, this._orgUnitResId, this.currentTabStatus.tabEnum]));
  }

  conditionDisplayEditButton(model: Deposit | undefined): boolean {
    if (isUndefined(model)) {
      return true;
    }
    return model.status === Deposit.StatusEnum.INPROGRESS || model.status === Deposit.StatusEnum.INERROR;
  }

  conditionDisplayDeleteButton(model: Deposit | undefined): boolean {
    if (isUndefined(model)) {
      return true;
    }
    return model.status === Deposit.StatusEnum.INPROGRESS || model.status === Deposit.StatusEnum.INERROR;
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "title",
        header: MARK_AS_TRANSLATABLE("deposit.table.header.title"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "publicationDate",
        header: MARK_AS_TRANSLATABLE("deposit.table.header.publicationDate"),
        type: FieldTypeEnum.date,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "creation.when" as any,
        header: MARK_AS_TRANSLATABLE("deposit.table.header.creation.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        width: "145px",
      },
      {
        field: "lastUpdate.when" as any,
        header: MARK_AS_TRANSLATABLE("deposit.table.header.lastUpdate.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.descending,
        isFilterable: true,
        isSortable: true,
      },
    ];
  }

  @Override()
  protected defineActions(): DataTableActions<Deposit>[] {
    return [
      {
        logo: IconNameEnum.download,
        callback: (model: Deposit) => this.download(model),
        placeholder: MARK_AS_TRANSLATABLE("crud.list.action.download"),
        displayOnCondition: (model: Deposit) => !isNullOrUndefined(model) && !isNullOrUndefined(model.status) && model.status === PackageStatusEnum.COMPLETED,
      },
    ];
  }

  @Override()
  goToEdit(deposit: Deposit): void {
    this._store.dispatch(new Navigate([AppRoutesEnum.deposit, deposit.organizationalUnitId, DepositRoutesEnum.detail, deposit.resId, DepositRoutesEnum.edit]));
  }

  showDetail(deposit: Deposit): void {
    this._store.dispatch(new Navigate([AppRoutesEnum.deposit, deposit.organizationalUnitId, DepositRoutesEnum.detail, deposit.resId]));
  }

  create(element: ElementRef): void {
    this._store.dispatch(new Navigate([RoutesEnum.deposit, this._orgUnitResId, DepositRoutesEnum.create]));
  }

  private updateQueryParameterWithOrgUnit(queryParameters: QueryParameters, orgUnit: OrganizationalUnit | undefined): QueryParameters | undefined {
    if (isNullOrUndefined(orgUnit)) {
      return queryParameters;
    }
    UserPreferencesUtil.setPreferredOrgUnitInDepositMenu(orgUnit.resId);
    MappingObjectUtil.set(queryParameters.search.searchItems, this.KEY_ORGANIZATIONAL_UNIT, orgUnit.resId);
    return queryParameters;
  }

  private updateQueryParameterWithStatus(queryParameters: QueryParameters): QueryParameters | undefined {
    if (isNullOrUndefined(this.currentTabStatus)) {
      return queryParameters;
    }
    const indexOf = this.columns.findIndex(c => c.field === this.KEY_STATUS);
    if (isUndefined(this.currentTabStatus.depositStatusEnum)) {
      if (indexOf === -1) {
        this.columns.push(this.columnStatus);
        this.columns = [...this.columns];
        MappingObjectUtil.delete(queryParameters.search.searchItems, this.KEY_STATUS);
      }
    } else {
      if (indexOf !== -1) {
        this.columns.splice(indexOf, 1);
        this.columns = [...this.columns];
      }
      MappingObjectUtil.set(queryParameters.search.searchItems, this.KEY_STATUS, this.currentTabStatus.depositStatusEnum);
    }
    return queryParameters;
  }

  onQueryParametersEvent(queryParameters: QueryParameters, needCopy: boolean = true): void {
    if (needCopy) {
      queryParameters = QueryParametersUtil.clone(queryParameters);
    }
    this.updateQueryParameterWithOrgUnit(queryParameters, {resId: this._orgUnitResId});
    this.updateQueryParameterWithStatus(queryParameters);
    super.onQueryParametersEvent(queryParameters);
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() organizationalUnitId: string;
}

interface DepositTabStatus {
  // tabIndex: DepositTabStatusIndexEnum;
  tabEnum: DepositTabStatusEnum;
  depositStatusEnum: Deposit.StatusEnum | undefined;
  labelToTranslate: string;
  elementCount?: Observable<number> | undefined;
}
