import {
  ChangeDetectionStrategy,
  Component,
  Input,
  OnInit,
  Output,
} from "@angular/core";
import {SearchScopeEnum} from "@app/features/home/enums/search-scope.enum";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {ArchiveMetadata} from "@app/shared/models/business/archive-metadata.model";
import {MetadataUtil} from "@app/shared/utils/metadata.util";
import {Deposit} from "@models";
import {GetShortDoiWrapper} from "@shared/components/presentationals/shared-doi-menu/shared-doi-menu.presentational";
import {AccessLevelEnumHelper} from "@shared/enums/business/access-level-enum.helper";
import {FileSizePipe} from "@shared/pipes/file-size.pipe";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  DateUtil,
  isNullOrUndefined,
  ObservableUtil,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-home-archive-detail",
  templateUrl: "./home-archive-detail.presentational.html",
  styleUrls: ["./home-archive-detail.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class HomeArchiveDetailPresentational extends SharedAbstractPresentational implements OnInit {
  listInfo: Info[];

  private readonly _downloadBS: BehaviorSubject<ArchiveMetadata | undefined> = new BehaviorSubject<ArchiveMetadata | undefined>(undefined);
  @Output("downloadChange")
  readonly downloadObs: Observable<ArchiveMetadata | undefined> = ObservableUtil.asObservable(this._downloadBS);

  private readonly _addToCartBS: BehaviorSubject<ArchiveMetadata | undefined> = new BehaviorSubject<ArchiveMetadata | undefined>(undefined);
  @Output("addToCartChange")
  readonly addToCartObs: Observable<ArchiveMetadata | undefined> = ObservableUtil.asObservable(this._addToCartBS);

  private readonly _askAccessBS: BehaviorSubject<ArchiveMetadata | undefined> = new BehaviorSubject<ArchiveMetadata | undefined>(undefined);
  @Output("askAccessChange")
  readonly askAccessObs: Observable<ArchiveMetadata | undefined> = ObservableUtil.asObservable(this._askAccessBS);

  @Input()
  archiveMetadata: ArchiveMetadata;

  @Input()
  searchScope: SearchScopeEnum;

  @Input()
  isLoadingPrepareDownload: boolean;

  @Input()
  isLoggedIn: boolean;

  showCode: boolean = false;

  fileSizePipe: FileSizePipe;

  private readonly _shortDoiBS: BehaviorSubject<GetShortDoiWrapper | undefined> = new BehaviorSubject<GetShortDoiWrapper | undefined>(undefined);
  @Output("getShortDoi")
  readonly shortDoiObs: Observable<GetShortDoiWrapper | undefined> = ObservableUtil.asObservable(this._shortDoiBS);

  get typeInfoEnum(): typeof TypeInfoEnum {
    return TypeInfoEnum;
  }

  ngOnInit(): void {
    const metadata = this.archiveMetadata.metadata;
    this.fileSizePipe = new FileSizePipe();
    this.listInfo = [
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.title"),
        value: MetadataUtil.getTitle(this.searchScope, metadata),
      },
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.description"),
        value: MetadataUtil.getDescription(this.searchScope, metadata),
      },
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.orgUnit"),
        value: MetadataUtil.getOrganizationalUnitResId(metadata),
        type: TypeInfoEnum.orgUnit,
      },
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.type"),
        value: MetadataUtil.getType(this.searchScope, metadata),
      },
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.authors"),
        isList: true,
      },
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.publicationYear"),
        value: MetadataUtil.getPublicationYear(this.searchScope, metadata),
      },
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.DOI"),
        value: MetadataUtil.getDOI(this.searchScope, metadata),
        type: TypeInfoEnum.doi,
      },
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.licenses"),
        value: MetadataUtil.getLicenses(this.searchScope, metadata),
      },
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.accessLevel"),
        value: AccessLevelEnumHelper.getLabel(MetadataUtil.getAccessLevel(this.searchScope, metadata)),
        type: TypeInfoEnum.valueToTranslate,
      },
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.embargoAccessLevel"),
        value: MetadataUtil.getEmbargoAccessLevel(this.searchScope, metadata),
      },
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.size"),
        value: this.fileSizePipe.transform(MetadataUtil.getSize(metadata)),
      },
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.files"),
        value: MetadataUtil.getFileNumber(metadata),
      },
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.embargoEndDate"),
        value: DateUtil.convertDateToDateString(MetadataUtil.getEmbargoEndDate(this.searchScope, metadata)),
      },
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.submittedDate"),
        value: DateUtil.convertDateToDateTimeString(MetadataUtil.getSubmittedDate(this.searchScope, metadata)),
      },
      {
        labelToTranslate: MARK_AS_TRANSLATABLE("homePage.archive.detail.acceptedDate"),
        value: DateUtil.convertDateToDateTimeString(MetadataUtil.getAcceptedDate(this.searchScope, metadata)),
      },
      {
        labelToTranslate: this.getMetadataLabel(),
        value: MetadataUtil.getDatacite(this.searchScope, this.archiveMetadata.metadata),
        code: true,
      },
    ];
  }

  getAuthorsList(): string[] {
    return MetadataUtil.getAuthors(this.searchScope, this.archiveMetadata.metadata);
  }

  download(archiveMetadata: ArchiveMetadata): void {
    this._downloadBS.next(archiveMetadata);
  }

  getMetadataLabel(): string {
    if (this.searchScope === SearchScopeEnum.public) {
      return MARK_AS_TRANSLATABLE("homePage.archive.detail.datacite");
    } else {
      return MARK_AS_TRANSLATABLE("homePage.archive.detail.dlcm");
    }
  }

  getMetadata(): string {
    return MetadataUtil.getDatacite(this.searchScope, this.archiveMetadata.metadata);
  }

  getShortDoi(longDoi: GetShortDoiWrapper): void {
    this._shortDoiBS.next(longDoi);
  }

  addToCart(archiveMetadata: ArchiveMetadata): void {
    this._addToCartBS.next(archiveMetadata);
  }

  askAccess(archiveMetadata: ArchiveMetadata): void {
    this._askAccessBS.next(archiveMetadata);
  }

  isPublicMetadata(): boolean {
    if (isNullOrUndefined(this.archiveMetadata)) {
      return false;
    }
    return MetadataUtil.getAccessLevel(this.searchScope, this.archiveMetadata.metadata) === Deposit.AccessEnum.PUBLIC;
  }

  isRestrictedOrClosedAccessLevel(): boolean {
    if (isNullOrUndefined(this.archiveMetadata)) {
      return false;
    }
    const accessLevel = MetadataUtil.getAccessLevel(this.searchScope, this.archiveMetadata.metadata);
    return accessLevel === Deposit.AccessEnum.RESTRICTED || accessLevel === Deposit.AccessEnum.CLOSED;
  }
}

class Info {
  labelToTranslate: string;
  value?: string;
  code?: boolean = false;
  isList?: boolean = false;
  type?: TypeInfoEnum;
}

enum TypeInfoEnum {
  doi = 1,
  orgUnit = 2,
  valueToTranslate = 3,
}
