import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  HostBinding,
  OnInit,
} from "@angular/core";
import {
  ActivatedRoute,
  Router,
} from "@angular/router";
import {SharedAbstractRoutable} from "@app/shared/components/routables/shared-abstract/shared-abstract.routable";
import {
  AppRoutesEnum,
  HomePageRoutesEnum,
  RoutesEnum,
} from "@app/shared/enums/routes.enum";
import {ArchiveMetadata} from "@app/shared/models/business/archive-metadata.model";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {DlcmEnvironment} from "@environments/environment.defaults.model";
import {HomeOrgUnitArchiveAction} from "@home/stores/organizational-unit/archive/home-organizational-unit-archive.action";
import {HomeOrgUnitAction} from "@home/stores/organizational-unit/home-organizational-unit.action";
import {Navigate} from "@ngxs/router-plugin";
import {
  Select,
  Store,
} from "@ngxs/store";
import {environment} from "environments/environment";
import {
  combineLatest,
  Observable,
} from "rxjs";
import {
  distinctUntilChanged,
  tap,
} from "rxjs/operators";
import {
  isEmptyArray,
  Paging,
  QueryParameters,
  StoreUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-home-browsing-list-archives-routable",
  templateUrl: "./home-browsing-list-archives.routable.html",
  styleUrls: ["./home-browsing-list-archives.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class HomeBrowsingListArchivesRoutable extends SharedAbstractRoutable implements OnInit {
  @Select((state: LocalStateModel) => state.home.home_organizationalUnit.home_organizationalUnit_archive.selected) listObs: Observable<ArchiveMetadata[]>;
  @Select((state: LocalStateModel) => StoreUtil.isLoadingState(state.home.home_organizationalUnit.home_organizationalUnit_archive)) isLoadingObs: Observable<boolean>;
  @Select((state: LocalStateModel) => StoreUtil.isLoadingState(state.home.home_organizationalUnit)) isLoadingOrgUnitObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state.home.home_organizationalUnit.home_organizationalUnit_archive.queryParameters) queryParametersObs: Observable<QueryParameters>;

  @HostBinding("class.have-no-items") haveNoItems: boolean = true;
  orgUnitId: string;
  isArchiveDetail: boolean | undefined = undefined;

  get environment(): DlcmEnvironment {
    return environment;
  }

  constructor(private _activatedRoute: ActivatedRoute,
              private _route: Router,
              private _store: Store,
              private _cd: ChangeDetectorRef) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();

    this.subscribe(this._activatedRoute.url,
      (url) => {
        this.isArchiveDetail = !isEmptyArray(this._activatedRoute.snapshot.children);
        this.orgUnitId = this._activatedRoute.snapshot.paramMap.get(AppRoutesEnum.paramIdWithoutPrefixParam);
        this._store.dispatch(new HomeOrgUnitAction.GetById(this.orgUnitId));
      });

    this.subscribe(combineLatest([this.listObs, this.isLoadingObs]).pipe(
      distinctUntilChanged(),
      tap(([list, isLoading]) => {
        if (isLoading) {
          this.haveNoItems = true;
          return;
        }
        this.haveNoItems = isEmptyArray(list);
      }),
    ));
  }

  navigateDetail(archive: ArchiveMetadata): void {
    this._store.dispatch(new Navigate([AppRoutesEnum.homePage, HomePageRoutesEnum.browsing, this.orgUnitId, HomePageRoutesEnum.detail, archive.resId]));
  }

  pageChange($event: Paging): void {
    const queryParameters = new QueryParameters();
    queryParameters.paging = $event;
    this._store.dispatch(new HomeOrgUnitArchiveAction.ChangeQueryParameters(this.orgUnitId, queryParameters, false));
  }

  close(): void {
    this._store.dispatch(new Navigate([RoutesEnum.homePageBrowsing]));
  }
}
