import {
  ChangeDetectionStrategy,
  Component,
  OnDestroy,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {
  ActivatedRoute,
  UrlSegment,
} from "@angular/router";
import {SearchScopeEnum} from "@app/features/home/enums/search-scope.enum";
import {HomeAction} from "@app/features/home/stores/home.action";
import {HomeState} from "@app/features/home/stores/home.state";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {ArchiveMetadata} from "@app/shared/models/business/archive-metadata.model";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {AppAction} from "@app/stores/app.action";
import {AppCartAction} from "@app/stores/cart/app-cart.action";
import {HomeBrowsingListArchivesRoutable} from "@home/components/routables/home-browsing-list-archives/home-browsing-list-archives.routable";
import {HomeHelper} from "@home/helpers/home.helper";
import {HomeService} from "@home/services/home.service";
import {AccessOrganizationalUnit} from "@models";
import {
  Select,
  Store,
} from "@ngxs/store";
import {GetShortDoiWrapper} from "@shared/components/presentationals/shared-doi-menu/shared-doi-menu.presentational";
import {AppRoutesEnum} from "@shared/enums/routes.enum";
import {Observable} from "rxjs";
import {
  isNullOrUndefined,
  MemoizedUtil,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-home-detail-routable",
  templateUrl: "./home-detail.routable.html",
  styleUrls: ["./home-detail.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class HomeDetailRoutable extends SharedAbstractPresentational implements OnInit, OnDestroy {
  isLoadingPrepareDownloadObs: Observable<boolean>;
  @Select(HomeState.isLoading) isLoadingObs: Observable<boolean>;
  currentObs: Observable<ArchiveMetadata> = MemoizedUtil.select(this._store, HomeState, state => state.current);
  @Select(HomeState.searchScope) searchScopeObs: Observable<SearchScopeEnum>;
  @Select(HomeState) stateObs: AccessOrganizationalUnit;
  @Select((state: LocalStateModel) => state.application.isLoggedIn) isLoggedInObs: Observable<boolean>;

  isInHomeBrowsing: boolean | undefined = undefined;
  private _resId: string;

  constructor(private readonly _store: Store,
              private readonly _dialog: MatDialog,
              private readonly _homeService: HomeService,
              protected route: ActivatedRoute) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();
    this._retrieveCurrentModelWithUrl();
    this._initIsLoadingPrepareDownloadObs();
    this._computeIsInHomeBrowsing();
  }

  ngOnDestroy(): void {
    this._store.dispatch(new HomeAction.CleanCurrent());
  }

  private _retrieveCurrentModelWithUrl(): void {
    this._resId = this.route.snapshot.paramMap.get(AppRoutesEnum.paramIdWithoutPrefixParam);
    this.getById(this._resId);
  }

  private getById(id: string): void {
    this._store.dispatch(new HomeAction.SearchDetail(id));
  }

  private _initIsLoadingPrepareDownloadObs(): void {
    this.isLoadingPrepareDownloadObs = HomeState.isDownloadPreparation(this._store, this._resId);
  }

  download(archive: ArchiveMetadata): void {
    this._store.dispatch(new HomeAction.Download(archive));
  }

  addToCart(archive: ArchiveMetadata): void {
    this._store.dispatch(new AppCartAction.AddToCart(archive));
  }

  askAccess(archive: ArchiveMetadata): void {
    this._homeService.requestToBeMember(archive);
  }

  back(): void {
    const urlSegmentToParent: UrlSegment[] = [];
    this.route.snapshot.parent.pathFromRoot.forEach((activatedRouteSnap) => {
      urlSegmentToParent.push(...activatedRouteSnap.url);
    });
    let search = undefined;
    if (!this.isInHomeBrowsing) {
      search = this._store.selectSnapshot(HomeState.search);
    }
    HomeHelper.navigateToSearch(this._store, search, urlSegmentToParent.map(u => u.path));
  }

  private _computeIsInHomeBrowsing(): boolean {
    this.isInHomeBrowsing = !isNullOrUndefined(this.route.snapshot.parent.component) && this.route.snapshot.parent.component["name"] === HomeBrowsingListArchivesRoutable.name;
    return this.isInHomeBrowsing;
  }

  getMessageBackButtonToTranslate(): string {
    if (this.isInHomeBrowsing) {
      return MARK_AS_TRANSLATABLE("home.archive.navigation.backToHomeBrowsing");
    }
    return MARK_AS_TRANSLATABLE("home.archive.navigation.backToSearch");
  }

  getShortDoi(doiWrapper: GetShortDoiWrapper): void {
    this._store.dispatch(new AppAction.GetShortDoi(doiWrapper));
  }
}
