import {
  ChangeDetectionStrategy,
  Component,
  OnDestroy,
  OnInit,
} from "@angular/core";
import {
  ActivatedRoute,
  Params,
} from "@angular/router";
import {SearchScopeEnum} from "@app/features/home/enums/search-scope.enum";
import {Archive} from "@app/features/home/models/archive.model";
import {HomeAction} from "@app/features/home/stores/home.action";
import {HomeState} from "@app/features/home/stores/home.state";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {FieldTypeEnum} from "@app/shared/enums/field-type.enum";
import {RoutesEnum} from "@app/shared/enums/routes.enum";
import {ArchiveMetadata} from "@app/shared/models/business/archive-metadata.model";
import {DataTableColumns} from "@app/shared/models/data-table-columns.model";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {AppState} from "@app/stores/app.state";
import {AppCartAction} from "@app/stores/cart/app-cart.action";
import {HomeHelper} from "@home/helpers/home.helper";
import {HomeService} from "@home/services/home.service";
import {Deposit} from "@models";
import {TranslateService} from "@ngx-translate/core";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {AccessLevelEnumHelper} from "@shared/enums/business/access-level-enum.helper";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {ThemeEnum} from "@shared/enums/theme.enum";
import {DataTableActions} from "@shared/models/data-table-actions.model";
import {MetadataUtil} from "@shared/utils/metadata.util";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {tap} from "rxjs/internal/operators/tap";
import {distinctUntilChanged} from "rxjs/operators";
import {
  isEmptyArray,
  isNullOrUndefined,
  MARK_AS_TRANSLATABLE,
  MemoizedUtil,
  OrderEnum,
  QueryParameters,
  StringUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-home-search-routable",
  templateUrl: "./home-search.routable.html",
  styleUrls: ["./home-search.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class HomeSearchRoutable extends SharedAbstractPresentational implements OnInit, OnDestroy {
  isLoggedInObs: Observable<boolean> = MemoizedUtil.select(this.store, AppState, state => state.isLoggedIn);
  searchObs: Observable<string> = MemoizedUtil.select(this.store, HomeState, state => state.search);
  queryParametersObs: Observable<QueryParameters> = MemoizedUtil.select(this.store, HomeState, state => state.queryParameters);
  listArchiveMetadataObs: Observable<ArchiveMetadata[]> = MemoizedUtil.select(this.store, HomeState, state => state.list);
  searchScopeObs: Observable<SearchScopeEnum> = MemoizedUtil.select(this.store, HomeState, state => state.searchScope);
  isLoadingObs: Observable<boolean> = MemoizedUtil.isLoading(this.store, HomeState);
  themeObs: Observable<ThemeEnum> = MemoizedUtil.select(this.store, AppState, state => state.theme);

  listAdaptedBS: BehaviorSubject<Archive[]> = new BehaviorSubject([]);
  listAdaptedObs: Observable<Archive[]> = this.listAdaptedBS.asObservable();

  columns: DataTableColumns[] = [
    {
      field: "organizationalUnitId",
      header: MARK_AS_TRANSLATABLE("access.table.header.organizationalUnit"),
      type: FieldTypeEnum.string,
      order: OrderEnum.none,
      component: DataTableComponentEnum.organizationalUnitName,
      isSortable: false,
      isFilterable: false,
    },
    {
      field: "title",
      header: MARK_AS_TRANSLATABLE("access.table.header.title"),
      type: FieldTypeEnum.string,
      order: OrderEnum.none,
      isSortable: false,
      isFilterable: false,
    },
    {
      field: "yearPublicationDate",
      header: MARK_AS_TRANSLATABLE("access.table.header.yearPublicationDate"),
      type: FieldTypeEnum.number,
      order: OrderEnum.none,
      isSortable: false,
      isFilterable: false,
      width: "100px",
    },
    {
      field: "accessLevel",
      header: MARK_AS_TRANSLATABLE("access.table.header.accessLevel"),
      type: FieldTypeEnum.string,
      order: OrderEnum.none,
      filterEnum: AccessLevelEnumHelper.getListKeyValue(),
      translate: true,
      isSortable: false,
      isFilterable: false,
      width: "100px",
    },
    {
      field: "size",
      header: MARK_AS_TRANSLATABLE("access.table.header.size"),
      type: FieldTypeEnum.size,
      order: OrderEnum.none,
      isSortable: false,
      isFilterable: false,
      width: "100px",
    },
    {
      field: "files",
      header: MARK_AS_TRANSLATABLE("access.table.header.files"),
      type: FieldTypeEnum.number,
      order: OrderEnum.none,
      isSortable: false,
      isFilterable: false,
      width: "100px",
    },
  ];

  actions: DataTableActions<Archive>[] = [
    {
      logo: IconNameEnum.addToCart,
      callback: current => this.addToCart(current.archiveMetadata),
      placeholder: MARK_AS_TRANSLATABLE("home.list.action.addToCart"),
      displayOnCondition: current => MemoizedUtil.selectSnapshot(this.store, AppState, state => state.isLoggedIn),
    },
    {
      logo: IconNameEnum.download,
      callback: current => this.download(current.archiveMetadata),
      placeholder: MARK_AS_TRANSLATABLE("home.list.action.download"),
      displayOnCondition: current => this.store.selectSnapshot((state: LocalStateModel) => state.application.isLoggedIn) || current.accessLevel === Deposit.AccessEnum.PUBLIC,
    },
    {
      logo: IconNameEnum.sendRequest,
      callback: current => this._homeService.requestToBeMember(current.archiveMetadata),
      placeholder: MARK_AS_TRANSLATABLE("home.list.action.askAccess"),
      displayOnCondition: current => this.store.selectSnapshot((state: LocalStateModel) => state.application.isLoggedIn) && (current.accessLevel === Deposit.AccessEnum.RESTRICTED || current.accessLevel === Deposit.AccessEnum.CLOSED),
    },
  ];

  searchValueInUrl: string = StringUtil.stringEmpty;

  constructor(private store: Store,
              private translate: TranslateService,
              private readonly _route: ActivatedRoute,
              private readonly _actions$: Actions,
              private readonly _homeService: HomeService) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.subscribe(this.observeSearchQueryParam());
    this.subscribe(this.transformListArchive());
  }

  private transformListArchive(): Observable<ArchiveMetadata[]> {
    return this.listArchiveMetadataObs
      .pipe(
        distinctUntilChanged(),
        tap((listArchives) => {
          if (isNullOrUndefined(listArchives) || isEmptyArray(listArchives)) {
            this.listAdaptedBS.next([]);
            return;
          }
          const searchScope = this.store.selectSnapshot(HomeState.searchScope);
          const newList: Archive[] = [];
          listArchives.forEach((archive: ArchiveMetadata) => {
            newList.push({
              resId: archive.resId,
              organizationalUnitId: MetadataUtil.getOrganizationalUnitResId(archive.metadata),
              title: MetadataUtil.getTitle(searchScope, archive.metadata),
              yearPublicationDate: MetadataUtil.getPublicationYear(searchScope, archive.metadata),
              accessLevel: MetadataUtil.getAccessLevel(searchScope, archive.metadata) as Deposit.AccessEnum,
              size: "" + MetadataUtil.getSize(archive.metadata),
              files: MetadataUtil.getFileNumber(archive.metadata),
              archiveMetadata: archive,
            });
          });
          this.listAdaptedBS.next(newList);
        }),
      );
  }

  private observeSearchQueryParam(): Observable<Params> {
    return this._route.queryParams.pipe(tap(param => {
      let searchTerm = StringUtil.stringEmpty;
      const queryParamSearch = param[HomeHelper.SEARCH_QUERY_PARAM];
      if (!isNullOrUndefined(queryParamSearch)) {
        searchTerm = queryParamSearch;
      }
      this.searchValueInUrl = searchTerm;
      this.store.dispatch(new HomeAction.Search(true, searchTerm));
    }));
  }

  search(searchTerm: string): void {
    HomeHelper.navigateToSearch(this.store, searchTerm);
  }

  onQueryParametersEvent($event: QueryParameters): void {
    this.store.dispatch(new HomeAction.ChangeQueryParameters($event));
  }

  showDetail(archive: Archive): void {
    this.store.dispatch(new Navigate([RoutesEnum.homePageDetail, archive.resId]));
  }

  searchScopeChange(searchScope: SearchScopeEnum): void {
    this.store.dispatch(new HomeAction.ChangeSearchScope(searchScope));
  }

  goToArchivalBrowsing(): void {
    this.store.dispatch(new Navigate([RoutesEnum.homePageBrowsing]));
  }

  ngOnDestroy(): void {
    super.ngOnDestroy();
  }

  download(archive: ArchiveMetadata): void {
    this.store.dispatch(new HomeAction.Download(archive));
  }

  addToCart(archive: ArchiveMetadata): void {
    this.store.dispatch(new AppCartAction.AddToCart(archive));
  }

  navigate($event: RoutesEnum): void {
    this.store.dispatch(new Navigate([$event]));
  }
}
