import {
  HttpClient,
  HttpHeaders,
  HttpResponse,
} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {SearchScopeEnum} from "@app/features/home/enums/search-scope.enum";
import {Rss} from "@app/features/home/models/rss.model";
import {HomeAction} from "@app/features/home/stores/home.action";
import {
  HomeOrganizationalUnitState,
  HomeOrganizationalUnitStateModel,
} from "@app/features/home/stores/organizational-unit/home-organizational-unit.state";
import {AccessResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {urlSeparator} from "@app/shared/enums/routes.enum";
import {SessionStorageEnum} from "@app/shared/enums/session-storage.enum";
import {
  ArchiveMetadata,
  MetadataEnum,
} from "@app/shared/models/business/archive-metadata.model";
import {UserPreferencesUtil} from "@app/shared/utils/user-preferences.util";
import {AppStateModel} from "@app/stores/app.state";
import {environment} from "@environments/environment";
import {Order} from "@models";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiActionEnum} from "@shared/enums/api-action.enum";
import {DownloadService} from "@shared/services/download.service";
import {MetadataUtil} from "@shared/utils/metadata.util";
import {NgxXml2jsonService} from "ngx-xml2json";
import {
  interval,
  Observable,
  of,
} from "rxjs";
import {tap} from "rxjs/internal/operators/tap";
import {
  catchError,
  filter,
  map,
  mergeMap,
  startWith,
  takeWhile,
} from "rxjs/operators";
import {
  ApiService,
  BasicState,
  CollectionTyped,
  defaultResourceStateInitValue,
  ErrorDto,
  HttpStatus,
  isNullOrUndefined,
  isWhiteString,
  MappingObjectUtil,
  MemoizedUtil,
  NotificationService,
  QueryParameters,
  QueryParametersUtil,
  ResourceStateModel,
  StoreUtil,
  StringUtil,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

export interface HomeStateModel extends ResourceStateModel<ArchiveMetadata> {
  search: string;
  searchScope: SearchScopeEnum;
  rss: Rss | undefined;
  home_organizationalUnit: HomeOrganizationalUnitStateModel;
  isPooling: string[];
  isPendingRequest: string[];
  isFirstTimeError: string[];
}

@Injectable()
@State<HomeStateModel>({
  name: LocalStateEnum.home,
  defaults: {
    ...defaultResourceStateInitValue(),
    searchScope: UserPreferencesUtil.getPreferredHomePageSearchScope(),
    search: null,
    queryParameters: new QueryParameters(environment.defaultPageSizeHomePage),
    rss: undefined,
    home_organizationalUnit: {isLoadingCounter: 0} as HomeOrganizationalUnitStateModel,
    isPooling: [],
    isPendingRequest: [],
    isFirstTimeError: [],
  },
  children: [
    HomeOrganizationalUnitState,
  ],
})
export class HomeState extends BasicState<HomeStateModel> {
  protected _urlResource: string;

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected httpClient: HttpClient,
              protected notificationService: NotificationService,
              private ngxXml2jsonService: NgxXml2jsonService,
              protected actions$: Actions,
              private downloadService: DownloadService) {
    super();
  }

  private readonly DEFAULT_SEARCH: string = "*";
  private readonly IS_POOLING_KEY: keyof HomeStateModel = "isPooling";
  private readonly IS_PENDING_REQUEST_KEY: keyof HomeStateModel = "isPendingRequest";
  private readonly IS_FIRST_TIME_ERROR_KEY: keyof HomeStateModel = "isFirstTimeError";

  @Selector()
  static current(state: HomeStateModel): ArchiveMetadata {
    return state.current;
  }

  @Selector()
  static searchScope(state: HomeStateModel): SearchScopeEnum {
    return state.searchScope;
  }

  @Selector()
  static search(state: HomeStateModel): string {
    return state.search;
  }

  @Selector()
  static isLoading(state: HomeStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  static isDownloadPreparation(store: Store, archiveResId: string): Observable<boolean> {
    return MemoizedUtil.select(store, HomeState, state => state.isPooling).pipe(
      map(list => list.includes(archiveResId)),
    );
  }

  @Selector()
  static currentTitle(state: HomeStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return MetadataUtil.getTitle(SearchScopeEnum.public, state.current.metadata);
  }

  @Action(HomeAction.ChangeQueryParameters)
  changeQueryParameters(ctx: StateContext<HomeStateModel>, action: HomeAction.ChangeQueryParameters): void {
    ctx.patchState({
      queryParameters: action.queryParameters,
    });
    ctx.dispatch(new HomeAction.Search(false, ctx.getState().search));
  }

  @Action(HomeAction.ChangeSearchScope)
  changeSearchScope(ctx: StateContext<HomeStateModel>, action: HomeAction.ChangeSearchScope): void {
    ctx.patchState({
      searchScope: action.searchScope,
    });
    sessionStorage.setItem(SessionStorageEnum.homePageSearchScope, action.searchScope);
    const search = ctx.getState().search;
    if (!isWhiteString(search) && !isNullOrUndefined(search)) {
      ctx.dispatch(new HomeAction.Search(true, ctx.getState().search));
    }
  }

  @Action(HomeAction.Search)
  search(ctx: StateContext<HomeStateModel>, action: HomeAction.Search): Observable<CollectionTyped<ArchiveMetadata>> {
    const searchString = this.getSearchStringToApply(ctx, action);
    let queryParameters = StoreUtil.getQueryParametersToApply(action.queryParameters, ctx);
    if (action.resetPagination) {
      queryParameters = QueryParametersUtil.resetToFirstPage(queryParameters);
    }
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
      search: searchString,
      queryParameters: queryParameters,
    });

    const pathFilter = "query";
    queryParameters = QueryParametersUtil.clone(queryParameters);
    MappingObjectUtil.set(QueryParametersUtil.getSearchItems(queryParameters), pathFilter, searchString);

    if (searchString === StringUtil.stringEmpty) {
      ctx.dispatch(new HomeAction.SearchSuccess(null));
    } else {
      const urlMetadata = this.getUrlMetadata(ctx);
      return this.apiService.get<ArchiveMetadata>(urlMetadata + urlSeparator + "search", queryParameters)
        .pipe(
          StoreUtil.cancelUncompleted(ctx, this.actions$, [HomeAction.Search]),
          tap(collection => ctx.dispatch(new HomeAction.SearchSuccess(collection))),
          catchError(error => {
            ctx.dispatch(new HomeAction.SearchFail());
            throw error;
          }),
        );
    }
  }

  private getUrlMetadata(ctx: StateContext<HomeStateModel>): string {
    const isLoggedIn: boolean = this.store.selectSnapshot<boolean>(state => (state.application as AppStateModel).isLoggedIn);
    if (!isLoggedIn) {
      return AccessResourceApiEnum.publicMetadata;
    }
    switch (ctx.getState().searchScope) {
      case SearchScopeEnum.all:
      // return AccessResourceApiEnum.allMetadata; // TO DISCUSS : All metadata endpoint display public + private metadata = we have all entry duplicate in datatable
      case SearchScopeEnum.private:
        return AccessResourceApiEnum.privateMetadata;
      case SearchScopeEnum.public:
      default:
        return AccessResourceApiEnum.publicMetadata;
    }
  }

  private getSearchStringToApply(ctx: StateContext<HomeStateModel>, action: HomeAction.Search): string {
    if (isNullOrUndefined(action.search)) {
      return this.getDefaultSearchStringIfNotDefined(ctx);
    }
    return action.search;
  }

  private getDefaultSearchStringIfNotDefined(ctx: StateContext<HomeStateModel>): string {
    if (ctx.getState().search === null || ctx.getState().search === undefined) {
      return this.DEFAULT_SEARCH;
    }
    return ctx.getState().search;
  }

  @Action(HomeAction.SearchSuccess)
  searchSuccess(ctx: StateContext<HomeStateModel>, action: HomeAction.SearchSuccess): void {
    const queryParameters = StoreUtil.updateQueryParameters(ctx, action.list);

    ctx.patchState({
      list: isNullOrUndefined(action.list) ? [] : action.list._data,
      total: isNullOrUndefined(action.list) ? 0 : action.list._page.totalItems,
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
      queryParameters,
    });
  }

  @Action(HomeAction.SearchFail)
  searchFail(ctx: StateContext<HomeStateModel>): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(HomeAction.SearchDetail)
  detail(ctx: StateContext<HomeStateModel>, action: HomeAction.SearchDetail): Observable<ArchiveMetadata> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return this.apiService.getById<ArchiveMetadata>(this.getUrlMetadata(ctx), action.resId)
      .pipe(
        tap(model => ctx.dispatch(new HomeAction.SearchDetailSuccess(model))),
        catchError(error => {
          ctx.dispatch(new HomeAction.SearchDetailFail());
          throw error;
        }),
      );
  }

  @Action(HomeAction.SearchDetailSuccess)
  detailSuccess(ctx: StateContext<HomeStateModel>, action: HomeAction.SearchDetailSuccess): void {
    ctx.patchState({
      current: action.model,
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(HomeAction.GetRss)
  getRss(ctx: StateContext<HomeStateModel>, action: HomeAction.GetRss): Observable<Rss> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    let headers = new HttpHeaders();
    headers = headers.set("Accept", ["application/xml"]);
    headers = headers.set("Content-Type", "text/xml");

    return this.httpClient.get("./api/rss", {
      headers,
      observe: "response",
      responseType: "text",
    }).pipe(
      map((r: HttpResponse<string>) => {
        const parser = new DOMParser();
        const xml = parser.parseFromString(r.body, "text/xml");
        const obj = this.ngxXml2jsonService.xmlToJson(xml) as { rss: Rss };
        ctx.dispatch(new HomeAction.GetRssSuccess(obj.rss));
        return obj.rss;
      }),
    );
  }

  @Action(HomeAction.GetRssSuccess)
  getRssSuccess(ctx: StateContext<HomeStateModel>, action: HomeAction.GetRssSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
      rss: action.rss,
    });
  }

  @Action(HomeAction.GetRssFail)
  getRssFail(ctx: StateContext<HomeStateModel>, action: HomeAction.GetRssFail): void {
    this.notificationService.showError(MARK_AS_TRANSLATABLE("welcome.rss.notification.error"));
  }

  @Action(HomeAction.Download)
  download(ctx: StateContext<HomeStateModel>, action: HomeAction.Download): any {
    this.notificationService.showInformation(MARK_AS_TRANSLATABLE("home.archive.browsing.download.notification.inPreparation"), true);

    this._setIsPooling(ctx, action, true);
    this._setIsPendingRequest(ctx, action, false);
    this._setIsFirstTimeError(ctx, action, false);

    return interval(2000)
      .pipe(
        takeWhile(() => this._getIsPooling(ctx, action)),
        filter(() => !this._getIsPendingRequest(ctx, action)),
        startWith(0),
        mergeMap(() => {
          this._setIsPendingRequest(ctx, action, true);
          return this._getDownloadStatus(ctx, action.archiveMetadata.resId).pipe(
            tap(() => this._setIsPendingRequest(ctx, action, false)),
          );
        }),
        tap((res: string) => {
          if (res === StringUtil.stringEmpty) {
            this._prepareDownload(ctx, action.archiveMetadata.resId);
            return;
          }
          if (res === Order.StatusEnum.READY) {
            this._setIsPooling(ctx, action, false);
            ctx.dispatch(new HomeAction.DownloadStart(action));
            return;
          }
          if (res === Order.StatusEnum.INERROR) {
            this._setIsPooling(ctx, action, false);
            if (this._getIsFirstTimeError(ctx, action) === false) {
              ctx.dispatch(new HomeAction.DownloadFail(action));
              this._setIsFirstTimeError(ctx, action, true);
            }
            return;
          }
        }),
        catchError(error => {
          this._setIsPooling(ctx, action, false);
          throw error;
        }),
      );
  }

  private _getDownloadStatus(ctx: StateContext<HomeStateModel>, resId: string): Observable<string> {
    let headers = new HttpHeaders();
    headers = headers.set("Content-Type", "text/plain;charset=UTF-8");

    return this.httpClient.get(this.getUrlMetadata(ctx) + urlSeparator + resId
      + urlSeparator + ApiActionEnum.DOWNLOAD_STATUS, {
      headers,
      responseType: "text",
    }).pipe(
      catchError((e: ErrorDto) => {
        if (e.status === HttpStatus.INTERNAL_SERVER_ERROR) {
          return of(Order.StatusEnum.INERROR);
        }
        return of(StringUtil.stringEmpty);
      }),
    );
  }

  private _prepareDownload(ctx: StateContext<HomeStateModel>, resId: string): void {
    let headers = new HttpHeaders();
    headers = headers.set("Content-Type", "text/plain;charset=UTF-8");
    this.httpClient.post(this.getUrlMetadata(ctx) + urlSeparator + resId
      + urlSeparator + ApiActionEnum.PREPARE_DOWNLOAD, {headers, responseType: "text"})
      .subscribe();
  }

  @Action(HomeAction.DownloadStart)
  downloadStart(ctx: StateContext<HomeStateModel>, action: HomeAction.DownloadStart): void {
    const archiveMetadata = (action.parentAction as HomeAction.Download).archiveMetadata;
    const fileName = MetadataUtil.getTitle(ctx.getState().searchScope, archiveMetadata.metadata) + ".zip";
    const fileSize = archiveMetadata.metadata[MetadataEnum.aipSize];

    this.downloadService.download(`${this.getUrlMetadata(ctx)}/${archiveMetadata.resId}/${ApiActionEnum.DL}`, fileName, fileSize)
      .subscribe(() => {
        ctx.dispatch(new HomeAction.DownloadSuccess(action));
      });
  }

  @Action(HomeAction.DownloadSuccess)
  downloadSuccess(ctx: StateContext<HomeStateModel>, action: HomeAction.DownloadSuccess): void {
    this.notificationService.showSuccess(MARK_AS_TRANSLATABLE("home.archive.browsing.download.notification.success"));
  }

  @Action(HomeAction.DownloadFail)
  downloadFail(ctx: StateContext<HomeStateModel>, action: HomeAction.DownloadFail): void {
    this.notificationService.showError(MARK_AS_TRANSLATABLE("home.archive.browsing.download.notification.error"));
  }

  @Action(HomeAction.CleanCurrent)
  cleanCurrent(ctx: StateContext<HomeStateModel>, action: HomeAction.CleanCurrent): void {
    ctx.patchState({
      current: undefined,
    });
  }

  private _getIsPooling(ctx: StateContext<HomeStateModel>, action: HomeAction.Download): boolean {
    return this._isIncludedInList(ctx, action, this.IS_POOLING_KEY);
  }

  private _setIsPooling(ctx: StateContext<HomeStateModel>, action: HomeAction.Download, isTrue: boolean): void {
    return this._setInList(ctx, action, this.IS_POOLING_KEY, isTrue);
  }

  private _getIsPendingRequest(ctx: StateContext<HomeStateModel>, action: HomeAction.Download): boolean {
    return this._isIncludedInList(ctx, action, this.IS_PENDING_REQUEST_KEY);
  }

  private _setIsPendingRequest(ctx: StateContext<HomeStateModel>, action: HomeAction.Download, isTrue: boolean): void {
    return this._setInList(ctx, action, this.IS_PENDING_REQUEST_KEY, isTrue);
  }

  private _getIsFirstTimeError(ctx: StateContext<HomeStateModel>, action: HomeAction.Download): boolean {
    return this._isIncludedInList(ctx, action, this.IS_FIRST_TIME_ERROR_KEY);
  }

  private _setIsFirstTimeError(ctx: StateContext<HomeStateModel>, action: HomeAction.Download, isTrue: boolean): void {
    return this._setInList(ctx, action, this.IS_FIRST_TIME_ERROR_KEY, isTrue);
  }

  private _isIncludedInList(ctx: StateContext<HomeStateModel>, action: HomeAction.Download, key: keyof HomeStateModel): boolean {
    return (ctx.getState()[key] as string[]).includes(action.archiveMetadata.resId);
  }

  private _setInList(ctx: StateContext<HomeStateModel>, action: HomeAction.Download, key: keyof HomeStateModel, isTrue: boolean): void {
    if (isTrue) {
      this._addInList(ctx, action, key);
    } else {
      this._removeInList(ctx, action, key);
    }
  }

  private _addInList(ctx: StateContext<HomeStateModel>, action: HomeAction.Download, key: keyof HomeStateModel): void {
    const list = ctx.getState()[key] as string[];
    if (!list.includes(action.archiveMetadata.resId)) {
      ctx.patchState({
        [key]: [...list, action.archiveMetadata.resId],
      });
    }
  }

  private _removeInList(ctx: StateContext<HomeStateModel>, action: HomeAction.Download, key: keyof HomeStateModel): void {
    const list = ctx.getState()[key] as string[];
    const archiveId = action.archiveMetadata.resId;
    const indexOf = list.indexOf(archiveId);
    if (indexOf !== -1) {
      const listPending = [...list];
      listPending.splice(indexOf, 1);
      ctx.patchState({
        [key]: listPending,
      });
    }
  }
}

