import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {AppState} from "@app/stores/app.state";
import {Order} from "@models";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {OrderStatusEnumHelper} from "@order/enums/order-status.enum";
import {orderMyOrderActionNameSpace} from "@order/features/my-order/stores/order-my-order.action";
import {OrderMyOrderStateModel} from "@order/features/my-order/stores/order-my-order.state";
import {OrderQueryTypeEnumHelper} from "@order/helpers/order-query-type-enum.helper";
import {SharedAbstractListRoutable} from "@shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {SessionStorageEnum} from "@shared/enums/session-storage.enum";
import {SessionStorageHelper} from "@shared/helpers/session-storage.helper";
import {RouterExtService} from "@shared/services/router-ext.service";
import {Observable} from "rxjs";
import {
  MappingObjectUtil,
  ObjectUtil,
  OrderEnum,
  OverrideProperty,
  QueryParameters,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-order-my-order-list-routable",
  templateUrl: "../../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class OrderMyOrderListRoutable extends SharedAbstractListRoutable<Order, OrderMyOrderStateModel> implements OnInit {
  readonly KEY_CREATE_BUTTON: string = undefined;
  readonly KEY_REFRESH_BUTTON: string = MARK_AS_TRANSLATABLE("order.myOrder.refresh");
  readonly KEY_BACK_BUTTON: string = MARK_AS_TRANSLATABLE("order.myOrder.back");
  readonly KEY_PARAM_NAME: keyof Order & string = "name";
  private readonly KEY_CREATOR: string = "creation.who";

  @Select(AppState.currentUser) currentUserApplicationRoleObs: Observable<string[]>;

  columnsSkippedToClear: string[] = [this.KEY_CREATOR];

  @OverrideProperty()
  listNewId: string[] = SessionStorageHelper.getListItem(SessionStorageEnum.newOrderAvailable);

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, LocalStateEnum.order_myOrder, orderMyOrderActionNameSpace, {
      canCreate: false,
    }, LocalStateEnum.order);
  }

  conditionDisplayEditButton(model: Order | undefined): boolean {
    return model.status !== Order.StatusEnum.READY;
  }

  conditionDisplayDeleteButton(model: Order | undefined): boolean {
    return true;
  }

  ngOnInit(): void {
    super.ngOnInit();
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "queryType",
        header: MARK_AS_TRANSLATABLE("order.myOrder.table.header.queryType"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        translate: true,
        filterEnum: OrderQueryTypeEnumHelper.getListKeyValue(),
      },
      {
        field: "name",
        header: MARK_AS_TRANSLATABLE("order.myOrder.table.header.name"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "status",
        header: MARK_AS_TRANSLATABLE("order.myOrder.table.header.status"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        translate: true,
        filterEnum: OrderStatusEnumHelper.getListKeyValue(),
      },
      {
        field: "creation.when" as any,
        header: MARK_AS_TRANSLATABLE("order.myOrder.table.header.creation.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "lastUpdate.when" as any,
        header: MARK_AS_TRANSLATABLE("order.myOrder.table.header.lastUpdate.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.descending,
        isFilterable: true,
        isSortable: true,
      },
    ];
  }

  private updateQueryParameterWithCurrentUser(queryParameters: QueryParameters): QueryParameters | undefined {
    queryParameters = ObjectUtil.clone(queryParameters);
    queryParameters.search = ObjectUtil.clone(queryParameters.search);
    const userTargetedUid = this._store.selectSnapshot(AppState.currentUser).externalUid;
    MappingObjectUtil.set(queryParameters.search.searchItems, this.KEY_CREATOR, userTargetedUid);
    return queryParameters;
  }

  onQueryParametersEvent(queryParameters: QueryParameters): void {
    this.updateQueryParameterWithCurrentUser(queryParameters);
    super.onQueryParametersEvent(queryParameters);
  }
}
