import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {NotificationModeEnum} from "@app/features/preservation-space/notification/enums/notification-mode.enum";
import {NotificationHelper} from "@app/features/preservation-space/notification/helper/notification.helper";
import {
  PreservationSpaceNotificationState,
  PreservationSpaceNotificationStateModel,
} from "@app/features/preservation-space/notification/stores/preservation-space-notification.state";
import {NotificationDlcm} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SharedAbstractListRoutable} from "@shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {NotificationStatusEnumHelper} from "@shared/enums/notification-status.enum";
import {NotificationTypeEnumHelper} from "@shared/enums/notification-type.enum";
import {
  PreservationSpaceRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {SessionStorageEnum} from "@shared/enums/session-storage.enum";
import {SessionStorageHelper} from "@shared/helpers/session-storage.helper";
import {RouterExtService} from "@shared/services/router-ext.service";
import {sharedOrgUnitActionNameSpace} from "@shared/stores/organizational-unit/shared-organizational-unit.action";
import {SharedOrganizationalUnitState} from "@shared/stores/organizational-unit/shared-organizational-unit.state";
import {Observable} from "rxjs";
import {
  isNullOrUndefined,
  MemoizedUtil,
  OrderEnum,
  Override,
  ResourceNameSpace,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";
import {
  PreservationSpaceNotificationAction,
  preservationSpaceNotificationActionNameSpace,
} from "../../../stores/preservation-space-notification.action";

@Component({
  selector: "dlcm-preservation-space-notification-list-routable",
  templateUrl: "./preservation-space-notification-list.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class PreservationSpaceNotificationListRoutable extends SharedAbstractListRoutable<NotificationDlcm, PreservationSpaceNotificationStateModel> implements OnInit {
  readonly KEY_CREATE_BUTTON: string = MARK_AS_TRANSLATABLE("preservationSpace.notifications.button.new");
  readonly KEY_REFRESH_BUTTON: string = MARK_AS_TRANSLATABLE("preservationSpace.notifications.button.refresh");
  readonly KEY_BACK_BUTTON: string | undefined = MARK_AS_TRANSLATABLE("preservationSpace.notifications.button.goBackToPreservationSpace");
  readonly KEY_PARAM_NAME: keyof NotificationDlcm & string = "notificationType";
  readonly KEY_QUERY_PARAMETER_NOTIFIED_ORGUNIT_ID: keyof NotificationDlcm | string = "notifiedOrgUnit.resId";

  orgUnitIdObs: Observable<string> = MemoizedUtil.select(this._store, PreservationSpaceNotificationState, state => state.orgUnitId);
  modeObs: Observable<NotificationModeEnum> = MemoizedUtil.select(this._store, PreservationSpaceNotificationState, state => state.mode);

  sharedOrgUnitActionNameSpace: ResourceNameSpace = sharedOrgUnitActionNameSpace;
  sharedOrganizationalUnitState: typeof SharedOrganizationalUnitState = SharedOrganizationalUnitState;

  get notificationModeEnum(): typeof NotificationModeEnum {
    return NotificationModeEnum;
  }

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, LocalStateEnum.preservationSpace_notification, preservationSpaceNotificationActionNameSpace, {
      canCreate: false,
      canGoBack: false,
      canRefresh: true,
    }, LocalStateEnum.preservationSpace);
  }

  ngOnInit(): void {
    this._retrieveModeFromUrl();
    super.ngOnInit();
  }

  private _retrieveModeFromUrl(): void {
    let orgUnitId = undefined;
    const queryParam = this._route.snapshot.queryParamMap;
    if (queryParam.has(NotificationHelper.KEY_ORGUNIT_ID)) {
      this.columnsSkippedToClear = [this.KEY_QUERY_PARAMETER_NOTIFIED_ORGUNIT_ID];
      orgUnitId = queryParam.get(NotificationHelper.KEY_ORGUNIT_ID);
    }
    const mode = NotificationHelper.getMode(this._route);
    if (mode === NotificationModeEnum.inbox) {
      this.listNewId = SessionStorageHelper.getListItem(SessionStorageEnum.notificationInboxPending);
    }
    this._store.dispatch(new PreservationSpaceNotificationAction.SetMode(mode, orgUnitId));
  }

  @Override()
  showDetail(model: NotificationDlcm): void {
    const url = this._route.snapshot.parent.url.toString();
    let route = RoutesEnum.preservationSpaceNotificationInboxDetail;
    if (url === PreservationSpaceRoutesEnum.notificationSent) {
      route = RoutesEnum.preservationSpaceNotificationSentDetail;
    }
    this._store.dispatch(new Navigate([route, model.resId]));
  }

  conditionDisplayEditButton(model: NotificationDlcm | undefined): boolean {
    return false;
  }

  conditionDisplayDeleteButton(model: NotificationDlcm | undefined): boolean {
    return false;
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "notificationType",
        header: MARK_AS_TRANSLATABLE("preservationSpace.notifications.table.header.notificationType"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        translate: true,
        filterEnum: NotificationTypeEnumHelper.getListKeyValue(),
      },
      {
        field: "notificationStatus",
        header: MARK_AS_TRANSLATABLE("preservationSpace.notifications.table.header.notificationStatus"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
        translate: true,
        filterEnum: NotificationStatusEnumHelper.getListKeyValue(),
      },
      {
        field: "emitter.person.fullName" as any,
        header: MARK_AS_TRANSLATABLE("preservationSpace.notifications.table.header.emitter"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: false,
      },
      {
        field: "creation.when" as any,
        header: MARK_AS_TRANSLATABLE("preservationSpace.notifications.table.header.creation.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: false,
        width: "145px",
      },
    ];

    const orgUnitId = MemoizedUtil.selectSnapshot(this._store, PreservationSpaceNotificationState, state => state.orgUnitId);
    if (isNullOrUndefined(orgUnitId)) {
      this.columns.push({
        field: "notifiedOrgUnit.name" as any,
        header: MARK_AS_TRANSLATABLE("preservationSpace.notifications.table.header.notifiedOrgUnit"),
        type: FieldTypeEnum.searchableSingleSelect,
        order: OrderEnum.none,
        filterableField: this.KEY_QUERY_PARAMETER_NOTIFIED_ORGUNIT_ID as any,
        sortableField: this.KEY_QUERY_PARAMETER_NOTIFIED_ORGUNIT_ID as any,
        resourceNameSpace: this.sharedOrgUnitActionNameSpace,
        resourceState: this.sharedOrganizationalUnitState as any,
        isFilterable: true,
        isSortable: true,
      });
    }
  }
}
