import {Injectable} from "@angular/core";
import {NotificationModeEnum} from "@app/features/preservation-space/notification/enums/notification-mode.enum";
import {NotificationHelper} from "@app/features/preservation-space/notification/helper/notification.helper";
import {
  PreservationSpaceNotificationAction,
  preservationSpaceNotificationActionNameSpace,
} from "@app/features/preservation-space/notification/stores/preservation-space-notification.action";
import {NotificationDlcm} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiActionEnum} from "@shared/enums/api-action.enum";
import {AdminResourceApiEnum} from "@shared/enums/api.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {Result} from "@shared/models/business/result.model";
import {
  Observable,
  pipe,
} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  CollectionTyped,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  MappingObjectUtil,
  NotificationService,
  OverrideDefaultAction,
  QueryParametersUtil,
  ResourceState,
  ResourceStateModel,
  SolidifyStateError,
  StoreUtil,
  MARK_AS_TRANSLATABLE,
  urlSeparator,
} from "solidify-frontend";
import {defaultStatusHistoryInitValue} from "@shared/stores/status-history/status-history.state";
import {
  PreservationSpaceNotificationStatusHistoryState,
  PreservationSpaceNotificationStatusHistoryStateModel,
} from "@app/features/preservation-space/notification/stores/status-history/preservation-space-notification-status-history.state";

export interface PreservationSpaceNotificationStateModel extends ResourceStateModel<NotificationDlcm> {
  mode: NotificationModeEnum;
  orgUnitId: string | undefined;
  preservationSpace_notification_statusHistory: PreservationSpaceNotificationStatusHistoryStateModel;
}

@Injectable()
@State<PreservationSpaceNotificationStateModel>({
  name: LocalStateEnum.preservationSpace_notification,
  defaults: {
    ...defaultResourceStateInitValue(),
    mode: NotificationModeEnum.inbox,
    orgUnitId: undefined,
    preservationSpace_notification_statusHistory: {...defaultStatusHistoryInitValue()},
  },
  children: [
    PreservationSpaceNotificationStatusHistoryState,
  ],
})
export class PreservationSpaceNotificationState extends ResourceState<PreservationSpaceNotificationStateModel, NotificationDlcm> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationSpaceNotificationActionNameSpace,
      routeRedirectUrlAfterSuccessCreateAction: (resId: string) => RoutesEnum.preservationSpaceNotificationInboxDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessUpdateAction: (resId: string) => RoutesEnum.preservationSpaceNotificationInboxDetail + urlSeparator + resId,
      notificationResourceCreateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("preservationSpace.notifications.notification.resource.create"),
      notificationResourceDeleteSuccessTextToTranslate: MARK_AS_TRANSLATABLE("preservationSpace.notifications.notification.resource.delete"),
      notificationResourceUpdateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("preservationSpace.notifications.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.notifications;
  }

  @Selector()
  static isLoading(state: PreservationSpaceNotificationStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: PreservationSpaceNotificationStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static currentTitle(state: PreservationSpaceNotificationStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return state.current.emitter["person"].fullName;
  }

  @Selector()
  static isReadyToBeDisplayed(state: PreservationSpaceNotificationStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: PreservationSpaceNotificationStateModel): boolean {
    return true;
  }

  @Action(PreservationSpaceNotificationAction.SetMode)
  setMode(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.SetMode): void {
    ctx.patchState({
      mode: action.mode,
      orgUnitId: action.orgUnitId,
    });
  }

  @OverrideDefaultAction()
  @Action(PreservationSpaceNotificationAction.GetAll)
  getAll(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.GetAll): Observable<CollectionTyped<NotificationDlcm>> {
    let reset = {};
    if (!action.keepCurrentContext) {
      reset = {
        list: undefined,
        total: 0,
      };
    }
    let queryParameters = StoreUtil.getQueryParametersToApply(action.queryParameters, ctx);
    const orgUnitId = ctx.getState().orgUnitId;
    if (!isNullOrUndefined(orgUnitId)) {
      queryParameters = QueryParametersUtil.clone(queryParameters);
      const search = QueryParametersUtil.getSearchItems(queryParameters);
      MappingObjectUtil.set(search, "notifiedOrgUnit.resId", orgUnitId);
    }
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
      queryParameters: queryParameters,
      ...reset,
    });
    let mode = ApiActionEnum.INBOX;
    if (ctx.getState().mode === NotificationModeEnum.sent) {
      mode = ApiActionEnum.SENT;
    }
    return this.apiService.get<NotificationDlcm>(this._urlResource + urlSeparator + mode, ctx.getState().queryParameters)
      .pipe(
        action.cancelIncomplete ? StoreUtil.cancelUncompleted(ctx, this.actions$, [this._nameSpace.GetAll, Navigate]) : pipe(),
        tap((collection: CollectionTyped<NotificationDlcm>) => {
          ctx.dispatch(new PreservationSpaceNotificationAction.GetAllSuccess(action, collection));
        }),
        catchError(error => {
          ctx.dispatch(new PreservationSpaceNotificationAction.GetAllFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @OverrideDefaultAction()
  @Action(PreservationSpaceNotificationAction.GetById)
  getById(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.GetById): Observable<NotificationDlcm> {
    let reset = {};
    if (!action.keepCurrentContext) {
      reset = {
        current: undefined,
      };
    }
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
      ...reset,
    });

    let mode = ApiActionEnum.INBOX;
    if (ctx.getState().mode === NotificationModeEnum.sent) {
      mode = ApiActionEnum.SENT;
    }
    return this.apiService.getById<NotificationDlcm>(this._urlResource + urlSeparator + mode, action.id)
      .pipe(
        tap((model: NotificationDlcm) => {
          ctx.dispatch(new PreservationSpaceNotificationAction.GetByIdSuccess(action, model));
        }),
        catchError(error => {
          ctx.dispatch(new PreservationSpaceNotificationAction.GetByIdFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(PreservationSpaceNotificationAction.Accept)
  accept(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.Accept): Observable<Result> {
    return this.apiService.post<Result>(this._urlResource + urlSeparator + action.notificationId + urlSeparator + ApiActionEnum.ACCEPT)
      .pipe(
        tap(result => ctx.dispatch(new PreservationSpaceNotificationAction.AcceptSuccess(action))),
        catchError(error => {
          ctx.dispatch(new PreservationSpaceNotificationAction.AcceptFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(PreservationSpaceNotificationAction.AcceptSuccess)
  acceptSuccess(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.AcceptSuccess): void {
    this.redirectToNotificationDetail(ctx, (action.parentAction as PreservationSpaceNotificationAction.Accept).notificationId);
    this.notificationService.showSuccess(MARK_AS_TRANSLATABLE("preservationSpace.notifications.accept.success"));
  }

  @Action(PreservationSpaceNotificationAction.AcceptFail)
  acceptFail(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.AcceptFail): void {
    this.notificationService.showError(MARK_AS_TRANSLATABLE("preservationSpace.notifications.accept.fail"));
  }

  @Action(PreservationSpaceNotificationAction.Refuse)
  refuse(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.Refuse): Observable<Result> {
    return this.apiService.post<Result>(this._urlResource + urlSeparator + action.notificationId + urlSeparator + ApiActionEnum.REFUSE)
      .pipe(
        tap(result => ctx.dispatch(new PreservationSpaceNotificationAction.RefuseSuccess(action))),
        catchError(error => {
          ctx.dispatch(new PreservationSpaceNotificationAction.RefuseFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(PreservationSpaceNotificationAction.RefuseSuccess)
  refuseSuccess(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.RefuseSuccess): void {
    this.redirectToNotificationDetail(ctx, (action.parentAction as PreservationSpaceNotificationAction.Refuse).notificationId);
    this.notificationService.showSuccess(MARK_AS_TRANSLATABLE("preservationSpace.notifications.refuse.success"));
  }

  @Action(PreservationSpaceNotificationAction.RefuseFail)
  refuseFail(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.RefuseFail): void {
    this.notificationService.showError(MARK_AS_TRANSLATABLE("preservationSpace.notifications.refuse.fail"));
  }

  @Action(PreservationSpaceNotificationAction.Wait)
  wait(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.Wait): Observable<Result> {
    return this.apiService.post<Result>(this._urlResource + urlSeparator + action.notificationId + urlSeparator + ApiActionEnum.WAIT)
      .pipe(
        tap(result => ctx.dispatch(new PreservationSpaceNotificationAction.WaitSuccess(action))),
        catchError(error => {
          ctx.dispatch(new PreservationSpaceNotificationAction.WaitFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(PreservationSpaceNotificationAction.WaitSuccess)
  waitSuccess(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.WaitSuccess): void {
    this.redirectToNotificationDetail(ctx, (action.parentAction as PreservationSpaceNotificationAction.Wait).notificationId);
    this.notificationService.showSuccess(MARK_AS_TRANSLATABLE("preservationSpace.notifications.wait.success"));
  }

  @Action(PreservationSpaceNotificationAction.WaitFail)
  waitFail(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.WaitFail): void {
    this.notificationService.showError(MARK_AS_TRANSLATABLE("preservationSpace.notifications.wait.fail"));
  }

  private redirectToNotificationDetail(ctx: StateContext<PreservationSpaceNotificationStateModel>, notificationId: string): void {
    ctx.dispatch(new Navigate([RoutesEnum.preservationSpaceNotificationInboxDetail, notificationId], NotificationHelper.getUrlQueryParam(ctx.getState().orgUnitId)));
    ctx.dispatch(new PreservationSpaceNotificationAction.GetById(notificationId));
  }

  @OverrideDefaultAction()
  @Action(PreservationSpaceNotificationAction.Clean)
  clean(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.Clean): void {
    const orgUnitId = ctx.getState().orgUnitId;
    const mode = ctx.getState().mode;
    super.clean(ctx, action);
    ctx.patchState({
      orgUnitId,
      mode,
    });
  }
}
