import {PersonRole} from "@admin/models/person-role.model";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Input,
  Output,
} from "@angular/core";
import {
  FormBuilder,
  Validators,
} from "@angular/forms";
import {NotificationHelper} from "@app/features/preservation-space/notification/helper/notification.helper";
import {environment} from "@environments/environment";
import {
  OrganizationalUnit,
  PreservationPolicy,
  ResearchDomain,
  Role,
  SubmissionPolicy,
} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {SharedAbstractFormPresentational} from "@shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {PersonOrgUnitRoleMode} from "@shared/components/presentationals/shared-person-orgunit-role/shared-person-orgunit-role.presentational";
import {LocalModelAttributeEnum} from "@shared/enums/model-attribute.enum";
import {
  HomePageRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {BreakpointService} from "@shared/services/breakpoint.service";
import {SecurityService} from "@shared/services/security.service";
import {sharedResearchDomainActionNameSpace} from "@shared/stores/research-domain/shared-research-domain.action";
import {SharedResearchDomainState} from "@shared/stores/research-domain/shared-research-domain.state";
import {RegexpUtil} from "@shared/utils/regexp.util";
import _ from "lodash";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  DateUtil,
  isEmptyString,
  isNullOrUndefined,
  MappingObject,
  MappingObjectUtil,
  MARK_AS_TRANSLATABLE,
  NotificationService,
  ObservableUtil,
  PropertyName,
  ResourceNameSpace,
  SolidifyValidator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-orgunit-form",
  templateUrl: "./orgunit-form.presentational.html",
  styleUrls: ["../../../../../../shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class OrgunitFormPresentational extends SharedAbstractFormPresentational<OrganizationalUnit> {
  private readonly KEY_SOURCE: keyof ResearchDomain = "source";
  readonly TIME_BEFORE_DISPLAY_TOOLTIP: number = environment.timeBeforeDisplayTooltipOnInput;
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  @Input()
  researchDomainSources: string[];

  @Input()
  listSubmissionPolicies: SubmissionPolicy[];

  @Input()
  listPreservationPolicies: PreservationPolicy[];

  @Input()
  selectedPersonRole: PersonRole[];

  @Input()
  listRole: Role[];

  @Input()
  isManager: boolean;

  @Input()
  urlQueryParameters: MappingObject<string | undefined>;

  researchDomainLabelCallback: (value: ResearchDomain) => string = (value: ResearchDomain) => `[${value.source}] ${value.name}`;

  protected readonly _requestToBeMemberBS: BehaviorSubject<void> = new BehaviorSubject<void>(undefined);
  @Output("requestToBeMemberChange")
  readonly requestToBeMemberObs: Observable<void> = ObservableUtil.asObservable(this._requestToBeMemberBS);

  sharedResearchDomainActionNameSpace: ResourceNameSpace = sharedResearchDomainActionNameSpace;
  sharedResearchDomainState: typeof SharedResearchDomainState = SharedResearchDomainState;

  extraSearchParameterSource: MappingObject = {};
  personIdAutomaticallyAdded: string | undefined = undefined;

  get personOrgUnitRoleMode(): typeof PersonOrgUnitRoleMode {
    return PersonOrgUnitRoleMode;
  }

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              private readonly _fb: FormBuilder,
              public readonly breakpointService: BreakpointService,
              private readonly _notificationService: NotificationService,
              public readonly securityService: SecurityService) {
    super(_changeDetectorRef, _elementRef);
  }

  protected initNewForm(): void {
    this.form = this._fb.group({
      [this.formDefinition.name]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.description]: ["", [SolidifyValidator]],
      [this.formDefinition.url]: ["", [SolidifyValidator, Validators.pattern(RegexpUtil.url)]],
      [this.formDefinition.openingDate]: [""],
      [this.formDefinition.closingDate]: [""],
      [this.formDefinition.personRole]: ["", [SolidifyValidator]],
      [this.formDefinition.researchDomains]: ["", [SolidifyValidator]],
      [this.formDefinition.keywords]: [[], [SolidifyValidator]],
    });
  }

  protected bindFormTo(organizationalUnit: OrganizationalUnit): void {
    this.form = this._fb.group({
      [this.formDefinition.name]: [organizationalUnit.name, [Validators.required, SolidifyValidator]],
      [this.formDefinition.description]: [organizationalUnit.description, [SolidifyValidator]],
      [this.formDefinition.url]: [organizationalUnit.url, [SolidifyValidator, Validators.pattern(RegexpUtil.url)]],
      [this.formDefinition.openingDate]: [organizationalUnit.openingDate],
      [this.formDefinition.closingDate]: [organizationalUnit.closingDate],
      [this.formDefinition.personRole]: ["", [SolidifyValidator]],
      [this.formDefinition.researchDomains]: [_.map(organizationalUnit.researchDomains, LocalModelAttributeEnum.resId), [SolidifyValidator]],
      [this.formDefinition.keywords]: [isNullOrUndefined(organizationalUnit.keywords) ? [] : [...organizationalUnit.keywords], [SolidifyValidator]],
    });

    this._addRoleAskingInNotificationRequest();
  }

  private _addRoleAskingInNotificationRequest(): void {
    if (isNullOrUndefined(this.urlQueryParameters) || MappingObjectUtil.size(this.urlQueryParameters) === 0) {
      return;
    }
    const roleId = MappingObjectUtil.get(this.urlQueryParameters, NotificationHelper.KEY_ROLE_ID);
    const personId = MappingObjectUtil.get(this.urlQueryParameters, NotificationHelper.KEY_PERSON_ID);
    if (isNullOrUndefined(roleId) || isNullOrUndefined(personId)) {
      return;
    }
    this.personIdAutomaticallyAdded = personId;
    const existingPerson = this.selectedPersonRole.find(p => p.resId === personId);
    if (isNullOrUndefined(existingPerson)) {
      this.selectedPersonRole = [...this.selectedPersonRole, {
        resId: personId,
        roles: [{
          resId: roleId,
        }],
      }];
      this._notificationService.showInformation(MARK_AS_TRANSLATABLE("organizationalUnit.notification.personAddedToOrgUnit"));
    } else {
      this._notificationService.showInformation(MARK_AS_TRANSLATABLE("organizationalUnit.notification.personAlreadyInOrgUnit"));
    }
  }

  protected treatmentBeforeSubmit(organizationalUnit: OrganizationalUnit): OrganizationalUnit {
    organizationalUnit.openingDate = DateUtil.convertToLocalDateDateSimple(organizationalUnit.openingDate);
    organizationalUnit.closingDate = DateUtil.convertToLocalDateDateSimple(organizationalUnit.closingDate);
    organizationalUnit.researchDomains = [];
    if (this.form.get(this.formDefinition.researchDomains).value !== "") {
      this.form.get(this.formDefinition.researchDomains).value.forEach(resId => {
        organizationalUnit.researchDomains.push({resId: resId});
      });
    }
    return organizationalUnit;
  }

  requestToBeMember(): void {
    this._requestToBeMemberBS.next();
  }

  goToReceivedRequestNotification(): void {
    this._navigateWithQueryParamBS.next(new Navigate([RoutesEnum.preservationSpaceNotificationInbox], NotificationHelper.getUrlQueryParam(this.model.resId)));
  }

  goToSentRequestNotification(): void {
    this._navigateWithQueryParamBS.next(new Navigate([RoutesEnum.preservationSpaceNotificationSent], NotificationHelper.getUrlQueryParam(this.model.resId)));
  }

  goToDeposit(): void {
    this._navigateBS.next([RoutesEnum.deposit, this.model.resId]);
  }

  goToArchive(): void {
    this._navigateBS.next([RoutesEnum.homePage, HomePageRoutesEnum.browsing, this.model.resId]);
  }

  filterResearchDomain(source: string): void {
    this._computeExtraSearchParameterCategory(source);
  }

  private _computeExtraSearchParameterCategory(value: string): void {
    if (isNullOrUndefined(value) || isEmptyString(value)) {
      MappingObjectUtil.delete(this.extraSearchParameterSource, this.KEY_SOURCE as string);
      return;
    }
    MappingObjectUtil.set(this.extraSearchParameterSource, this.KEY_SOURCE as string, value);
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() name: string;
  @PropertyName() description: string;
  @PropertyName() url: string;
  @PropertyName() closingDate: string;
  @PropertyName() openingDate: string;
  @PropertyName() personRole: string;
  @PropertyName() researchDomains: string;
  @PropertyName() keywords: string;
}
