import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {NotificationHelper} from "@app/features/preservation-space/notification/helper/notification.helper";
import {OrgunitRequestAccessDialog} from "@app/features/preservation-space/organizational-unit/components/dialogs/orgunit-request-access/orgunit-request-access.dialog";
import {orgUnitActionNameSpace} from "@app/features/preservation-space/organizational-unit/stores/organizational-unit.action";
import {OrganizationalUnitStateModel} from "@app/features/preservation-space/organizational-unit/stores/organizational-unit.state";
import {AppUserState} from "@app/stores/user/app-user.state";
import {
  NotificationType,
  OrganizationalUnit,
} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SharedAbstractListRoutable} from "@shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {DataTableActions} from "@shared/models/data-table-actions.model";
import {RouterExtService} from "@shared/services/router-ext.service";
import {SecurityService} from "@shared/services/security.service";
import {SharedNotificationAction} from "@shared/stores/notification/shared-notification.action";
import {tap} from "rxjs/operators";
import {
  isNullOrUndefined,
  MARK_AS_TRANSLATABLE,
  OrderEnum,
  ResourceState,
} from "solidify-frontend";

@Component({
  selector: "dlcm-orgunit-list-routable",
  templateUrl: "../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class OrgunitListRoutable extends SharedAbstractListRoutable<OrganizationalUnit, OrganizationalUnitStateModel> {
  readonly KEY_CREATE_BUTTON: string = MARK_AS_TRANSLATABLE("organizationalUnit.new");
  readonly KEY_REFRESH_BUTTON: string = MARK_AS_TRANSLATABLE("organizationalUnit.refresh");
  readonly KEY_BACK_BUTTON: string | undefined = undefined;
  readonly KEY_PARAM_NAME: keyof OrganizationalUnit & string = "name";

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog,
              private readonly _securityService: SecurityService) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, LocalStateEnum.organizationalUnit, orgUnitActionNameSpace, {
      canCreate: false,
      canGoBack: false,
    });
  }

  protected defineActions(): DataTableActions<OrganizationalUnit>[] {
    return [
      {
        logo: IconNameEnum.edit,
        callback: model => this.goToEdit(model),
        placeholder: MARK_AS_TRANSLATABLE("crud.list.action.goToEdit"),
        displayOnCondition: model => this.conditionDisplayEditButton(model),
      },
      {
        logo: IconNameEnum.deposit,
        callback: model => this.goToDeposit(model),
        placeholder: MARK_AS_TRANSLATABLE("crud.list.action.goToDeposit"),
        displayOnCondition: model => this._securityService.isRootOrAdmin() || this._securityService.isMemberOfOrgUnit(model.resId),
      },
      {
        logo: IconNameEnum.requestSent,
        callback: model => this.goToSentRequestNotification(model),
        placeholder: MARK_AS_TRANSLATABLE("crud.list.action.goToSentRequestNotification"),
        displayOnCondition: model => true,
      },
      {
        logo: IconNameEnum.requestInbox,
        callback: model => this.goToReceivedRequestNotification(model),
        placeholder: MARK_AS_TRANSLATABLE("crud.list.action.goToReceivedRequestNotification"),
        displayOnCondition: model => this._securityService.isRootOrAdmin() || this._securityService.isMemberOfOrgUnit(model.resId),
      },
      {
        logo: IconNameEnum.sendRequest,
        callback: model => this.requestToBeMember(model.resId),
        placeholder: MARK_AS_TRANSLATABLE("organizationalUnit.requestToBeMember"),
        displayOnCondition: model => !this._securityService.isRootOrAdmin() && !this._securityService.isMemberOfOrgUnit(model.resId),
      },
      {
        logo: IconNameEnum.member,
        callback: model => null,
        placeholder: MARK_AS_TRANSLATABLE("organizationalUnit.list.indicator.memberOfOrgUnit"),
        displayOnCondition: model => !this._securityService.isRootOrAdmin() && this._securityService.isMemberOfOrgUnit(model.resId),
        disableCondition: model => true,
      },
    ];
  }

  conditionDisplayEditButton(model: OrganizationalUnit | undefined): boolean {
    if (isNullOrUndefined(model)) {
      return true;
    }
    return this._securityService.isRootOrAdmin();
  }

  conditionDisplayDeleteButton(model: OrganizationalUnit | undefined): boolean {
    return false;
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "name",
        header: MARK_AS_TRANSLATABLE("organizationalUnit.table.header.name"),
        type: FieldTypeEnum.string,
        order: OrderEnum.ascending,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "openingDate",
        header: MARK_AS_TRANSLATABLE("organizationalUnit.table.header.openingDate"),
        type: FieldTypeEnum.date,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
      },
      {
        field: "closingDate" as any,
        header: MARK_AS_TRANSLATABLE("organizationalUnit.table.header.closingDate"),
        type: FieldTypeEnum.date,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
      },
    ];
  }

  private goToReceivedRequestNotification(orgUnit: OrganizationalUnit): void {
    this._store.dispatch(new Navigate([RoutesEnum.preservationSpaceNotificationInbox], NotificationHelper.getUrlQueryParam(orgUnit.resId)));
  }

  private goToSentRequestNotification(orgUnit: OrganizationalUnit): void {
    this._store.dispatch(new Navigate([RoutesEnum.preservationSpaceNotificationSent], NotificationHelper.getUrlQueryParam(orgUnit.resId)));
  }

  goToDeposit(orgUnit: OrganizationalUnit): void {
    this._store.dispatch(new Navigate([RoutesEnum.deposit, orgUnit.resId]));
  }

  requestToBeMember(orgUnitResId: string): void {
    this.subscribe(this._dialog.open(OrgunitRequestAccessDialog, {
      minWidth: "500px",
    }).afterClosed().pipe(
      tap((message: string | undefined) => {
        if (isNullOrUndefined(message)) {
          return;
        }
        this.sendRequestToBeMember(orgUnitResId, message);
      }),
    ));
  }

  private sendRequestToBeMember(orgUnitResId: string, message: string): void {
    this._store.dispatch(new SharedNotificationAction.Create({
      model: {
        emitter: ResourceState.currentSnapshot(this._store, AppUserState),
        notifiedOrgUnit: {
          resId: orgUnitResId,
        },
        message: message,
        notificationType: NotificationType.NotificationTypeEnum.JOIN_ORG_UNIT_REQUEST,
      },
    }));
  }
}
