import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
} from "@angular/core";
import {FormBuilder} from "@angular/forms";
import {sharedLicenseActionNameSpace} from "@app/shared/stores/license/shared-license.action";
import {appAuthorizedOrganizationalUnitNameSpace} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.action";
import {AppAuthorizedOrganizationalUnitState} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.state";
import {
  Aip,
  AipChecksumCheck,
} from "@models";
import {TranslateService} from "@ngx-translate/core";
import {SharedAbstractFormPresentational} from "@shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {AccessLevelEnumHelper} from "@shared/enums/business/access-level-enum.helper";
import {PackageStatusEnumHelper} from "@shared/enums/business/package-status.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {SharedLicenseState} from "@shared/stores/license/shared-license.state";
import {FileUtil} from "@shared/utils/file.util";
import {
  DateUtil,
  isNullOrUndefined,
  KeyValue,
  PropertyName,
  ResourceNameSpace,
  StringUtil,
  MARK_AS_TRANSLATABLE,
  urlSeparator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-aip-form",
  templateUrl: "./aip-form.presentational.html",
  styleUrls: ["./aip-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AipFormPresentational extends SharedAbstractFormPresentational<Aip> {
  accessEnumValues: KeyValue[] = AccessLevelEnumHelper.getListKeyValue();
  packageStatusEnumValues: KeyValue[] = PackageStatusEnumHelper.getListKeyValue();

  sharedLicenseActionNameSpace: ResourceNameSpace = sharedLicenseActionNameSpace;
  sharedLicenseState: typeof SharedLicenseState = SharedLicenseState;

  appAuthorizedOrganizationalUnitNameSpace: ResourceNameSpace = appAuthorizedOrganizationalUnitNameSpace;
  appAuthorizedOrganizationalUnitState: typeof AppAuthorizedOrganizationalUnitState = AppAuthorizedOrganizationalUnitState;

  public formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              private readonly _fb: FormBuilder,
              private readonly _translate: TranslateService) {
    super(_changeDetectorRef, _elementRef);
  }

  protected initNewForm(): void {
    this.form = this._fb.group({});
  }

  protected bindFormTo(aip: Aip): void {
    this.form = this._fb.group({
      [this.formDefinition.id]: [aip.resId, []],
      [this.formDefinition.organizationalUnitId]: [aip.info.organizationalUnitId, []],
      [this.formDefinition.name]: [aip.info.name, []],
      [this.formDefinition.description]: [aip.info.description, []],
      [this.formDefinition.yearOfCreation]: [DateUtil.extractYearFromDate(aip.creation.when), []],
      [this.formDefinition.status]: [aip.info.status, []],
      [this.formDefinition.lastchecksumCheck]: [this.extractChecksumCheck(aip.checksumCheck), []],
      [this.formDefinition.accessLevel]: [aip.info.access, []],
      [this.formDefinition.licenseId]: [aip.info.licenseId, []],
      [this.formDefinition.complianceLevel]: [aip.info.complianceLevel, []],
      [this.formDefinition.container]: [aip.archiveContainer, []],
      [this.formDefinition.archive]: [aip.archiveId, []],
      [this.formDefinition.size]: [FileUtil.transformFileSize(aip.archiveSize), []],
      [this.formDefinition.files]: [aip.archiveFileNumber, []],
      [this.formDefinition.metadataVersion]: [aip.info.metadataVersion, []],
    });
  }

  protected treatmentBeforeSubmit(aip: Aip): Aip {
    return aip;
  }

  extractChecksumCheck(checksum: AipChecksumCheck): string {
    let lastChecksumCheckNever = StringUtil.stringEmpty;
    let lastChecksumCheckSuccess = StringUtil.stringEmpty;
    let lastChecksumCheckFail = StringUtil.stringEmpty;

    const keyLabelNever = MARK_AS_TRANSLATABLE("aip.form.lastChecksumCheck.never");
    const keyLabelSuccess = MARK_AS_TRANSLATABLE("aip.form.lastChecksumCheck.success");
    const keyLabelFail = MARK_AS_TRANSLATABLE("aip.form.lastChecksumCheck.fail");

    this._translate.get([
      keyLabelNever,
      keyLabelSuccess,
      keyLabelFail,
    ]).subscribe(translation => {
      lastChecksumCheckNever = translation[keyLabelNever];
      lastChecksumCheckSuccess = translation[keyLabelSuccess];
      lastChecksumCheckFail = translation[keyLabelFail];
    });

    if (isNullOrUndefined(checksum)) {
      return lastChecksumCheckNever;
    }

    const strDate = DateUtil.convertToLocalDateDateSimple(checksum.checkDate);
    const str = checksum.checkingSucceed ? lastChecksumCheckSuccess : lastChecksumCheckFail;
    return strDate + " (" + str + ")";
  }

  getSipPath(sipId: string): string {
    return "#" + urlSeparator + RoutesEnum.preservationSipDetail + urlSeparator + sipId;
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() id: string;
  @PropertyName() organizationalUnitId: string;
  @PropertyName() name: string;
  @PropertyName() description: string;
  @PropertyName() yearOfCreation: string;
  @PropertyName() status: string;
  @PropertyName() lastchecksumCheck: string;
  @PropertyName() accessLevel: string;
  @PropertyName() licenseId: string;
  @PropertyName() complianceLevel: string;
  @PropertyName() container: string;
  @PropertyName() archive: string;
  @PropertyName() size: string;
  @PropertyName() files: string;
  @PropertyName() metadataVersion: string;
}
