import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
  ViewChild,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {MatTabGroup} from "@angular/material/tabs";
import {
  ActivatedRoute,
  NavigationCancel,
  NavigationEnd,
  Router,
} from "@angular/router";
import {
  PreservationAipState,
  PreservationAipStateModel,
} from "@app/features/preservation/aip/stores/aip.state";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {environment} from "@environments/environment";
import {Aip} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {AipTabEnum} from "@preservation/aip/enums/aip-tab.enum";
import {
  AipHelper,
  AipMode,
} from "@preservation/aip/helpers/aip.helper";
import {
  PreservationAipAction,
  preservationAipActionNameSpace,
} from "@preservation/aip/stores/aip.action";
import {PreservationAipStatusHistoryAction} from "@preservation/aip/stores/status-history/aip-status-history.action";
import {PreservationAipStatusHistoryState} from "@preservation/aip/stores/status-history/aip-status-history.state";
import {SharedHistoryDialog} from "@shared/components/dialogs/shared-history/shared-history.dialog";
import {SharedAbstractDetailEditCommonRoutable} from "@shared/components/routables/shared-abstract-detail-edit-common/shared-abstract-detail-edit-common.routable";
import {PackageStatusEnum} from "@shared/enums/business/package-status.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
  RoutesEnum,
  urlSeparator,
} from "@shared/enums/routes.enum";
import {ExtraButtonToolbar} from "@shared/models/extra-button-toolbar.model";
import {StatusHistoryDialog} from "@shared/models/status-history-dialog.model";
import {StatusHistory} from "@shared/models/status-history.model";
import {Observable} from "rxjs";
import {
  distinctUntilChanged,
  filter,
  tap,
} from "rxjs/operators";
import {
  isNullOrUndefined,
  MemoizedUtil,
  OverrideProperty,
  QueryParameters,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-aip-detail-edit-routable",
  templateUrl: "./aip-detail-edit.routable.html",
  styleUrls: ["./aip-detail-edit.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AipDetailEditRoutable extends SharedAbstractDetailEditCommonRoutable<Aip, PreservationAipStateModel> implements OnInit {
  @Select(PreservationAipState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(PreservationAipState.isReadyToBeDisplayed) isReadyToBeDisplayedObs: Observable<boolean>;

  historyObs: Observable<StatusHistory[]> = MemoizedUtil.select(this._store, PreservationAipStatusHistoryState, state => state.history);
  isLoadingHistoryObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, PreservationAipStatusHistoryState);
  queryParametersObs: Observable<QueryParameters> = MemoizedUtil.select(this._store, PreservationAipStatusHistoryState, state => state.queryParameters);

  readonly KEY_PARAM_NAME: keyof Aip & string = "archiveId";

  tabSelected: AipTabEnum;

  @OverrideProperty()
  readonly editAvailable: boolean = false;

  @OverrideProperty()
  readonly deleteAvailable: boolean = false;

  @ViewChild("matTabGroup")
  readonly matTabGroup: MatTabGroup;

  storagion_number: number | undefined = undefined;
  mode: AipMode;

  listExtraButtons: ExtraButtonToolbar<Aip>[] = [
    {
      color: "primary",
      icon: IconNameEnum.simpleChecksum,
      displayCondition: current => !isNullOrUndefined(current) && !isNullOrUndefined(current.info) && current.info.status === PackageStatusEnum.COMPLETED,
      callback: () => this.simpleChecksum(),
      labelToTranslate: MARK_AS_TRANSLATABLE("app.navigation.button.simpleCheck"),
      order: 40,

    },
    {
      color: "primary",
      icon: IconNameEnum.doubleChecksum,
      displayCondition: current => !isNullOrUndefined(current) && !isNullOrUndefined(current.info) && current.info.status === PackageStatusEnum.COMPLETED,
      callback: () => this.deepChecksum(),
      labelToTranslate: MARK_AS_TRANSLATABLE("app.navigation.button.deepCheck"),
      order: 40,
    },
    {
      color: "primary",
      icon: IconNameEnum.reindex,
      displayCondition: current => !isNullOrUndefined(current) && !isNullOrUndefined(current.info) && current.info.status === PackageStatusEnum.COMPLETED,
      callback: () => this.reIndex(),
      labelToTranslate: MARK_AS_TRANSLATABLE("app.navigation.button.reindex"),
      order: 40,
    },
    {
      color: "primary",
      icon: IconNameEnum.check,
      displayCondition: current => !isNullOrUndefined(current) && !isNullOrUndefined(current.info) && current.info.status === PackageStatusEnum.COMPLETED,
      callback: () => this.check(),
      labelToTranslate: MARK_AS_TRANSLATABLE("app.navigation.button.check"),
      order: 40,
    },
    {
      color: "primary",
      icon: IconNameEnum.download,
      displayCondition: current => !isNullOrUndefined(current) && !isNullOrUndefined(current.info) && current.info.status === PackageStatusEnum.COMPLETED,
      callback: () => this.download(),
      labelToTranslate: MARK_AS_TRANSLATABLE("app.navigation.button.download"),
      order: 40,
    },
    {
      color: "primary",
      icon: IconNameEnum.resume,
      displayCondition: current => !isNullOrUndefined(current) && !isNullOrUndefined(current.info) && current.info.status !== PackageStatusEnum.COMPLETED,
      callback: () => this.resume(),
      labelToTranslate: MARK_AS_TRANSLATABLE("app.navigation.button.resume"),
      order: 40,
    },
  ];

  constructor(protected _store: Store,
              protected route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _router: Router,
              protected readonly dialog: MatDialog) {
    super(_store, route, _actions$, _changeDetector, dialog, LocalStateEnum.preservation_aip, preservationAipActionNameSpace, LocalStateEnum.preservation);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.mode = AipHelper.determineMode(this._store);
    if (this.mode === AipMode.AIP) {
      this.storagion_number = +this._route.snapshot.parent.paramMap.get(PreservationPlanningRoutesEnum.storagionNumberWithoutPrefixParam);
    }
    this.retrieveCurrentModelWithUrl();

    this._computeCurrentTab();

    this.subscribe(this._router.events
      .pipe(
        filter(event => event instanceof NavigationCancel || event instanceof NavigationEnd),
        distinctUntilChanged(),
        tap(event => {
          this.retrieveResIdFromUrl();
          this._computeCurrentTab();
          this.matTabGroup.selectedIndex = this.tabSelected;
        }),
      ),
    );
  }

  getSubResourceWithParentId(id: string): void {
  }

  backToList(): void {
    if (this.mode === AipMode.AIP) {
      this._store.dispatch(new Navigate([RoutesEnum.preservationAip + urlSeparator + this.storagion_number]));
    } else {
      this._store.dispatch(new Navigate([RoutesEnum.preservationAipDownloaded]));
    }
  }

  simpleChecksum(): void {
    this._store.dispatch(new PreservationAipAction.SimpleChecksum(this._resId));
  }

  deepChecksum(): void {
    this._store.dispatch(new PreservationAipAction.DeepChecksum(this._resId));
  }

  reIndex(): void {
    this._store.dispatch(new PreservationAipAction.Reindex(this._resId));
  }

  check(): void {
    this._store.dispatch(new PreservationAipAction.Check(this._resId));
  }

  resume(): void {
    this._store.dispatch(new PreservationAipAction.Resume(this._resId));
  }

  download(): void {
    this._store.dispatch(new PreservationAipAction.Download(this._resId));
  }

  get packageStatusEnum(): typeof PackageStatusEnum {
    return PackageStatusEnum;
  }

  private _computeCurrentTab(): void {
    const tabRouteSelected = AipHelper.getTabRouteSelected(this._route);
    this.tabSelected = AipHelper.getTabSelectedIndexWithRoute(tabRouteSelected);
  }

  navigateToTab(tabSelected: AipTabEnum): void {
    const tabRouteSelected = AipHelper.getTabRouteSelectedWithTabIndex(tabSelected);
    const pathAipDetail = RoutesEnum.preservationAip + urlSeparator + this.storagion_number + urlSeparator + PreservationPlanningRoutesEnum.aipDetail + AppRoutesEnum.separator;
    const path = [pathAipDetail, this._resId, tabRouteSelected];
    this._store.dispatch(new Navigate(path));
  }

  showHistory(): void {
    const dialogRef = this._dialog.open(SharedHistoryDialog, {
      width: environment.modalWidth,
      data: {
        parentId: null,
        resourceResId: this._resId,
        name: this._state,
        statusHistory: this.historyObs,
        isLoading: this.isLoadingHistoryObs,
        queryParametersObs: this.queryParametersObs,
        state: PreservationAipStatusHistoryAction,
      } as StatusHistoryDialog,
    });
  }
}
