import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {
  ActivatedRoute,
  NavigationCancel,
  NavigationEnd,
  Router,
} from "@angular/router";
import {
  PreservationAipState,
  PreservationAipStateModel,
} from "@app/features/preservation/aip/stores/aip.state";
import {SharedAbstractListRoutable} from "@app/shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@app/shared/enums/field-type.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {appAuthorizedOrganizationalUnitNameSpace} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.action";
import {AppAuthorizedOrganizationalUnitState} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.state";
import {Aip} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {
  AipHelper,
  AipMode,
} from "@preservation/aip/helpers/aip.helper";
import {
  PreservationAipAction,
  preservationAipActionNameSpace,
} from "@preservation/aip/stores/aip.action";
import {PackageStatusEnumHelper} from "@shared/enums/business/package-status.enum";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {
  PreservationPlanningRoutesEnum,
  RoutesEnum,
  urlSeparator,
} from "@shared/enums/routes.enum";
import {RouterExtService} from "@shared/services/router-ext.service";
import {Observable} from "rxjs";
import {
  distinctUntilChanged,
  filter,
  tap,
} from "rxjs/operators";
import {
  isNullOrUndefined,
  MappingObjectUtil,
  OrderEnum,
  Override,
  OverrideProperty,
  QueryParametersUtil,
  ResourceNameSpace,
  ResourceState,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-aip-list-routable",
  templateUrl: "../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AipListRoutable extends SharedAbstractListRoutable<Aip, PreservationAipStateModel> implements OnInit, OnDestroy {
  readonly KEY_BACK_BUTTON: string | undefined = MARK_AS_TRANSLATABLE("preservation.aip.button.back");
  readonly KEY_CREATE_BUTTON: string = undefined;
  readonly KEY_PARAM_NAME: keyof Aip & string = undefined;
  readonly KEY_REFRESH_BUTTON: string = MARK_AS_TRANSLATABLE("preservation.aip.button.refresh");

  appAuthorizedOrganizationalUnitNameSpace: ResourceNameSpace = appAuthorizedOrganizationalUnitNameSpace;
  appAuthorizedOrganizationalUnitState: typeof AppAuthorizedOrganizationalUnitState = AppAuthorizedOrganizationalUnitState;

  mode: AipMode;

  @OverrideProperty()
  skipInitialQuery: boolean = true;

  private readonly KEY_ARCHIVAL_UNIT: keyof Aip = "archivalUnit";

  columnsSkippedToClear: string[] = [
    this.KEY_ARCHIVAL_UNIT,
  ];

  storagion_number: number | undefined = undefined;

  archivalUnit(value: PreservationPlanningRoutesEnum): void {
    if (value === PreservationPlanningRoutesEnum.aipTabUnit) {
      this.getAipListWithParameter(true);
    } else if (value === PreservationPlanningRoutesEnum.aipTabCollections) {
      this.getAipListWithParameter(false);
    } else {
      this.getAipListWithParameter(undefined);
    }
  }

  constructor(protected readonly store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtService,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog,
              protected readonly _router: Router) {
    super(store, _changeDetector, _route, _routerExt, _actions$, _dialog, LocalStateEnum.preservation_aip, preservationAipActionNameSpace, {
      canCreate: false,
    }, LocalStateEnum.preservation);
  }

  ngOnInit(): void {
    super.ngOnInit();

    this.mode = AipHelper.determineMode(this._store);
    this.updateListWithCurrentTab();
    this.subscribe(this.observeRoutingUpdate());
    if (this.mode === AipMode.AIP) {
      this.storagion_number = +this._route.snapshot.parent.parent.paramMap.get(PreservationPlanningRoutesEnum.storagionNumberWithoutPrefixParam);
    }
  }

  private observeRoutingUpdate(): Observable<any> {
    return this._router.events
      .pipe(
        filter(event => event instanceof NavigationCancel || event instanceof NavigationEnd),
        distinctUntilChanged(),
        tap(event => {
          this.updateListWithCurrentTab();
        }),
      );
  }

  private updateListWithCurrentTab(): void {
    const tab = this._route.snapshot.paramMap.get(PreservationPlanningRoutesEnum.tabWithoutPrefixParam);
    this.archivalUnit(tab as PreservationPlanningRoutesEnum);
  }

  @Override()
  showDetail(model: Aip): void {
    if (this.mode === AipMode.AIP) {
      this.store.dispatch(new Navigate([RoutesEnum.preservationAip + urlSeparator + this.storagion_number + urlSeparator + PreservationPlanningRoutesEnum.aipDetail, model.resId]));
    } else {
      this.store.dispatch(new Navigate([RoutesEnum.preservationAipDownloadedDetail, model.resId]));
    }
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "info.name" as any,
        header: MARK_AS_TRANSLATABLE("aip.table.header.name"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "smartSize",
        header: MARK_AS_TRANSLATABLE("aip.table.header.size"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        filterableField: "archiveSize",
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "archiveFileNumber",
        header: MARK_AS_TRANSLATABLE("aip.table.header.files"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        filterableField: "archiveFileNumber",
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "info.status" as any,
        header: MARK_AS_TRANSLATABLE("aip.table.header.status"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        translate: true,
        filterEnum: PackageStatusEnumHelper.getListKeyValue(),
      },
      {
        field: "info.organizationalUnitId" as any,
        header: MARK_AS_TRANSLATABLE("aip.table.header.organizationalUnit"),
        type: FieldTypeEnum.searchableSingleSelect,
        order: OrderEnum.none,
        component: DataTableComponentEnum.organizationalUnitName,
        isFilterable: true,
        isSortable: true,
        resourceNameSpace: this.appAuthorizedOrganizationalUnitNameSpace,
        resourceState: this.appAuthorizedOrganizationalUnitState as any,
      },
      {
        field: "creation.when" as any,
        header: MARK_AS_TRANSLATABLE("aip.table.header.creation.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
      },
      {
        field: "lastUpdate.when" as any,
        header: MARK_AS_TRANSLATABLE("aip.table.header.lastUpdate.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
      },
    ];
  }

  getAipListWithParameter(isArchivalUnit: boolean | undefined): void {
    let queryParameter = ResourceState.queryParametersSnapshot(this.store, PreservationAipState);
    queryParameter = QueryParametersUtil.clone(queryParameter);
    if (isNullOrUndefined(isArchivalUnit)) {
      MappingObjectUtil.delete(queryParameter.search.searchItems, this.KEY_ARCHIVAL_UNIT);
    } else {
      MappingObjectUtil.set(queryParameter.search.searchItems, this.KEY_ARCHIVAL_UNIT, isArchivalUnit.toString());
    }
    this.store.dispatch(new PreservationAipAction.GetAll(queryParameter, false));
  }

  conditionDisplayEditButton(model: Aip): boolean {
    return false;
  }

  conditionDisplayDeleteButton(model: Aip): boolean {
    return false;
  }

  back(): void {
    if (this.mode === AipMode.AIP) {
      this._store.dispatch(new Navigate([RoutesEnum.preservationAip]));
    } else {
      this._store.dispatch(new Navigate([RoutesEnum.preservationPlanning]));
    }
  }

  ngOnDestroy(): void {
    super.ngOnDestroy();
  }
}
