import {
  ChangeDetectionStrategy,
  Component,
  OnInit,
} from "@angular/core";
import {MatTabChangeEvent} from "@angular/material/tabs";
import {
  ActivatedRoute,
  NavigationCancel,
  NavigationEnd,
  Router,
} from "@angular/router";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";
import {
  AipHelper,
  AipMode,
} from "@preservation/aip/helpers/aip.helper";
import {SharedAbstractRoutable} from "@shared/components/routables/shared-abstract/shared-abstract.routable";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
} from "@shared/enums/routes.enum";
import {Observable} from "rxjs";
import {
  distinctUntilChanged,
  filter,
  tap,
} from "rxjs/operators";

@Component({
  selector: "dlcm-tabs-list-routable",
  templateUrl: "./aip-tabs.routable.html",
  styleUrls: ["./aip-tabs.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AipTabsRoutable extends SharedAbstractRoutable implements OnInit {
  private readonly ALL_TAB_INDEX: number = 0;
  private readonly UNIT_TAB_INDEX: number = 1;
  private readonly COLLECTION_TAB_INDEX: number = 2;
  tabSelected: number = this.ALL_TAB_INDEX;

  private storagionNode: number | undefined = undefined;

  mode: AipMode;

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _router: Router) {
    super();
  }

  ngOnInit(): void {
    this.mode = AipHelper.determineMode(this._store);
    this.updateCurrentTab();
    this.subscribe(this.observeRoutingUpdate());
  }

  private observeRoutingUpdate(): Observable<any> {
    return this._router.events
      .pipe(
        filter(event => event instanceof NavigationCancel || event instanceof NavigationEnd),
        distinctUntilChanged(),
        tap(event => {
          this.updateCurrentTab();
        }),
      );
  }

  private updateCurrentTab(): void {
    if (this.mode === AipMode.AIP) {
      this.storagionNode = +this._route.snapshot.parent.paramMap.get(PreservationPlanningRoutesEnum.storagionNumberWithoutPrefixParam);
    }
    const tab = this._route.snapshot.children[0].paramMap.get(PreservationPlanningRoutesEnum.tabWithoutPrefixParam);
    switch (tab) {
      case PreservationPlanningRoutesEnum.aipTabAll:
        this.tabSelected = this.ALL_TAB_INDEX;
        break;
      case PreservationPlanningRoutesEnum.aipTabUnit:
        this.tabSelected = this.UNIT_TAB_INDEX;
        break;
      case PreservationPlanningRoutesEnum.aipTabCollections:
        this.tabSelected = this.COLLECTION_TAB_INDEX;
        break;
    }
  }

  onTabChanged(tabEvent: MatTabChangeEvent): void {
    const index = tabEvent.index;
    let tab = PreservationPlanningRoutesEnum.aipTabAll;
    switch (index) {
      case this.ALL_TAB_INDEX:
        tab = PreservationPlanningRoutesEnum.aipTabAll;
        break;
      case this.UNIT_TAB_INDEX:
        tab = PreservationPlanningRoutesEnum.aipTabUnit;
        break;
      case this.COLLECTION_TAB_INDEX:
        tab = PreservationPlanningRoutesEnum.aipTabCollections;
        break;
    }
    if (this.mode === AipMode.AIP) {
      this._store.dispatch(new Navigate([AppRoutesEnum.preservation, PreservationPlanningRoutesEnum.aip, this.storagionNode, PreservationPlanningRoutesEnum.aipList, tab]));
    } else {
      this._store.dispatch(new Navigate([AppRoutesEnum.preservation, PreservationPlanningRoutesEnum.aipDownloaded, tab]));
    }
  }

}
