import {HttpParams} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {
  ActivatedRoute,
  Router,
} from "@angular/router";
import {
  defaultAipOrgUnitValue,
  PreservationAipOrganizationalUnitState,
  PreservationAipOrganizationalUnitStateModel,
} from "@app/features/preservation/aip/stores/organizational-unit/aip-organizational-unit.state";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {Aip} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {AipHelper} from "@preservation/aip/helpers/aip.helper";
import {
  defaultAipAipValue,
  PreservationAipAipState,
  PreservationAipAipStateModel,
} from "@preservation/aip/stores/aip-aip/aip-aip.state";
import {
  PreservationAipAction,
  preservationAipActionNameSpace,
} from "@preservation/aip/stores/aip.action";
import {
  PreservationAipStatusHistoryState,
  PreservationAipStatusHistoryStateModel,
} from "@preservation/aip/stores/status-history/aip-status-history.state";

import {ApiActionEnum} from "@shared/enums/api-action.enum";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
  RoutesEnum,
  urlSeparator,
} from "@shared/enums/routes.enum";
import {DownloadService} from "@shared/services/download.service";
import {defaultStatusHistoryInitValue} from "@shared/stores/status-history/status-history.state";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  NotificationService,
  OverrideDefaultAction,
  ResourceState,
  ResourceStateModel,
  SolidifyStateError,
  StoreUtil,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

export interface PreservationAipStateModel extends ResourceStateModel<Aip> {
  preservation_aip_organizationalUnit: PreservationAipOrganizationalUnitStateModel;
  preservation_aip_aip: PreservationAipAipStateModel;
  preservation_aip_statusHistory: PreservationAipStatusHistoryStateModel;
}

@Injectable()
@State<PreservationAipStateModel>({
  name: LocalStateEnum.preservation_aip,
  defaults: {
    ...defaultResourceStateInitValue(),
    preservation_aip_organizationalUnit: defaultAipOrgUnitValue(),
    preservation_aip_aip: defaultAipAipValue(),
    preservation_aip_statusHistory: {...defaultStatusHistoryInitValue()},
  },
  children: [
    PreservationAipOrganizationalUnitState,
    PreservationAipAipState,
    PreservationAipStatusHistoryState,
  ],
})
export class PreservationAipState extends ResourceState<PreservationAipStateModel, Aip> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions,
              private readonly _router: Router,
              private readonly _route: ActivatedRoute,
              private downloadService: DownloadService) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationAipActionNameSpace,
    });
  }

  protected get _urlResource(): string {
    return AipHelper.generateUrlResource(this.store);
  }

  @Selector()
  static currentAipName(state: PreservationAipStateModel): string | undefined {
    return state.current.info.name;
  }

  @Selector()
  static isLoadingWithDependency(state: PreservationAipStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static isReadyToBeDisplayed(state: PreservationAipStateModel): boolean {
    return !isNullOrUndefined(state.current);
  }

  @Selector()
  static isLoading(state: PreservationAipStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Action(PreservationAipAction.Download)
  download(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.Download): void {
    const fileName = "aip_" + action.id + ".zip";
    this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.file.download"), true);
    this.downloadService.download(`${this._urlResource}/${action.id}/${ApiActionEnum.DL}`, fileName).subscribe(() => {
      this.notificationService.showSuccess(MARK_AS_TRANSLATABLE("notification.aip.file.downloadWithSuccess"));
    });
  }

  @OverrideDefaultAction()
  @Action(PreservationAipAction.GetById)
  getById(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.GetById): Observable<Aip> {
    let reset = {};
    if (!action.keepCurrentContext) {
      reset = {
        current: undefined,
      };
    }
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
      ...reset,
    });

    return this.apiService.getById<Aip>(AipHelper.generateUrlResource(this.store, action.storagionNumber), action.id)
      .pipe(
        tap((model: Aip) => {
          ctx.dispatch(new PreservationAipAction.GetByIdSuccess(action, model));
        }),
        catchError(error => {
          ctx.dispatch(new PreservationAipAction.GetByIdFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(PreservationAipAction.Reindex)
  reindex(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.Reindex): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    const params = new HttpParams().set("level", "deep");
    return this.apiService.post<any>(`${this._urlResource}/${action.id}/${ApiActionEnum.REINDEX}`, {params: params})
      .pipe(
        tap(result => {
          ctx.dispatch(new PreservationAipAction.ReindexSuccess(action));
        }),
        catchError(error => {
          ctx.dispatch(new PreservationAipAction.ReindexFail(action));
          throw error;
        }),
      );
  }

  @Action(PreservationAipAction.ReindexSuccess)
  reindexSuccess(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.ReindexSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.reindex.success"), true);
  }

  @Action(PreservationAipAction.ReindexFail)
  reindexFail(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.ReindexFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.reindex.fail"));
  }

  @Action(PreservationAipAction.DeepChecksum)
  deepChecksum(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.DeepChecksum): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    const params = new HttpParams().set("level", "deep");
    return this.apiService.post<any>(`${this._urlResource}/${action.id}/${ApiActionEnum.CHECK_FIXITY}`, {params: params})
      .pipe(
        tap(result => {
          ctx.dispatch(new PreservationAipAction.DeepChecksumSuccess(action));
        }),
        catchError(error => {
          ctx.dispatch(new PreservationAipAction.DeepChecksumFail(action));
          throw error;
        }),
      );
  }

  @Action(PreservationAipAction.DeepChecksumSuccess)
  deepChecksumSuccess(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.DeepChecksumSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.deepChecksum.success"), true);
  }

  @Action(PreservationAipAction.DeepChecksumFail)
  deepChecksumFail(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.DeepChecksumFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.deepChecksum.fail"));
  }

  @Action(PreservationAipAction.SimpleChecksum)
  simpleChecksum(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.SimpleChecksum): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    const params = new HttpParams().set("level", "simple");
    return this.apiService.post<any>(`${this._urlResource}/${action.id}/${ApiActionEnum.CHECK_FIXITY}`, {params: params})
      .pipe(
        tap(result => {
          ctx.dispatch(new PreservationAipAction.SimpleChecksumSuccess(action));
        }),
        catchError(error => {
          ctx.dispatch(new PreservationAipAction.SimpleChecksumFail(action));
          throw error;
        }),
      );
  }

  @Action(PreservationAipAction.SimpleChecksumSuccess)
  simpleChecksumSuccess(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.SimpleChecksumSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.simpleChecksum.success"), true);
  }

  @Action(PreservationAipAction.SimpleChecksumFail)
  simpleChecksumFail(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.SimpleChecksumFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.simpleChecksum.fail"));
  }

  @Action(PreservationAipAction.Resume)
  resume(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.Resume): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    return this.apiService.post<string>(`${this._urlResource}/${action.id}/${ApiActionEnum.RESUME}`, null)
      .pipe(
        tap(result => {
          ctx.dispatch(new PreservationAipAction.ResumeSuccess(action));
        }),
        catchError(error => {
          ctx.dispatch(new PreservationAipAction.ResumeFail(action));
          throw error;
        }),
      );
  }

  @Action(PreservationAipAction.ResumeSuccess)
  resumeSuccess(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.ResumeSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.resume.success"), true);
  }

  @Action(PreservationAipAction.ResumeFail)
  resumeFail(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.ResumeFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.resume.fail"));
  }

  @Action(PreservationAipAction.GoToAip)
  goToAip(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.GoToAip): void {
    ctx.patchState({
      current: action.aip,
    });
    const pathAipDetail = RoutesEnum.preservationAip + urlSeparator + action.storagion_number + urlSeparator + PreservationPlanningRoutesEnum.aipDetail + AppRoutesEnum.separator;
    const path = [pathAipDetail, action.aip.resId];
    ctx.dispatch(new Navigate(path));
  }

  @Action(PreservationAipAction.Check)
  check(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.Check): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    return this.apiService.post<string>(`${this._urlResource}/${action.id}/${ApiActionEnum.CHECK}`, null)
      .pipe(
        tap(result => {
          ctx.dispatch(new PreservationAipAction.CheckSuccess(action));
        }),
        catchError(error => {
          ctx.dispatch(new PreservationAipAction.CheckFail(action));
          throw error;
        }),
      );
  }

  @Action(PreservationAipAction.CheckSuccess)
  checkSuccess(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.CheckSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.check.success"), true);
  }

  @Action(PreservationAipAction.CheckFail)
  checkFail(ctx: StateContext<PreservationAipStateModel>, action: PreservationAipAction.CheckFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.check.fail"));
  }
}
