import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Input,
  OnInit,
  Output,
} from "@angular/core";
import {
  FormBuilder,
  Validators,
} from "@angular/forms";
import {SharedAbstractFormPresentational} from "@app/shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {BaseFormDefinition} from "@app/shared/models/base-form-definition.model";
import {Dip} from "@models";
import {DipExtended} from "@preservation/dip/models/dip-extended.model";
import {BaseResourceApiEnum} from "@shared/enums/api.enum";
import {AccessLevelEnumHelper} from "@shared/enums/business/access-level-enum.helper";
import {
  PackageStatusEnum,
  PackageStatusEnumHelper,
} from "@shared/enums/business/package-status.enum";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
} from "@shared/enums/routes.enum";
import {Storage} from "@shared/models/storage.model";
import {BreakpointService} from "@shared/services/breakpoint.service";
import {sharedOrgUnitActionNameSpace} from "@shared/stores/organizational-unit/shared-organizational-unit.action";
import {SharedOrganizationalUnitState} from "@shared/stores/organizational-unit/shared-organizational-unit.state";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  KeyValue,
  ObservableUtil,
  Override,
  PropertyName,
  ResourceNameSpace,
  SolidifyValidator,
  urlSeparator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-dip-form",
  templateUrl: "./dip-form.presentational.html",
  styleUrls: ["./dip-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DipFormPresentational extends SharedAbstractFormPresentational<Dip> implements OnInit {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  accessEnumValues: KeyValue[] = AccessLevelEnumHelper.getListKeyValue();
  packageStatusEnumValues: KeyValue[] = PackageStatusEnumHelper.getListKeyValue();

  sharedOrgUnitActionNameSpace: ResourceNameSpace = sharedOrgUnitActionNameSpace;
  sharedOrgUnitState: typeof SharedOrganizationalUnitState = SharedOrganizationalUnitState;

  @Input()
  isReady: boolean = false;

  listStoragions: Storage[] = BaseResourceApiEnum.archivalStorageList;

  get packageStatusEnum(): typeof PackageStatusEnum {
    return PackageStatusEnum;
  }

  private readonly _orgUnitValueBS: BehaviorSubject<string | undefined> = new BehaviorSubject<string | undefined>(undefined);
  @Output("orgUnitChange")
  readonly orgUnitValueObs: Observable<string | undefined> = ObservableUtil.asObservable(this._orgUnitValueBS);

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              private readonly _fb: FormBuilder,
              public readonly breakpointService: BreakpointService) {
    super(_changeDetectorRef, _elementRef);
  }

  ngOnInit(): void {
    super.ngOnInit();
  }

  protected initNewForm(): void {
  }

  protected bindFormTo(dip: DipExtended): void {
    this.form = this._fb.group({
      [this.formDefinition.organizationalUnitId]: [dip.info.organizationalUnitId, [SolidifyValidator]],
      [this.formDefinition.resId]: [dip.resId, [SolidifyValidator]],
      [this.formDefinition.name]: [dip.info.name, [Validators.required, SolidifyValidator]],
      [this.formDefinition.description]: [dip.info.description, [SolidifyValidator]],
      [this.formDefinition.status]: [dip.info.status, [SolidifyValidator]],
      [this.formDefinition.access]: [dip.info.access, [SolidifyValidator]],
      [this.formDefinition.complianceLevel]: [dip.info.complianceLevel, [SolidifyValidator]],
    });
    this.isValidWhenDisable = this.form.valid;
  }

  protected treatmentBeforeSubmit(dip: DipExtended): DipExtended {
    const dipToSubmit: DipExtended = {
      info: {
        name: this.form.get(this.formDefinition.name).value,
        description: this.form.get(this.formDefinition.description).value,
      },
    };
    return dipToSubmit;
  }

  @Override()
  protected disableSpecificField(): void {
    this.form.get(this.formDefinition.resId).disable();
    this.form.get(this.formDefinition.status).disable();
    this.form.get(this.formDefinition.access).disable();
    this.form.get(this.formDefinition.complianceLevel).disable();
  }

  getAipPath(aipId: string, storagionIndex: number): string {
    return "#" + urlSeparator + AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.aip + urlSeparator + storagionIndex + urlSeparator + PreservationPlanningRoutesEnum.aipDetail + urlSeparator + aipId;
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() organizationalUnitId: string;
  @PropertyName() resId: string;
  @PropertyName() name: string;
  @PropertyName() description: string;
  @PropertyName() status: string;
  @PropertyName() access: string;
  @PropertyName() complianceLevel: string;
  @PropertyName() aip: string;
}
