import {
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {MatTabGroup} from "@angular/material/tabs";
import {
  ActivatedRoute,
  NavigationCancel,
  NavigationEnd,
  Router,
} from "@angular/router";
import {DipStateModel} from "@app/features/preservation/dip/stores/dip.state";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {environment} from "@environments/environment";
import {Dip} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {DipFormPresentational} from "@preservation/dip/components/presentationals/dip-form/dip-form.presentational";
import {DipTabEnum} from "@preservation/dip/enums/dip-tab.enum";
import {DipHelper} from "@preservation/dip/helpers/dip.helper";
import {
  PreservationDipAction,
  preservationDipActionNameSpace,
} from "@preservation/dip/stores/dip.action";
import {PreservationDipStatusHistoryAction} from "@preservation/dip/stores/status-history/dip-status-history.action";
import {DipStatusHistoryState} from "@preservation/dip/stores/status-history/dip-status-history.state";
import {SharedHistoryDialog} from "@shared/components/dialogs/shared-history/shared-history.dialog";
import {SharedAbstractDetailEditRoutable} from "@shared/components/routables/shared-abstract-detail-edit/shared-abstract-detail-edit.routable";
import {PackageStatusEnum} from "@shared/enums/business/package-status.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {
  DepositRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {ExtraButtonToolbar} from "@shared/models/extra-button-toolbar.model";
import {StatusHistoryDialog} from "@shared/models/status-history-dialog.model";
import {StatusHistory} from "@shared/models/status-history.model";
import {SecurityService} from "@shared/services/security.service";
import {Observable} from "rxjs";
import {tap} from "rxjs/internal/operators/tap";
import {
  distinctUntilChanged,
  filter,
} from "rxjs/operators";
import {
  isNullOrUndefined,
  MemoizedUtil,
  Override,
  OverrideProperty,
  QueryParameters,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-dip-detail-edit-routable",
  templateUrl: "./dip-detail-edit.routable.html",
  styleUrls: ["./dip-detail-edit.routable.scss"],
})
export class DipDetailEditRoutable extends SharedAbstractDetailEditRoutable<Dip, DipStateModel> implements OnInit, OnDestroy {
  historyObs: Observable<StatusHistory[]> = MemoizedUtil.select(this._store, DipStatusHistoryState, state => state.history);
  isLoadingHistoryObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, DipStatusHistoryState);
  queryParametersObs: Observable<QueryParameters> = MemoizedUtil.select(this._store, DipStatusHistoryState, state => state.queryParameters);

  @ViewChild("formPresentational")
  readonly formPresentational: DipFormPresentational;

  @ViewChild("matTabGroup")
  readonly matTabGroup: MatTabGroup;

  message: string;
  tabSelected: DipTabEnum;

  @OverrideProperty()
  readonly editAvailable: boolean = false;

  @OverrideProperty()
  readonly deleteAvailable: boolean = false;

  readonly KEY_PARAM_NAME: keyof Dip & string = undefined;

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _dialog: MatDialog,
              protected readonly _router: Router,
              private readonly _securityService: SecurityService) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.preservation_dip, preservationDipActionNameSpace, LocalStateEnum.preservation);
  }

  ngOnInit(): void {
    super.ngOnInit();

    this.retrieveCurrentModelWithUrl();

    this._computeCurrentTab();
    this.subscribe(this._router.events
      .pipe(
        filter(event => event instanceof NavigationCancel || event instanceof NavigationEnd),
        distinctUntilChanged(),
        tap(event => {
          this._computeCurrentTab();
          this.matTabGroup.selectedIndex = this.tabSelected;
        }),
      ),
    );
  }

  ngOnDestroy(): void {
    super.ngOnDestroy();
    this.cleanState();
  }

  private _computeCurrentTab(): void {
    const tabRouteSelected = DipHelper.getTabRouteSelected(this._route);
    this.tabSelected = DipHelper.getTabSelectedIndexWithRoute(tabRouteSelected);
  }

  getSubResourceWithParentId(id: string): void {
  }

  showHistory(): void {
    const dialogRef = this._dialog.open(SharedHistoryDialog, {
      width: environment.modalWidth,
      data: {
        parentId: null,
        resourceResId: this._resId,
        name: this._state,
        statusHistory: this.historyObs,
        isLoading: this.isLoadingHistoryObs,
        queryParametersObs: this.queryParametersObs,
        state: PreservationDipStatusHistoryAction,
      } as StatusHistoryDialog,
    });
  }

  listExtraButtons: ExtraButtonToolbar<Dip>[] = [
    {
      color: "primary",
      icon: IconNameEnum.download,
      displayCondition: current => !isNullOrUndefined(current) && !isNullOrUndefined(current.info) && current.info.status === PackageStatusEnum.COMPLETED,
      callback: () => this.download(),
      labelToTranslate: MARK_AS_TRANSLATABLE("preservation.dip.button.download"),
      order: 40,
    },
    {
      color: "primary",
      icon: IconNameEnum.resume,
      displayCondition: current => !isNullOrUndefined(current) && !isNullOrUndefined(current.info) && current.info.status === PackageStatusEnum.IN_ERROR,
      callback: () => this.resume(),
      labelToTranslate: MARK_AS_TRANSLATABLE("preservation.dip.button.resume"),
      order: 40,
    },
  ];

  @Override()
  edit(): void {
    const tabRouteSelected = DipHelper.getTabRouteSelected(this._route);
    this._store.dispatch(new Navigate([RoutesEnum.preservationDipDetail, this._resId, tabRouteSelected, DepositRoutesEnum.edit]));
  }

  navigateToTab(tabSelected: DipTabEnum): void {
    const tabRouteSelected = DipHelper.getTabRouteSelectedWithTabIndex(tabSelected);
    const path = [RoutesEnum.preservationDipDetail, this._resId, tabRouteSelected];
    this._store.dispatch(new Navigate(path));
  }

  download(): void {
    this._store.dispatch(new PreservationDipAction.Download(this._resId));
  }

  resume(): void {
    this._store.dispatch(new PreservationDipAction.Resume(this._resId));
  }
}
