import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
  ViewChild,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {DipStateModel} from "@app/features/preservation/dip/stores/dip.state";
import {environment} from "@environments/environment";
import {Dip} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {DipFormPresentational} from "@preservation/dip/components/presentationals/dip-form/dip-form.presentational";
import {DipDataFile} from "@preservation/dip/models/dip-data-file.model";
import {PreservationDipDataFileAction} from "@preservation/dip/stores/data-file/dip-data-file.action";
import {DipDataFileState} from "@preservation/dip/stores/data-file/dip-data-file.state";
import {PreservationDipDataFileStatusHistoryAction} from "@preservation/dip/stores/data-file/status-history/dip-data-file-status-history.action";
import {DipDataFileStatusHistoryState} from "@preservation/dip/stores/data-file/status-history/dip-data-file-status-history.state";
import {preservationDipActionNameSpace} from "@preservation/dip/stores/dip.action";
import {
  SharedFileDetailDialog,
  SharedFileDetailDialogData,
} from "@shared/components/dialogs/shared-file-detail/shared-file-detail.dialog";
import {SharedAbstractDetailEditRoutable} from "@shared/components/routables/shared-abstract-detail-edit/shared-abstract-detail-edit.routable";
import {ComplianceLevelEnumHelper} from "@shared/enums/business/compliance-level.enum";
import {
  DataFileStatusEnum,
  DataFileStatusEnumHelper,
} from "@shared/enums/business/data-file-status.enum";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {AppRoutesEnum} from "@shared/enums/routes.enum";
import {DataTableActions} from "@shared/models/data-table-actions.model";
import {DataTableColumns} from "@shared/models/data-table-columns.model";
import {StatusHistory} from "@shared/models/status-history.model";
import {Observable} from "rxjs";
import {
  CompositionState,
  isNullOrUndefined,
  MemoizedUtil,
  OrderEnum,
  QueryParameters,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-dip-file-routable",
  templateUrl: "./dip-file.routable.html",
  styleUrls: ["./dip-file.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DipFileRoutable extends SharedAbstractDetailEditRoutable<Dip, DipStateModel> implements OnInit {
  isLoadingDataFileObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, DipDataFileState);
  listDataFileObs: Observable<DipDataFile[]> = CompositionState.list(this._store, DipDataFileState);
  queryParametersObs: Observable<QueryParameters> = CompositionState.queryParameters(this._store, DipDataFileState);

  isLoadingHistoryObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, DipDataFileStatusHistoryState);
  dataFileHistoryObs: Observable<StatusHistory[]> = MemoizedUtil.select(this._store, DipDataFileStatusHistoryState, state => state.history);
  queryParametersHistoryObs: Observable<QueryParameters> = MemoizedUtil.select(this._store, DipDataFileStatusHistoryState, state => state.queryParameters);

  @ViewChild("formPresentational")
  readonly formPresentational: DipFormPresentational;

  columns: DataTableColumns<DipDataFile>[];
  actions: DataTableActions<DipDataFile>[];

  readonly KEY_PARAM_NAME: keyof Dip & string = undefined;

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _dialog: MatDialog) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.preservation_dip, preservationDipActionNameSpace, LocalStateEnum.preservation);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.getCurrentModelOnParent();

    this.columns = [
      {
        field: "fileName",
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.fileName"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isSortable: true,
        isFilterable: false,
      },
      {
        field: "creation.when" as any,
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.creation.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
      },
      {
        field: "status",
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.status"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isSortable: false,
        isFilterable: false,
        translate: true,
        filterEnum: DataFileStatusEnumHelper.getListKeyValue(),
      },
      {
        field: "complianceLevel",
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.complianceLevel"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isSortable: false,
        isFilterable: false,
        translate: true,
        component: DataTableComponentEnum.conformityLevelStar,
        filterEnum: ComplianceLevelEnumHelper.getListKeyValue(),
      },
    ];

    this.actions = [
      {
        logo: IconNameEnum.resume,
        callback: (dipDataFile: DipDataFile) => this.resumeDataFile(this._resId, dipDataFile),
        placeholder: MARK_AS_TRANSLATABLE("crud.list.action.resume"),
        displayOnCondition: current => isNullOrUndefined(current) || current.status === DataFileStatusEnum.IN_ERROR,
      },
      {
        logo: IconNameEnum.download,
        callback: (dipDataFile: DipDataFile) => this.downloadDataFile(this._resId, dipDataFile),
        placeholder: MARK_AS_TRANSLATABLE("crud.list.action.download"),
        displayOnCondition: (dipDataFile: DipDataFile) => true,
      },
    ];
  }

  private getCurrentModelOnParent(): void {
    this._resId = this._route.snapshot.parent.paramMap.get(AppRoutesEnum.paramIdWithoutPrefixParam);
    this.getDepositById(this._resId);
  }

  private getDepositById(id: string): void {
    // this._store.dispatch(ResourceActionHelper.getById(preservationDipActionNameSpace, id, true));
    this.getSubResourceWithParentId(id);
  }

  getSubResourceWithParentId(id: string): void {
    // const queryParameter = new QueryParameters();
    // MappingObjectUtil.set(queryParameter.search.searchItems, "relativeLocation", "/");
    this._store.dispatch(new PreservationDipDataFileAction.GetAll(id, null, true));
    // this._store.dispatch(new PreservationDipDataFileAction.GetListFolder(id));
  }

  onQueryParametersEvent(queryParameters: QueryParameters): void {
    this._store.dispatch(new PreservationDipDataFileAction.ChangeQueryParameters(this._resId, queryParameters, true));
    // this._changeDetector.detectChanges(); // Allow to display spinner the first time
  }

  refresh(): void {
    this._store.dispatch(new PreservationDipDataFileAction.Refresh(this._resId));
  }

  download($event: DipDataFile): void {
    this._store.dispatch(new PreservationDipDataFileAction.Download(this._resId, $event));
  }

  showDetail(dipDataFile: DipDataFile): void {
    const dialogRef = this._dialog.open(SharedFileDetailDialog, {
      data: {
        dataFile: dipDataFile,
        resId: dipDataFile.resId,
        parentId: this._resId,
        isReadonly: true,
        isLoading: this.isLoadingHistoryObs,
        statusHistory: this.dataFileHistoryObs,
        queryParametersObs: this.queryParametersHistoryObs,
        state: PreservationDipDataFileStatusHistoryAction,
        name: LocalStateEnum.preservation_dip,
        buttons: [
          {
            labelToTranslate: MARK_AS_TRANSLATABLE("deposit.file.button.download"),
            color: "primary",
            icon: IconNameEnum.download,
            callback: (current) => this.downloadDataFile(this._resId, dipDataFile),
            order: 2,
          },
          {
            labelToTranslate: MARK_AS_TRANSLATABLE("deposit.file.button.resume"),
            color: "primary",
            icon: IconNameEnum.resume,
            callback: (current) => this.resumeDataFile(this._resId, dipDataFile),
            displayCondition: current => current.status === DataFileStatusEnum.IN_ERROR,
            order: 3,
          },
        ],
      } as SharedFileDetailDialogData<DipDataFile>,
      width: environment.modalWidth,
    });
  }

  private downloadDataFile(parentId: string, dataFile: DipDataFile): void {
    this._store.dispatch(new PreservationDipDataFileAction.Download(parentId, dataFile));
  }

  private resumeDataFile(parentId: string, dataFile: DipDataFile): void {
    this._store.dispatch(new PreservationDipDataFileAction.Resume(parentId, dataFile));

  }
}
