import {HttpClient} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {
  defaultDipDataFileValue,
  DipDataFileState,
  DipDataFileStateModel,
} from "@app/features/preservation/dip/stores/data-file/dip-data-file.state";
import {
  PreservationDipAction,
  preservationDipActionNameSpace,
} from "@app/features/preservation/dip/stores/dip.action";
import {
  DipStatusHistoryState,
  DipStatusHistoryStateModel,
} from "@app/features/preservation/dip/stores/status-history/dip-status-history.state";
import {AccessResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {Dip} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiActionEnum} from "@shared/enums/api-action.enum";
import {
  DepositRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {DownloadService} from "@shared/services/download.service";
import {defaultStatusHistoryInitValue} from "@shared/stores/status-history/status-history.state";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  NotificationService,
  OverrideDefaultAction,
  ResourceState,
  ResourceStateModel,
  StoreUtil,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

export interface DipStateModel extends ResourceStateModel<Dip> {
  preservation_dip_dataFile: DipDataFileStateModel;
  isLoadingDataFile: boolean;
  preservation_dip_statusHistory: DipStatusHistoryStateModel;
}

@Injectable()
@State<DipStateModel>({
  name: LocalStateEnum.preservation_dip,
  defaults: {
    ...defaultResourceStateInitValue(),
    preservation_dip_dataFile: defaultDipDataFileValue(),
    isLoadingDataFile: false,
    preservation_dip_statusHistory: {...defaultStatusHistoryInitValue()},
  },
  children: [
    DipDataFileState,
    DipStatusHistoryState,
  ],
})
export class DipState extends ResourceState<DipStateModel, Dip> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions,
              protected httpClient: HttpClient,
              private downloadService: DownloadService) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationDipActionNameSpace,
      notificationResourceUpdateFailTextToTranslate: MARK_AS_TRANSLATABLE("dip.notification.resource.update.fail"),
    });
  }

  protected get _urlResource(): string {
    return AccessResourceApiEnum.dip;
  }

  @Selector()
  static isLoading(state: DipStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static currentTitle(state: DipStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return state.current.info.name;
  }

  @Selector()
  static isLoadingWithDependency(state: DipStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static isReadyToBeDisplayed(state: DipStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: DipStateModel): boolean {
    return true;
  }

  @OverrideDefaultAction()
  @Action(PreservationDipAction.UpdateSuccess)
  updateSuccess(ctx: StateContext<DipStateModel>, action: PreservationDipAction.UpdateSuccess): void {
    super.updateSuccess(ctx, action);

    ctx.dispatch([
      new Navigate([RoutesEnum.preservationDipDetail, action.model.resId, DepositRoutesEnum.data]),
    ]);
  }

  @Action(PreservationDipAction.Download)
  download(ctx: StateContext<DipStateModel>, action: PreservationDipAction.Download): void {
    const fileName = "dip_" + action.id + ".zip";
    this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.dip.file.download"), true);
    this.downloadService.download(`${this._urlResource}/${action.id}/${ApiActionEnum.DL}`, fileName).subscribe(() => {
      this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.dip.file.downloadWithSuccess"), true);
    });
  }

  @Action(PreservationDipAction.Resume)
  resume(ctx: StateContext<DipStateModel>, action: PreservationDipAction.Resume): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    return this.httpClient.post<string>(`${this._urlResource}/${action.id}/${ApiActionEnum.RESUME}`, null)
      .pipe(
        tap(result => {
          ctx.dispatch(new PreservationDipAction.ResumeSuccess(action));
        }),
        catchError(error => {
          ctx.dispatch(new PreservationDipAction.ResumeFail(action));
          throw error;
        }),
      );
  }

  @Action(PreservationDipAction.ResumeSuccess)
  resumeSuccess(ctx: StateContext<DipStateModel>, action: PreservationDipAction.ResumeSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.dip.action.resume.success"), true);
  }

  @Action(PreservationDipAction.ResumeFail)
  resumeFail(ctx: StateContext<DipStateModel>, action: PreservationDipAction.ResumeFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showError(MARK_AS_TRANSLATABLE("notification.dip.action.resume.fail"));
  }
}
