import {
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {PreservationJobExecutionAction} from "@app/features/preservation/job/stores/preservation-job-execution/preservation-job-execution.action";
import {PreservationJobExecutionState} from "@app/features/preservation/job/stores/preservation-job-execution/preservation-job-execution.state";
import {PreservationJobAction} from "@app/features/preservation/job/stores/preservation-job.action";
import {PreservationJobState} from "@app/features/preservation/job/stores/preservation-job.state";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {PreservationJob} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  ofActionCompleted,
  Store,
} from "@ngxs/store";
import {JobStatusEnum} from "@shared/enums/business/job-status.enum";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {PollingHelper} from "@shared/helpers/polling.helper";
import {JobExecution} from "@shared/models/business/job-execution.model";
import {DataTableColumns} from "@shared/models/data-table-columns.model";
import {Observable} from "rxjs";
import {
  distinctUntilChanged,
  filter,
  take,
  tap,
} from "rxjs/operators";
import {
  CompositionState,
  isNullOrUndefined,
  isTrue,
  MemoizedUtil,
  OrderEnum,
  QueryParameters,
  ResourceState,
  StringUtil,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-job-execution-container",
  templateUrl: "./job-execution-list.container.html",
  styleUrls: ["./job-execution-list.container.scss"],
})
export class JobExecutionListContainer extends SharedAbstractPresentational implements OnInit {
  isLoadingObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, PreservationJobExecutionState);
  listObs: Observable<JobExecution[]> = CompositionState.list(this._store, PreservationJobExecutionState);
  queryParametersObs: Observable<QueryParameters> = CompositionState.queryParameters(this._store, PreservationJobExecutionState);
  jobObs: Observable<PreservationJob> = ResourceState.current(this._store, PreservationJobState);
  columns: DataTableColumns<JobExecution>[];
  paramMessage: { name: string } = {name: StringUtil.stringEmpty};
  private readonly INTERVAL_REFRESH_IN_SECOND: number = 0.2;

  pollingToUpdateProgressBar: boolean = false;
  private _resId: string;

  constructor(private readonly _store: Store,
              private readonly _changeDetector: ChangeDetectorRef,
              private readonly _route: ActivatedRoute,
              private readonly _actions$: Actions,
              private readonly _dialog: MatDialog) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.defineColumns();
    this.retrieveCurrentModelWithUrl();

    this.subscribe(PollingHelper.startPollingObs({
      intervalRefreshInSecond: this.INTERVAL_REFRESH_IN_SECOND,
      filter: () => isTrue(this.pollingToUpdateProgressBar),
      actionToDo: () => this.getAll(),
    }));

    this.subscribe(this.jobObs.pipe(filter(job => !isNullOrUndefined(job))), job => this.paramMessage = {name: job.name});
    this.subscribe(this.listObs.pipe(
      distinctUntilChanged(),
      filter(list => !isNullOrUndefined(list))), list => {
      this.pollingToUpdateProgressBar = list.findIndex(jobExec => jobExec.status === JobStatusEnum.IN_PROGRESS || jobExec.status === JobStatusEnum.READY) !== -1;
    });
    this._changeDetector.detectChanges();
  }

  protected retrieveResIdFromUrl(): void {
    this._resId = this._route.snapshot.paramMap.get(AppRoutesEnum.paramIdWithoutPrefixParam);
    if (isNullOrUndefined(this._resId)) {
      this._resId = this._route.parent.snapshot.paramMap.get(AppRoutesEnum.paramIdWithoutPrefixParam);
    }
  }

  protected retrieveCurrentModelWithUrl(): void {
    this.retrieveResIdFromUrl();
    this.getById(this._resId);
  }

  protected getById(id: string): void {
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "runNumber",
        header: MARK_AS_TRANSLATABLE("preservation.jobExecution.table.header.runNumber"),
        type: FieldTypeEnum.string,
        order: OrderEnum.descending,
        isFilterable: false,
        isSortable: true,
        width: "80px",
      },
      {
        field: "status",
        header: MARK_AS_TRANSLATABLE("preservation.jobExecution.table.header.status"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
        translate: true,
        filterEnum: [
          {
            key: JobStatusEnum.COMPLETED,
            value: MARK_AS_TRANSLATABLE("preservation.jobExecution.jobStatusEnum.completed"),
          },
          {
            key: JobStatusEnum.IN_ERROR,
            value: MARK_AS_TRANSLATABLE("preservation.jobExecution.jobStatusEnum.inError"),
          },
          {
            key: JobStatusEnum.IN_PROGRESS,
            value: MARK_AS_TRANSLATABLE("preservation.jobExecution.jobStatusEnum.inProgress"),
          },
          {
            key: JobStatusEnum.PAUSED,
            value: MARK_AS_TRANSLATABLE("preservation.jobExecution.jobStatusEnum.paused"),
          },
          {
            key: JobStatusEnum.READY,
            value: MARK_AS_TRANSLATABLE("preservation.jobExecution.jobStatusEnum.ready"),
          },
        ],
        width: "160px",
      },
      {
        field: "completionStatus",
        header: MARK_AS_TRANSLATABLE("preservation.jobExecution.table.header.completionStatus"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
        component: DataTableComponentEnum.jobExecutionProgression,
      },
      {
        field: "startDate",
        header: MARK_AS_TRANSLATABLE("preservation.jobExecution.table.header.startDate"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
      },
      {
        field: "endDate",
        header: MARK_AS_TRANSLATABLE("preservation.jobExecution.table.header.endDate"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
      },
    ];
  }

  getAll(queryParameters?: QueryParameters): void {
    this._store.dispatch(new PreservationJobExecutionAction.GetAll(this._resId, queryParameters, true));
  }

  onQueryParametersEvent(queryParameters: QueryParameters): void {
    this.getAll(queryParameters);
    this._changeDetector.detectChanges(); // Allow to display spinner the first time
  }

  start(): void {
    this._store.dispatch(new PreservationJobAction.VerifyBeforeStart(this._resId));
    this.subscribe(this._actions$.pipe(ofActionCompleted(PreservationJobAction.StartSuccess))
      .pipe(
        take(1),
        tap(result => {
          if (result.result.successful === true) {
            this.getAll();
          }
        })));
  }

  resume(): void {
    this.subscribe(this._actions$.pipe(ofActionCompleted(PreservationJobAction.ResumeSuccess))
      .pipe(
        take(1),
        tap(result => {
          if (result.result.successful === true) {
            this.getAll();
          }
        })));
    this._store.dispatch(new PreservationJobAction.Resume(this._resId));
  }

  showExecutionDetail(jobExecution: JobExecution): void {
    const pathExecution = RoutesEnum.preservationPlanning + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.job + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.jobDetail + AppRoutesEnum.separator + this._resId + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.execution;
    const path = [pathExecution, jobExecution.resId];
    this._store.dispatch(new Navigate(path));
  }
}
