import {
  ChangeDetectorRef,
  Component,
  Inject,
  OnDestroy,
  OnInit,
} from "@angular/core";
import {
  MAT_DIALOG_DATA,
  MatDialog,
  MatDialogRef,
} from "@angular/material/dialog";
import {environment} from "@environments/environment";
import {Store} from "@ngxs/store";
import {PreservationAipStatusHistoryAction} from "@preservation/aip/stores/status-history/aip-status-history.action";
import {PreservationAipStatusHistoryState} from "@preservation/aip/stores/status-history/aip-status-history.state";
import {
  ErrorMessageWrapper,
  JobReportDetailErrorMessageDialog,
} from "@preservation/job/components/dialogs/job-report-detail-error-message/job-report-detail-error-message.dialog";
import {PreservationJobExecutionReportAction} from "@preservation/job/stores/preservation-job-execution/preservation-job-execution-report/preservation-job-execution-report.action";
import {PreservationJobExecutionReportState} from "@preservation/job/stores/preservation-job-execution/preservation-job-execution-report/preservation-job-execution-report.state";
import {SharedAbstractContainer} from "@shared/components/containers/shared-abstract/shared-abstract.container";
import {SharedHistoryDialog} from "@shared/components/dialogs/shared-history/shared-history.dialog";
import {JobReportAipStatusEnum} from "@shared/enums/business/job-report-aip-status.enum";
import {JobStatusEnumHelper} from "@shared/enums/business/job-status.enum";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {JobExecutionReportLine} from "@shared/models/business/job-execution-report-line.model";
import {DataTableActions} from "@shared/models/data-table-actions.model";
import {DataTableColumns} from "@shared/models/data-table-columns.model";
import {StatusHistoryDialog} from "@shared/models/status-history-dialog.model";
import {StatusHistory} from "@shared/models/status-history.model";
import {Observable} from "rxjs";
import {
  MemoizedUtil,
  OrderEnum,
  QueryParameters,
  StringUtil,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-job-report-detail-dialog",
  templateUrl: "./job-report-detail.dialog.html",
  styleUrls: ["./job-report-detail.dialog.scss"],
})
export class JobReportDetailDialog extends SharedAbstractContainer implements OnInit, OnDestroy {
  isLoadingObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, PreservationJobExecutionReportState);

  listObs: Observable<JobExecutionReportLine[]> = MemoizedUtil.select(this._store, PreservationJobExecutionReportState, state => state.lineReport);
  queryParametersObs: Observable<QueryParameters> = MemoizedUtil.select(this._store, PreservationJobExecutionReportState, state => state.queryParametersLines);

  historyObs: Observable<StatusHistory[]> = MemoizedUtil.select(this._store, PreservationAipStatusHistoryState, state => state.history);
  isLoadingHistoryObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, PreservationAipStatusHistoryState);
  queryParametersHistoryObs: Observable<QueryParameters> = MemoizedUtil.select(this._store, PreservationAipStatusHistoryState, state => state.queryParameters);

  paramMessage: { resId: string } = {resId: StringUtil.stringEmpty};

  columns: DataTableColumns<JobExecutionReportLine>[];
  actions: DataTableActions<JobExecutionReportLine>[] = [
    {
      logo: IconNameEnum.error,
      callback: (jobExecutionReportLine: JobExecutionReportLine) => this.showMessageError(jobExecutionReportLine),
      placeholder: MARK_AS_TRANSLATABLE("crud.list.action.errorMessage"),
      displayOnCondition: (jobExecutionReportLine: JobExecutionReportLine) => jobExecutionReportLine.status !== JobReportAipStatusEnum.PROCESSED,
    },
    {
      logo: IconNameEnum.history,
      callback: (jobExecutionReportLine: JobExecutionReportLine) => this.historyDataFile(jobExecutionReportLine),
      placeholder: MARK_AS_TRANSLATABLE("crud.list.action.history"),
      displayOnCondition: (jobExecutionReportLine: JobExecutionReportLine) => jobExecutionReportLine.status !== JobReportAipStatusEnum.ERROR,
    },
  ];

  constructor(private readonly _store: Store,
              private readonly _changeDetector: ChangeDetectorRef,
              protected readonly _dialogRef: MatDialogRef<JobReportDetailDialog>,
              protected readonly _dialog: MatDialog,
              @Inject(MAT_DIALOG_DATA) public data: ReportLinesWrapper) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.paramMessage = {resId: this.data.reportId};
    this.columns = [
      {
        field: "resId",
        header: MARK_AS_TRANSLATABLE("preservation.jobExecution.report.detail.table.header.aip"),
        type: FieldTypeEnum.string,
        order: OrderEnum.descending,
        isFilterable: false,
        isSortable: true,
      },
      {
        field: "status",
        header: MARK_AS_TRANSLATABLE("preservation.jobExecution.report.detail.table.header.status"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
        translate: true,
        filterEnum: JobStatusEnumHelper.getExecutionReportStatus(),
      },
      {
        field: "changeTime",
        header: MARK_AS_TRANSLATABLE("preservation.jobExecution.report.detail.table.header.startDate"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
      },
    ];
  }

  ngOnDestroy(): void {
    super.ngOnDestroy();
    this._store.dispatch(new PreservationJobExecutionReportAction.CleanQueryParametersLines());
  }

  onQueryParametersEvent(queryParameters: QueryParameters): void {
    this._store.dispatch(new PreservationJobExecutionReportAction.ChangeQueryParametersLines(queryParameters, this.data.executionId, this.data.preservationJobId, this.data.reportId));
    this._changeDetector.detectChanges();
  }

  showExecutionDetail(jobExecutionReportLine: JobExecutionReportLine): void {
    this._store.dispatch(new PreservationJobExecutionReportAction.GoToAip(jobExecutionReportLine.resId));
    this._dialogRef.close();
  }

  showMessageError(jobExecutionReportLine: JobExecutionReportLine): void {
    this._dialog.open(JobReportDetailErrorMessageDialog, {
      data: {
        errorMessage: jobExecutionReportLine.errorMessage,
        resId: jobExecutionReportLine.resId,
      } as ErrorMessageWrapper,
    });
  }

  historyDataFile(jobExecutionReportLine: JobExecutionReportLine): void {
    this._dialog.open(SharedHistoryDialog, {
      width: environment.modalWidth,
      data: {
        parentId: null,
        resourceResId: jobExecutionReportLine.resId,
        name: LocalStateEnum.preservation_aip,
        statusHistory: this.historyObs,
        isLoading: this.isLoadingHistoryObs,
        queryParametersObs: this.queryParametersHistoryObs,
        state: PreservationAipStatusHistoryAction,
      } as StatusHistoryDialog,
    });
  }
}

export interface ReportLinesWrapper {
  linesReport: JobExecutionReportLine[];
  preservationJobId: string;
  executionId: string;
  reportId: string;
}
