import {
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  ofActionCompleted,
  Select,
  Store,
} from "@ngxs/store";
import {
  JobReportDetailDialog,
  ReportLinesWrapper,
} from "@preservation/job/components/dialogs/job-report-detail/job-report-detail.dialog";
import {PreservationJobExecutionReportAction} from "@preservation/job/stores/preservation-job-execution/preservation-job-execution-report/preservation-job-execution-report.action";
import {PreservationJobExecutionAction} from "@preservation/job/stores/preservation-job-execution/preservation-job-execution.action";
import {PreservationJobExecutionState} from "@preservation/job/stores/preservation-job-execution/preservation-job-execution.state";
import {SharedAbstractRoutable} from "@shared/components/routables/shared-abstract/shared-abstract.routable";
import {JobStatusEnum} from "@shared/enums/business/job-status.enum";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {PollingHelper} from "@shared/helpers/polling.helper";
import {JobExecutionReport} from "@shared/models/business/job-execution-report.model";
import {JobExecution} from "@shared/models/business/job-execution.model";
import {DataTableColumns} from "@shared/models/data-table-columns.model";
import {ExtraButtonToolbar} from "@shared/models/extra-button-toolbar.model";
import {LocalStateModel} from "@shared/models/local-state.model";
import {Observable} from "rxjs";
import {
  take,
  tap,
} from "rxjs/operators";
import {
  BaseResource,
  CompositionState,
  isNullOrUndefined,
  OrderEnum,
  QueryParameters,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-job-execution-detail-routable",
  templateUrl: "./job-execution-detail.routable.html",
  styleUrls: ["./job-execution-detail.routable.scss"],
})
export class JobExecutionDetailRoutable extends SharedAbstractRoutable implements OnInit, OnDestroy {

  currentObs: Observable<JobExecution> = CompositionState.current(this._store, PreservationJobExecutionState);
  @Select(PreservationJobExecutionState.isReadyToBeDisplayed) isReadyToBeDisplayedObs: Observable<boolean>;
  @Select(PreservationJobExecutionState.isLoading) isLoadingObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state.preservation.preservation_job.preservation_job_execution.preservation_job_execution_report.reports) listReportObs: Observable<JobExecutionReport[]>;
  @Select((state: LocalStateModel) => state.preservation.preservation_job.preservation_job_execution.preservation_job_execution_report.queryParameters) queryParametersObs: Observable<QueryParameters>;

  readonly editAvailable: boolean = false;
  readonly deleteAvailable: boolean = false;

  columns: DataTableColumns<BaseResource & JobExecutionReport>[] = [
    {
      field: "executionNumber",
      header: MARK_AS_TRANSLATABLE("preservation.jobExecution.report.table.header.executionNumber"),
      type: FieldTypeEnum.string,
      order: OrderEnum.descending,
      isFilterable: false,
      isSortable: true,
    },
    {
      field: "processedItems",
      header: MARK_AS_TRANSLATABLE("preservation.jobExecution.report.table.header.processedItems"),
      type: FieldTypeEnum.string,
      order: OrderEnum.none,
      isFilterable: false,
      isSortable: false,
    },
    {
      field: "ignoredItems",
      header: MARK_AS_TRANSLATABLE("preservation.jobExecution.report.table.header.ignoredItems"),
      type: FieldTypeEnum.string,
      order: OrderEnum.none,
      isFilterable: false,
      isSortable: false,
    },
    {
      field: "inErrorItems",
      header: MARK_AS_TRANSLATABLE("preservation.jobExecution.report.table.header.inErrorItems"),
      type: FieldTypeEnum.string,
      order: OrderEnum.none,
      isFilterable: false,
      isSortable: false,
    },
  ];

  private _preservationJobId: string;
  private _resId: string;

  constructor(private readonly _store: Store,
              private readonly _route: ActivatedRoute,
              protected readonly _changeDetector: ChangeDetectorRef,
              private readonly _actions$: Actions,
              private readonly _dialog: MatDialog) {
    super();
    this.retrieveCurrentModelWithUrl();
  }

  protected retrieveCurrentModelWithUrl(): void {
    this.retrieveResIdFromUrl();
    this._store.dispatch(new PreservationJobExecutionAction.GetById(this._preservationJobId, this._resId));
  }

  protected retrieveResIdFromUrl(): void {
    this._resId = this._route.snapshot.paramMap.get(AppRoutesEnum.paramIdExecutionWithoutPrefixParam);
    this._preservationJobId = this._route.snapshot.parent.paramMap.get(AppRoutesEnum.paramIdWithoutPrefixParam);
  }

  ngOnInit(): void {
    super.ngOnInit();
  }

  ngOnDestroy(): void {
    this._store.dispatch(new PreservationJobExecutionReportAction.CleanQueryParameters());
  }

  backToList(): void {
    const pathExecution = RoutesEnum.preservationPlanning + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.job + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.jobDetail;
    const path = [pathExecution, this._preservationJobId];
    this._store.dispatch(new Navigate(path));
  }

  listExtraButtons(): ExtraButtonToolbar<JobExecution>[] {
    return [
      {
        color: "primary",
        icon: IconNameEnum.resume,
        labelToTranslate: MARK_AS_TRANSLATABLE("preservation.jobExecution.button.resume"),
        callback: () => this.resume(),
        displayCondition: () => !isNullOrUndefined(CompositionState.currentSnapshot(this._store, PreservationJobExecutionState)) && CompositionState.currentSnapshot(this._store, PreservationJobExecutionState).status === JobStatusEnum.IN_ERROR,
        order: 40,
      },
      {
        color: "primary",
        icon: IconNameEnum.refresh,
        labelToTranslate: MARK_AS_TRANSLATABLE("preservation.jobExecution.button.refresh"),
        callback: () => this.refresh(),
        order: 41,
      },
    ];
  }

  private resume(): void {
    this._store.dispatch(new PreservationJobExecutionAction.Resume(this._preservationJobId, this._resId));
    this.subscribe(this._actions$.pipe(ofActionCompleted(PreservationJobExecutionAction.ResumeSuccess))
      .pipe(
        take(1),
        tap(result => {
          if (result.result.successful) {
            this.refresh();
          }
        }),
      ));

    this.subscribe(PollingHelper.startPollingObs({
      intervalRefreshInSecond: 2,
      continueUntil: () => isNullOrUndefined(CompositionState.currentSnapshot(this._store, PreservationJobExecutionState)) || (CompositionState.currentSnapshot(this._store, PreservationJobExecutionState).status !== JobStatusEnum.IN_ERROR &&
        CompositionState.currentSnapshot(this._store, PreservationJobExecutionState).status !== JobStatusEnum.COMPLETED),
      actionToDo: () => this.refresh(),
    }));
  }

  showReportDetail(report: JobExecutionReport): void {
    this.subscribe(this._store.dispatch(new PreservationJobExecutionReportAction.ReportLines(this._resId, this._preservationJobId, report.resId, null))
      .pipe(
        tap((state: LocalStateModel) => {
          const reportLines = state.preservation.preservation_job.preservation_job_execution.preservation_job_execution_report.lineReport;
          const dialogRef = this._dialog.open(JobReportDetailDialog, {
            data: {
              linesReport: reportLines,
              preservationJobId: this._preservationJobId,
              executionId: this._resId,
              reportId: report.resId,
            } as ReportLinesWrapper,
          });
        }),
      ));
  }

  private refresh(): void {
    this._store.dispatch(new PreservationJobExecutionAction.GetById(this._preservationJobId, this._resId));
  }

  onQueryParametersEvent(queryParameters: QueryParameters): void {
    this._store.dispatch(new PreservationJobExecutionReportAction.ChangeQueryParameters(queryParameters, this._resId, this._preservationJobId));
    this._changeDetector.detectChanges();
  }
}
