import {
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {JobHelper} from "@app/features/preservation/job/helper/job.helper";
import {
  PreservationJobAction,
  preservationJobActionNameSpace,
} from "@app/features/preservation/job/stores/preservation-job.action";
import {PreservationJobStateModel} from "@app/features/preservation/job/stores/preservation-job.state";
import {PreservationJob} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {PreservationJobExtended} from "@preservation/job/models/preservation-job-extended.model";
import {SharedAbstractListRoutable} from "@shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {JobRecurrence} from "@shared/models/business/job-recurrence.model";
import {DataTableActions} from "@shared/models/data-table-actions.model";
import {RouterExtService} from "@shared/services/router-ext.service";
import {
  isNullOrUndefined,
  OrderEnum,
  Override,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";
import JobRecurrenceEnum = JobRecurrence.JobRecurrenceEnum;

@Component({
  selector: "dlcm-job-list-routable",
  templateUrl: "../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
})
export class JobListRoutable extends SharedAbstractListRoutable<PreservationJobExtended, PreservationJobStateModel> implements OnInit {
  readonly KEY_CREATE_BUTTON: string = MARK_AS_TRANSLATABLE("preservation.job.button.new");
  readonly KEY_REFRESH_BUTTON: string = MARK_AS_TRANSLATABLE("preservation.job.button.refresh");
  readonly KEY_BACK_BUTTON: string | undefined = MARK_AS_TRANSLATABLE("preservation.job.button.back");
  readonly KEY_PARAM_NAME: keyof PreservationJob & string = "name";

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, LocalStateEnum.preservation_job, preservationJobActionNameSpace, {
      listExtraButtons: [
        {
          color: "primary",
          icon: IconNameEnum.init,
          labelToTranslate: MARK_AS_TRANSLATABLE("preservation.job.button.init"),
          callback: () => this.init(),
          order: 40,
        },
      ],
    }, LocalStateEnum.preservation);
  }

  conditionDisplayEditButton(model: PreservationJobExtended | undefined): boolean {
    return true;
  }

  conditionDisplayDeleteButton(model: PreservationJobExtended | undefined): boolean {
    return true;
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.defineColumns();
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "name",
        header: MARK_AS_TRANSLATABLE("preservation.job.table.header.name"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "jobRecurrence" as any,
        header: MARK_AS_TRANSLATABLE("preservation.job.table.header.jobRecurrence"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        translate: true,
        filterEnum: JobHelper.getRecurrenceEnum(),
      },
      {
        field: "jobType",
        header: MARK_AS_TRANSLATABLE("preservation.job.table.header.jobType"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        translate: true,
        filterEnum: JobHelper.getTypeEnum(),
      },
      {
        field: "enable" as any,
        header: MARK_AS_TRANSLATABLE("preservation.job.table.header.enable"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        translate: true,
        filterEnum: [
          {
            key: "true",
            value: MARK_AS_TRANSLATABLE("preservation.job.state.enabled"),
          },
          {
            key: "false",
            value: MARK_AS_TRANSLATABLE("preservation.job.state.disabled"),
          },
        ],
      },
      {
        field: "lastUpdate.when" as any,
        header: MARK_AS_TRANSLATABLE("preservation.job.table.header.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.descending,
        isFilterable: true,
        isSortable: true,
      },
    ];
  }

  @Override()
  protected defineActions(): DataTableActions<PreservationJob>[] {
    return [
      ...super.defineActions(),
      {
        logo: IconNameEnum.run,
        callback: (model: PreservationJob) => this.start(model),
        placeholder: MARK_AS_TRANSLATABLE("crud.list.action.start"),
        displayOnCondition: (model: PreservationJob) => isNullOrUndefined(model) || (model.jobRecurrence !== JobRecurrenceEnum.ONCE),
      },
    ];
  }

  back(): void {
    this._store.dispatch(new Navigate([RoutesEnum.preservationPlanning]));
  }

  init(): void {
    this._store.dispatch(new PreservationJobAction.Init());
  }

  private start(job: PreservationJob): void {
    this._store.dispatch(new PreservationJobAction.VerifyBeforeStart(job.resId));
  }
}
