import {Injectable} from "@angular/core";
import {
  PreservationJobExecutionAction,
  preservationJobExecutionActionNameSpace,
} from "@app/features/preservation/job/stores/preservation-job-execution/preservation-job-execution.action";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {PreservationJobExecutionReportAction} from "@preservation/job/stores/preservation-job-execution/preservation-job-execution-report/preservation-job-execution-report.action";
import {
  defaultReportInitValue,
  PreservationJobExecutionReportState,
  PreservationJobExecutionReportStateModel,
} from "@preservation/job/stores/preservation-job-execution/preservation-job-execution-report/preservation-job-execution-report.state";
import {PreservationJobStateModel} from "@preservation/job/stores/preservation-job.state";
import {ApiActionEnum} from "@shared/enums/api-action.enum";
import {ApiResourceNameEnum} from "@shared/enums/api-resource-name.enum";
import {PreservationPlanningResourceApiEnum} from "@shared/enums/api.enum";
import {JobExecutionReport} from "@shared/models/business/job-execution-report.model";
import {JobExecution} from "@shared/models/business/job-execution.model";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  CompositionState,
  CompositionStateModel,
  defaultCompositionStateInitValue,
  isNullOrUndefined,
  NotificationService,
  OverrideDefaultAction,
  SolidifyStateError,
  StoreUtil,
  MARK_AS_TRANSLATABLE,
  urlSeparator,
} from "solidify-frontend";

export const defaultJobExecutionInitValue: () => PreservationJobExecutionStateModel = () =>
  ({
    ...defaultCompositionStateInitValue(),
    preservation_job_execution_report: {...defaultReportInitValue()},
  });

export interface PreservationJobExecutionStateModel extends CompositionStateModel<JobExecution> {
  preservation_job_execution_report: PreservationJobExecutionReportStateModel;
}

@Injectable()
@State<PreservationJobExecutionStateModel>({
  name: LocalStateEnum.preservation_job_execution,
  defaults: {
    ...defaultCompositionStateInitValue(),
    preservation_job_execution_report: {...defaultReportInitValue()},
  },
  children: [
    PreservationJobExecutionReportState,
  ],
})
export class PreservationJobExecutionState extends CompositionState<PreservationJobExecutionStateModel, JobExecution> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationJobExecutionActionNameSpace,
      resourceName: ApiResourceNameEnum.PRES_JOB_EXECUTION,
    });
  }

  @Selector()
  static currentTitle(state: PreservationJobExecutionStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return "Run " + state.current.runNumber;
  }

  @Selector()
  static isReadyToBeDisplayed(state: PreservationJobStateModel): boolean {
    return !isNullOrUndefined(state.current);
  }

  @Selector()
  static isLoading(state: PreservationJobStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static listReport(state: PreservationJobExecutionStateModel): JobExecutionReport[] {
    return state.preservation_job_execution_report.reports;
  }

  protected get _urlResource(): string {
    return PreservationPlanningResourceApiEnum.preservationJobs;
  }

  @Action(PreservationJobExecutionAction.Resume)
  resume(ctx: StateContext<PreservationJobExecutionStateModel>, action: PreservationJobExecutionAction.Resume): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return this.apiService.post<string>(this._urlResource + urlSeparator + action.parentId + urlSeparator + ApiResourceNameEnum.PRES_JOB_EXECUTION + urlSeparator + action.jobExecutionId + urlSeparator + ApiActionEnum.RESUME)
      .pipe(
        tap(result => {
          ctx.dispatch(new PreservationJobExecutionAction.ResumeSuccess(action));
        }),
        catchError(error => {
          ctx.dispatch(new PreservationJobExecutionAction.ResumeFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(PreservationJobExecutionAction.ResumeSuccess)
  resumeSuccess(ctx: StateContext<PreservationJobExecutionStateModel>, action: PreservationJobExecutionAction.ResumeSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showSuccess(MARK_AS_TRANSLATABLE("preservation.jobExecution.notification.resume.success"));
  }

  @Action(PreservationJobExecutionAction.ResumeFail)
  resumeFail(ctx: StateContext<PreservationJobExecutionStateModel>, action: PreservationJobExecutionAction.ResumeFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showError(MARK_AS_TRANSLATABLE("preservation.jobExecution.notification.resume.fail"));
  }

  @Action(PreservationJobExecutionAction.Start)
  start(ctx: StateContext<PreservationJobExecutionStateModel>, action: PreservationJobExecutionAction.Start): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return this.apiService.post<string>(this._urlResource + urlSeparator + action.parentId + urlSeparator + ApiResourceNameEnum.PRES_JOB_EXECUTION + urlSeparator + action.jobExecution.resId + urlSeparator + ApiActionEnum.START)
      .pipe(
        tap(result => {
          ctx.dispatch(new PreservationJobExecutionAction.StartSuccess(action));
        }),
        catchError(error => {
          ctx.dispatch(new PreservationJobExecutionAction.StartFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(PreservationJobExecutionAction.StartSuccess)
  startSuccess(ctx: StateContext<PreservationJobExecutionStateModel>, action: PreservationJobExecutionAction.StartSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showSuccess(MARK_AS_TRANSLATABLE("preservation.jobExecution.notification.start.success"));
  }

  @Action(PreservationJobExecutionAction.StartFail)
  startFail(ctx: StateContext<PreservationJobExecutionStateModel>, action: PreservationJobExecutionAction.StartFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showError(MARK_AS_TRANSLATABLE("preservation.jobExecution.notification.start.fail"));
  }

  @OverrideDefaultAction()
  @Action(PreservationJobExecutionAction.GetByIdSuccess)
  getByIdSuccess(ctx: StateContext<PreservationJobExecutionStateModel>, action: PreservationJobExecutionAction.GetByIdSuccess): void {
    ctx.patchState({
      current: action.model,
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    ctx.dispatch(new PreservationJobExecutionReportAction.Report(action.model.resId, action.model.jobId));
  }
}
