import {Injectable} from "@angular/core";
import {
  defaultJobExecutionInitValue,
  PreservationJobExecutionState,
  PreservationJobExecutionStateModel,
} from "@app/features/preservation/job/stores/preservation-job-execution/preservation-job-execution.state";
import {
  PreservationJobAction,
  preservationJobActionNameSpace,
} from "@app/features/preservation/job/stores/preservation-job.action";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {environment} from "@environments/environment";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {PreservationJobExtended} from "@preservation/job/models/preservation-job-extended.model";
import {ApiActionEnum} from "@shared/enums/api-action.enum";
import {ApiResourceNameEnum} from "@shared/enums/api-resource-name.enum";
import {PreservationPlanningResourceApiEnum} from "@shared/enums/api.enum";
import {JobStatusEnum} from "@shared/enums/business/job-status.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {JobExecution} from "@shared/models/business/job-execution.model";
import {JobRecurrence} from "@shared/models/business/job-recurrence.model";
import {JobType} from "@shared/models/business/job-type.model";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  defaultResourceStateInitValue,
  isEmptyArray,
  isNullOrUndefined,
  MappingObjectUtil,
  NotificationService,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
  SolidifyStateError,
  StoreUtil,
  MARK_AS_TRANSLATABLE,
  urlSeparator,
} from "solidify-frontend";

export interface PreservationJobStateModel extends ResourceStateModel<PreservationJobExtended> {
  listJobTypes: JobType[] | undefined;
  listJobRecurrences: JobRecurrence[] | undefined;
  preservation_job_execution: PreservationJobExecutionStateModel | undefined;
}

@Injectable()
@State<PreservationJobStateModel>({
  name: LocalStateEnum.preservation_job,
  defaults: {
    ...defaultResourceStateInitValue(),
    listJobTypes: undefined,
    listJobRecurrences: undefined,
    preservation_job_execution: {...defaultJobExecutionInitValue()},
  },
  children: [
    PreservationJobExecutionState,
  ],
})
export class PreservationJobState extends ResourceState<PreservationJobStateModel, PreservationJobExtended> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationJobActionNameSpace,
      routeRedirectUrlAfterSuccessCreateAction: (resId: string) => RoutesEnum.preservationJobDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessUpdateAction: (resId: string) => RoutesEnum.preservationJobDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessDeleteAction: RoutesEnum.preservationJob,
      notificationResourceCreateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("preservation.job.notification.resource.create"),
      notificationResourceDeleteSuccessTextToTranslate: MARK_AS_TRANSLATABLE("preservation.job.notification.resource.delete"),
      notificationResourceUpdateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("preservation.job.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return PreservationPlanningResourceApiEnum.preservationJobs;
  }

  @Selector()
  static isLoading(state: PreservationJobStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: PreservationJobStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static currentTitle(state: PreservationJobStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return state.current.name;
  }

  @Selector()
  static isReadyToBeDisplayed(state: PreservationJobStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: PreservationJobStateModel): boolean {
    return true;
  }

  @Action(PreservationJobAction.Init)
  init(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.Init): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return this.apiService.post<string>(this._urlResource + urlSeparator + ApiActionEnum.INIT).pipe(
      tap(result => {
        ctx.dispatch(new PreservationJobAction.InitSuccess(action));
      }),
      catchError(error => {
        ctx.dispatch(new PreservationJobAction.InitFail(action));
        throw new SolidifyStateError(this, error);
      }),
    );
  }

  @Action(PreservationJobAction.InitSuccess)
  initSuccess(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.InitSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showSuccess(MARK_AS_TRANSLATABLE("preservation.job.notification.init.success"));
  }

  @Action(PreservationJobAction.InitFail)
  initFail(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.InitFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showError(MARK_AS_TRANSLATABLE("preservation.job.notification.init.fail"));
  }

  @Action(PreservationJobAction.VerifyBeforeStart)
  verifyBeforeStart(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.VerifyBeforeStart): Observable<any> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    const JOB_EXEC_STATUS: keyof JobExecution = "status";
    const queryParameters = new QueryParameters(environment.minimalPageSizeToRetrievePaginationInfo);
    MappingObjectUtil.set(queryParameters.search.searchItems, JOB_EXEC_STATUS, JobStatusEnum.IN_PROGRESS);

    return this.apiService.get<JobExecution>(this._urlResource + urlSeparator + action.resId + urlSeparator + ApiResourceNameEnum.PRES_JOB_EXECUTION, queryParameters)
      .pipe(
        tap(result => {
          if (isEmptyArray(result._data)) {
            ctx.dispatch(new PreservationJobAction.VerifyBeforeStartSuccess(action));
          } else {
            ctx.dispatch(new PreservationJobAction.VerifyBeforeStartFail(action));
          }
        }),
        catchError(error => {
          ctx.dispatch(new PreservationJobAction.StartFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(PreservationJobAction.VerifyBeforeStartSuccess)
  verifyBeforeStartSuccess(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.VerifyBeforeStartSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    ctx.dispatch(new PreservationJobAction.Start((action.parentAction as PreservationJobAction.VerifyBeforeStart).resId));
  }

  @Action(PreservationJobAction.VerifyBeforeStartFail)
  verifyBeforeStartFail(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.VerifyBeforeStartFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showError(MARK_AS_TRANSLATABLE("preservation.job.notification.verify.fail"));
  }

  @Action(PreservationJobAction.Start)
  start(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.Start): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return this.apiService.post<string>(this._urlResource + urlSeparator + action.resId + urlSeparator + ApiActionEnum.START).pipe(
      tap(result => {
        ctx.dispatch(new PreservationJobAction.StartSuccess(action));
      }),
      catchError(error => {
        ctx.dispatch(new PreservationJobAction.StartFail(action));
        throw new SolidifyStateError(this, error);
      }),
    );
  }

  @Action(PreservationJobAction.StartSuccess)
  startSuccess(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.StartSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showSuccess(MARK_AS_TRANSLATABLE("preservation.job.notification.start.success"));
  }

  @Action(PreservationJobAction.StartFail)
  startFail(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.StartFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showError(MARK_AS_TRANSLATABLE("preservation.job.notification.start.fail"));
  }

  @Action(PreservationJobAction.Resume)
  resume(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.Resume): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return this.apiService.post<string>(this._urlResource + urlSeparator + action.resId + urlSeparator + ApiActionEnum.RESUME).pipe(
      tap(result => {
        ctx.dispatch(new PreservationJobAction.ResumeSuccess(action));
      }),
      catchError(error => {
        ctx.dispatch(new PreservationJobAction.ResumeFail(action));
        throw new SolidifyStateError(this, error);
      }),
    );
  }

  @Action(PreservationJobAction.ResumeSuccess)
  resumeSuccess(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.ResumeSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showSuccess(MARK_AS_TRANSLATABLE("preservation.job.notification.resume.success"));
  }

  @Action(PreservationJobAction.ResumeFail)
  resumeFail(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.ResumeFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showError(MARK_AS_TRANSLATABLE("preservation.job.notification.resume.fail"));
  }

  @Action(PreservationJobAction.ListJobTypes)
  listJobTypes(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.ListJobTypes): Observable<JobType[]> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return this.apiService.getList<JobType>(this._urlResource + urlSeparator + ApiActionEnum.LIST_JOB_TYPE, null).pipe(
      tap(result => {
        ctx.dispatch(new PreservationJobAction.ListJobTypesSuccess(action, result));
      }),
      catchError(error => {
        ctx.dispatch(new PreservationJobAction.ListJobTypesFail(action));
        throw new SolidifyStateError(this, error);
      }),
    );
  }

  @Action(PreservationJobAction.ListJobTypesSuccess)
  listJobTypesSuccess(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.ListJobTypesSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
      listJobTypes: action.listJobTypes,
    });
  }

  @Action(PreservationJobAction.ListJobTypesFail)
  listJobTypesFail(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.ListJobTypesFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(PreservationJobAction.ListJobRecurrences)
  listJobRecurrences(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.ListJobRecurrences): Observable<JobRecurrence[]> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return this.apiService.getList<JobRecurrence>(this._urlResource + urlSeparator + ApiActionEnum.LIST_JOB_RECURRENCE, null).pipe(
      tap(result => {
        ctx.dispatch(new PreservationJobAction.ListJobRecurrencesSuccess(action, result));
      }),
      catchError(error => {
        ctx.dispatch(new PreservationJobAction.ListJobRecurrencesFail(action));
        throw new SolidifyStateError(this, error);
      }),
    );
  }

  @Action(PreservationJobAction.ListJobRecurrencesSuccess)
  listJobRecurrencesSuccess(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.ListJobRecurrencesSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
      listJobRecurrences: action.listJobRecurrences,
    });
  }

  @Action(PreservationJobAction.ListJobRecurrencesFail)
  listJobRecurrencesFail(ctx: StateContext<PreservationJobStateModel>, action: PreservationJobAction.ListJobRecurrencesFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }
}
