import {
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {MatTabGroup} from "@angular/material/tabs";
import {
  ActivatedRoute,
  NavigationCancel,
  NavigationEnd,
  Router,
} from "@angular/router";
import {SipStateModel} from "@app/features/preservation/sip/stores/sip.state";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {environment} from "@environments/environment";
import {Sip} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SipFormPresentational} from "@preservation/sip/components/presentationals/sip-form/sip-form.presentational";
import {SipTabEnum} from "@preservation/sip/enums/sip-tab.enum";
import {SipHelper} from "@preservation/sip/helpers/sip.helper";
import {
  PreservationSipAction,
  preservationSipActionNameSpace,
} from "@preservation/sip/stores/sip.action";
import {PreservationSipStatusHistoryAction} from "@preservation/sip/stores/status-history/sip-status-history.action";
import {SipStatusHistoryState} from "@preservation/sip/stores/status-history/sip-status-history.state";
import {SharedHistoryDialog} from "@shared/components/dialogs/shared-history/shared-history.dialog";
import {SharedAbstractDetailEditRoutable} from "@shared/components/routables/shared-abstract-detail-edit/shared-abstract-detail-edit.routable";
import {PackageStatusEnum} from "@shared/enums/business/package-status.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {ExtraButtonToolbar} from "@shared/models/extra-button-toolbar.model";
import {StatusHistoryDialog} from "@shared/models/status-history-dialog.model";
import {StatusHistory} from "@shared/models/status-history.model";
import {SecurityService} from "@shared/services/security.service";
import {Observable} from "rxjs";
import {tap} from "rxjs/internal/operators/tap";
import {
  distinctUntilChanged,
  filter,
} from "rxjs/operators";
import {
  isNullOrUndefined,
  MemoizedUtil,
  OverrideProperty,
  QueryParameters,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-sip-detail-edit-routable",
  templateUrl: "./sip-detail-edit.routable.html",
  styleUrls: ["./sip-detail-edit.routable.scss"],
})
export class SipDetailEditRoutable extends SharedAbstractDetailEditRoutable<Sip, SipStateModel> implements OnInit, OnDestroy {
  historyObs: Observable<StatusHistory[]> = MemoizedUtil.select(this._store, SipStatusHistoryState, state => state.history);
  isLoadingHistoryObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, SipStatusHistoryState);
  queryParametersObs: Observable<QueryParameters> = MemoizedUtil.select(this._store, SipStatusHistoryState, state => state.queryParameters);

  @ViewChild("formPresentational")
  readonly formPresentational: SipFormPresentational;

  @ViewChild("matTabGroup")
  readonly matTabGroup: MatTabGroup;

  message: string;
  tabSelected: SipTabEnum;

  @OverrideProperty()
  readonly editAvailable: boolean = false;

  @OverrideProperty()
  readonly deleteAvailable: boolean = false;

  get packageStatusEnum(): typeof PackageStatusEnum {
    return PackageStatusEnum;
  }

  readonly KEY_PARAM_NAME: keyof Sip & string = undefined;
  readonly KEY_DELETE_BUTTON: string = undefined;
  readonly KEY_EDIT_BUTTON: string = undefined;
  readonly KEY_BACK_BUTTON: string = MARK_AS_TRANSLATABLE("preservation.sip.button.goBackToList");
  readonly KEY_BACK_TO_DETAIL_BUTTON: string = undefined;

  listExtraButtons: ExtraButtonToolbar<Sip>[] = [
    {
      color: "primary",
      icon: IconNameEnum.download,
      displayCondition: current => !isNullOrUndefined(current) && !isNullOrUndefined(current.info) && current.info.status === PackageStatusEnum.COMPLETED,
      callback: () => this.download(),
      labelToTranslate: MARK_AS_TRANSLATABLE("preservation.sip.button.download"),
      order: 40,
    },
    {
      color: "primary",
      icon: IconNameEnum.resume,
      displayCondition: current => !isNullOrUndefined(current) && !isNullOrUndefined(current.info) && current.info.status === PackageStatusEnum.IN_ERROR,
      callback: () => this.resume(),
      labelToTranslate: MARK_AS_TRANSLATABLE("preservation.sip.button.resume"),
      order: 40,
    },
  ];

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _dialog: MatDialog,
              protected readonly _router: Router,
              private readonly _securityService: SecurityService) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.preservation_sip, preservationSipActionNameSpace, LocalStateEnum.preservation);
  }

  ngOnInit(): void {
    super.ngOnInit();

    this.retrieveCurrentModelWithUrl();

    this._computeCurrentTab();
    this.subscribe(this._router.events
      .pipe(
        filter(event => event instanceof NavigationCancel || event instanceof NavigationEnd),
        distinctUntilChanged(),
        tap(event => {
          this._computeCurrentTab();
          this.matTabGroup.selectedIndex = this.tabSelected;
        }),
      ),
    );
  }

  ngOnDestroy(): void {
    super.ngOnDestroy();
    this.cleanState();
  }

  private _computeCurrentTab(): void {
    const tabRouteSelected = SipHelper.getTabRouteSelected(this._route);
    this.tabSelected = SipHelper.getTabSelectedIndexWithRoute(tabRouteSelected);
  }

  getSubResourceWithParentId(id: string): void {
  }

  showHistory(): void {
    const dialogRef = this._dialog.open(SharedHistoryDialog, {
      width: environment.modalWidth,
      data: {
        parentId: null,
        resourceResId: this._resId,
        name: this._state,
        statusHistory: this.historyObs,
        isLoading: this.isLoadingHistoryObs,
        queryParametersObs: this.queryParametersObs,
        state: PreservationSipStatusHistoryAction,
      } as StatusHistoryDialog,
    });
  }

  navigateToTab(tabSelected: SipTabEnum): void {
    const tabRouteSelected = SipHelper.getTabRouteSelectedWithTabIndex(tabSelected);
    const path = [RoutesEnum.preservationSipDetail, this._resId, tabRouteSelected];
    this._store.dispatch(new Navigate(path));
  }

  download(): void {
    this._store.dispatch(new PreservationSipAction.Download(this._resId));
  }

  resume(): void {
    this._store.dispatch(new PreservationSipAction.Resume(this._resId));
  }
}
