import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
  ViewChild,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {SipStateModel} from "@app/features/preservation/sip/stores/sip.state";
import {environment} from "@environments/environment";
import {Sip} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SipFormPresentational} from "@preservation/sip/components/presentationals/sip-form/sip-form.presentational";
import {SipDataFile} from "@preservation/sip/models/sip-data-file.model";
import {PreservationSipDataFileAction} from "@preservation/sip/stores/data-file/sip-data-file.action";
import {SipDataFileState} from "@preservation/sip/stores/data-file/sip-data-file.state";
import {PreservationSipDataFileStatusHistoryAction} from "@preservation/sip/stores/data-file/status-history/sip-data-file-status-history.action";
import {SipDataFileStatusHistoryState} from "@preservation/sip/stores/data-file/status-history/sip-data-file-status-history.state";
import {preservationSipActionNameSpace} from "@preservation/sip/stores/sip.action";
import {
  SharedFileDetailDialog,
  SharedFileDetailDialogData,
} from "@shared/components/dialogs/shared-file-detail/shared-file-detail.dialog";
import {SharedAbstractDetailEditRoutable} from "@shared/components/routables/shared-abstract-detail-edit/shared-abstract-detail-edit.routable";
import {ComplianceLevelEnumHelper} from "@shared/enums/business/compliance-level.enum";
import {
  DataFileStatusEnum,
  DataFileStatusEnumHelper,
} from "@shared/enums/business/data-file-status.enum";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {AppRoutesEnum} from "@shared/enums/routes.enum";
import {DataTableActions} from "@shared/models/data-table-actions.model";
import {DataTableColumns} from "@shared/models/data-table-columns.model";
import {StatusHistory} from "@shared/models/status-history.model";
import {Observable} from "rxjs";
import {
  CompositionState,
  isNullOrUndefined,
  MemoizedUtil,
  OrderEnum,
  QueryParameters,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-sip-file-routable",
  templateUrl: "./sip-file.routable.html",
  styleUrls: ["./sip-file.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SipFileRoutable extends SharedAbstractDetailEditRoutable<Sip, SipStateModel> implements OnInit {
  isLoadingDataFileObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, SipDataFileState);
  listDataFileObs: Observable<SipDataFile[]> = CompositionState.list(this._store, SipDataFileState);
  queryParametersObs: Observable<QueryParameters> = CompositionState.queryParameters(this._store, SipDataFileState);

  isLoadingHistoryObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, SipDataFileStatusHistoryState);
  dataFileHistoryObs: Observable<StatusHistory[]> = MemoizedUtil.select(this._store, SipDataFileStatusHistoryState, state => state.history);
  queryParametersHistoryObs: Observable<QueryParameters> = MemoizedUtil.select(this._store, SipDataFileStatusHistoryState, state => state.queryParameters);

  @ViewChild("formPresentational")
  readonly formPresentational: SipFormPresentational;

  columns: DataTableColumns<SipDataFile>[];
  actions: DataTableActions<SipDataFile>[];

  readonly KEY_PARAM_NAME: keyof Sip & string = undefined;

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _dialog: MatDialog) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.preservation_sip, preservationSipActionNameSpace, LocalStateEnum.preservation);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.getCurrentModelOnParent();

    this.columns = [
      {
        field: "fileName",
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.fileName"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isSortable: true,
        isFilterable: false,
      },
      {
        field: "creation.when" as any,
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.creation.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
      },
      {
        field: "status",
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.status"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isSortable: false,
        isFilterable: false,
        translate: true,
        filterEnum: DataFileStatusEnumHelper.getListKeyValue(),
      },
      {
        field: "complianceLevel",
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.complianceLevel"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isSortable: false,
        isFilterable: false,
        translate: true,
        component: DataTableComponentEnum.conformityLevelStar,
        filterEnum: ComplianceLevelEnumHelper.getListKeyValue(),
      },
    ];

    this.actions = [
      {
        logo: IconNameEnum.resume,
        callback: (sipDataFile: SipDataFile) => this.resumeDataFile(this._resId, sipDataFile),
        placeholder: MARK_AS_TRANSLATABLE("crud.list.action.resume"),
        displayOnCondition: current => isNullOrUndefined(current) || current.status === DataFileStatusEnum.IN_ERROR,
      },
      {
        logo: IconNameEnum.download,
        callback: (sipDataFile: SipDataFile) => this.downloadDataFile(this._resId, sipDataFile),
        placeholder: MARK_AS_TRANSLATABLE("crud.list.action.download"),
        displayOnCondition: (sipDataFile: SipDataFile) => true,
      },
    ];
  }

  private getCurrentModelOnParent(): void {
    this._resId = this._route.snapshot.parent.paramMap.get(AppRoutesEnum.paramIdWithoutPrefixParam);
    this.getDepositById(this._resId);
  }

  private getDepositById(id: string): void {
    // this._store.dispatch(ResourceActionHelper.getById(preservationSipActionNameSpace, id, true));
    this.getSubResourceWithParentId(id);
  }

  getSubResourceWithParentId(id: string): void {
    // const queryParameter = new QueryParameters();
    // MappingObjectUtil.set(queryParameter.search.searchItems, "relativeLocation", "/");
    this._store.dispatch(new PreservationSipDataFileAction.GetAll(id, null, true));
    // this._store.dispatch(new PreservationSipDataFileAction.GetListFolder(id));
  }

  onQueryParametersEvent(queryParameters: QueryParameters): void {
    this._store.dispatch(new PreservationSipDataFileAction.ChangeQueryParameters(this._resId, queryParameters, true));
    // this._changeDetector.detectChanges(); // Allow to display spinner the first time
  }

  refresh(): void {
    this._store.dispatch(new PreservationSipDataFileAction.Refresh(this._resId));
  }

  download($event: SipDataFile): void {
    this._store.dispatch(new PreservationSipDataFileAction.Download(this._resId, $event));
  }

  showDetail(sipDataFile: SipDataFile): void {
    const dialogRef = this._dialog.open(SharedFileDetailDialog, {
      data: {
        dataFile: sipDataFile,
        resId: sipDataFile.resId,
        parentId: this._resId,
        isReadonly: true,
        isLoading: this.isLoadingHistoryObs,
        statusHistory: this.dataFileHistoryObs,
        queryParametersObs: this.queryParametersHistoryObs,
        state: PreservationSipDataFileStatusHistoryAction,
        name: LocalStateEnum.preservation_sip,
        buttons: [
          {
            labelToTranslate: MARK_AS_TRANSLATABLE("deposit.file.button.download"),
            color: "primary",
            icon: IconNameEnum.download,
            callback: (current) => this.downloadDataFile(this._resId, sipDataFile),
            order: 2,
          },
          {
            labelToTranslate: MARK_AS_TRANSLATABLE("deposit.file.button.resume"),
            color: "primary",
            icon: IconNameEnum.resume,
            callback: (current) => this.resumeDataFile(this._resId, sipDataFile),
            displayCondition: current => current.status === DataFileStatusEnum.IN_ERROR,
            order: 3,
          },
        ],
      } as SharedFileDetailDialogData<SipDataFile>,
      width: environment.modalWidth,
    });
  }

  private downloadDataFile(parentId: string, dataFile: SipDataFile): void {
    this._store.dispatch(new PreservationSipDataFileAction.Download(parentId, dataFile));
  }

  private resumeDataFile(parentId: string, dataFile: SipDataFile): void {
    this._store.dispatch(new PreservationSipDataFileAction.Resume(parentId, dataFile));

  }
}
