import {HttpClient} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {
  defaultSipDataFileValue,
  SipDataFileState,
  SipDataFileStateModel,
} from "@app/features/preservation/sip/stores/data-file/sip-data-file.state";
import {
  PreservationSipAction,
  preservationSipActionNameSpace,
} from "@app/features/preservation/sip/stores/sip.action";
import {
  SipStatusHistoryState,
  SipStatusHistoryStateModel,
} from "@app/features/preservation/sip/stores/status-history/sip-status-history.state";
import {IngestResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {Sip} from "@models";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiActionEnum} from "@shared/enums/api-action.enum";
import {DownloadService} from "@shared/services/download.service";
import {defaultStatusHistoryInitValue} from "@shared/stores/status-history/status-history.state";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  NotificationService,
  ResourceState,
  ResourceStateModel,
  StoreUtil,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

export interface SipStateModel extends ResourceStateModel<Sip> {
  preservation_sip_dataFile: SipDataFileStateModel;
  isLoadingDataFile: boolean;
  preservation_sip_statusHistory: SipStatusHistoryStateModel;
}

@Injectable()
@State<SipStateModel>({
  name: LocalStateEnum.preservation_sip,
  defaults: {
    ...defaultResourceStateInitValue(),
    preservation_sip_dataFile: defaultSipDataFileValue(),
    isLoadingDataFile: false,
    preservation_sip_statusHistory: {...defaultStatusHistoryInitValue()},
  },
  children: [
    SipDataFileState,
    SipStatusHistoryState,
  ],
})
export class SipState extends ResourceState<SipStateModel, Sip> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions,
              protected httpClient: HttpClient,
              private downloadService: DownloadService) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationSipActionNameSpace,
    });
  }

  protected get _urlResource(): string {
    return IngestResourceApiEnum.sip;
  }

  @Selector()
  static isLoading(state: SipStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static currentTitle(state: SipStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return state.current.info.name;
  }

  @Selector()
  static isLoadingWithDependency(state: SipStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static isReadyToBeDisplayed(state: SipStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: SipStateModel): boolean {
    return true;
  }

  @Action(PreservationSipAction.Download)
  download(ctx: StateContext<SipStateModel>, action: PreservationSipAction.Download): void {
    const fileName = "sip_" + action.id + ".zip";
    this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.sip.file.download"), true);
    this.downloadService.download(`${this._urlResource}/${action.id}/${ApiActionEnum.DL}`, fileName).subscribe(() => {
      this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.sip.file.downloadWithSuccess"), true);
    });
  }

  @Action(PreservationSipAction.Resume)
  resume(ctx: StateContext<SipStateModel>, action: PreservationSipAction.Resume): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    return this.httpClient.post<string>(`${this._urlResource}/${action.id}/${ApiActionEnum.RESUME}`, null)
      .pipe(
        tap(result => {
          ctx.dispatch(new PreservationSipAction.ResumeSuccess(action));
        }),
        catchError(error => {
          ctx.dispatch(new PreservationSipAction.ResumeFail(action));
          throw error;
        }),
      );
  }

  @Action(PreservationSipAction.ResumeSuccess)
  resumeSuccess(ctx: StateContext<SipStateModel>, action: PreservationSipAction.ResumeSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.sip.action.resume.success"), true);
  }

  @Action(PreservationSipAction.ResumeFail)
  resumeFail(ctx: StateContext<SipStateModel>, action: PreservationSipAction.ResumeFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showError(MARK_AS_TRANSLATABLE("notification.sip.action.resume.fail"));
  }
}
