import {
  Inject,
  OnInit,
} from "@angular/core";
import {
  MAT_DIALOG_DATA,
  MatDialog,
  MatDialogRef,
} from "@angular/material/dialog";
import {SharedAbstractContainer} from "@app/shared/components/containers/shared-abstract/shared-abstract.container";
import {environment} from "@environments/environment";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SharedHistoryDialog} from "@shared/components/dialogs/shared-history/shared-history.dialog";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {HistoryStatusEnumHelper} from "@shared/helpers/history-status-enum.helper";
import {DataTableColumns} from "@shared/models/data-table-columns.model";
import {ExtraButtonToolbar} from "@shared/models/extra-button-toolbar.model";
import {StatusHistoryDialog} from "@shared/models/status-history-dialog.model";
import {StatusHistory} from "@shared/models/status-history.model";
import {BreakpointService} from "@shared/services/breakpoint.service";
import {StatusHistoryNamespace} from "@shared/stores/status-history/status-history-namespace.model";
import {Observable} from "rxjs";
import {
  BaseResource,
  ClipboardUtil,
  MARK_AS_TRANSLATABLE,
  NotificationService,
  OrderEnum,
  QueryParameters,
} from "solidify-frontend";

export abstract class SharedDetailDialog<TResource extends BaseResource> extends SharedAbstractContainer implements OnInit {
  paramMessage: { name: string } = {name: ""};
  columns: DataTableColumns[];

  protected constructor(protected readonly _store: Store,
                        protected readonly _dialog: MatDialog,
                        protected readonly _actions$: Actions,
                        protected readonly _breakpointService: BreakpointService,
                        protected readonly _notificationService: NotificationService,
                        protected readonly dialogRef: MatDialogRef<SharedDetailDialog<TResource>>,
                        @Inject(MAT_DIALOG_DATA) public data: SharedDetailDialogData<TResource>) {
    super();
  }

  get breakpointService(): BreakpointService {
    return this._breakpointService;
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.paramMessage.name = this.data.name;
    this.columns = [
      {
        field: "changeTime",
        header: MARK_AS_TRANSLATABLE("app.history.table.changeTime"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.descending,
        isSortable: true,
        isFilterable: false,
      },
      {
        field: "status",
        header: MARK_AS_TRANSLATABLE("app.history.table.status"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isSortable: false,
        isFilterable: false,
        filterEnum: HistoryStatusEnumHelper.getListKeyValue(),
        translate: true,
        width: "110px",
      },
      {
        field: "description",
        header: MARK_AS_TRANSLATABLE("app.history.table.description"),
        type: FieldTypeEnum.number,
        order: OrderEnum.none,
        isSortable: true,
        isFilterable: false,
      },
      {
        field: "creatorName",
        header: MARK_AS_TRANSLATABLE("app.history.table.createdBy"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isSortable: false,
        isFilterable: false,
        width: "140px",
      },
    ];
  }

  close(): void {
    this.dialogRef.close();
  }

  showHistory(): void {
    const dialogRef = this._dialog.open(SharedHistoryDialog, {
      width: environment.modalWidth,
      data: {
        parentId: this.data.parentId,
        resourceResId: this.data.resId,
        name: this.data.name,
        statusHistory: this.data.statusHistory,
        isLoading: this.data.isLoading,
        queryParametersObs: this.data.queryParametersObs,
        state: this.data.state,
      } as StatusHistoryDialog,
    });
  }

  getPuidLink(puid: string): string {
    return environment.urlNationalArchivePronom + puid;
  }

  copyKey(): void {
    ClipboardUtil.copyStringToClipboard(this.data.resId);
    this._notificationService.showInformation(MARK_AS_TRANSLATABLE("app.notification.idCopyToClipboard"));
  }

  pagination($event: QueryParameters): void {
    this._store.dispatch(new this.data.state.ChangeQueryParameters($event, this.data.resId, this.data.parentId));
  }

}

export interface SharedDetailDialogData<TResource extends BaseResource> {
  parentId: string;
  resId: string;
  isReadonly: boolean;
  statusHistory: Observable<StatusHistory[]>;
  isLoading: Observable<boolean>;
  queryParametersObs: Observable<QueryParameters>;
  state: StatusHistoryNamespace;
  name: LocalStateEnum;
  buttons: ExtraButtonToolbar<TResource>[];
}
