import {SearchScopeEnum} from "@app/features/home/enums/search-scope.enum";
import {
  ArchiveMetadata,
  MetadataEnum,
} from "@app/shared/models/business/archive-metadata.model";
import {isNullOrUndefined} from "solidify-frontend";
import Metadata = ArchiveMetadata.PublicMetadata;

const DATACITE_NAMESPACE: string = "http://datacite.org/schema/kernel-4";
const DLCM_URI_FINAL_ACCESS_LEVEL: string = "https://www.dlcm.ch/access-level/final";
const DLCM_URI_EMBARGO_ACCESS_LEVEL: string = "https://www.dlcm.ch/access-level/embargo";

export class MetadataUtil {
  private static readonly domParser: DOMParser = new DOMParser();

  private static parse(metadataXml: string): Document {
    return this.domParser.parseFromString(metadataXml, "text/xml");
  }

  private static prefixNSResolver(prefix: string): string | null {
    if (prefix === "d") {
      return DATACITE_NAMESPACE;
    } else {
      return null;
    }
  }

  private static getInformation(xpath: string, searchScope: SearchScopeEnum, metadata: Metadata): string {
    let metadataXml: string;
    if (searchScope === SearchScopeEnum.public || isNullOrUndefined(metadata[MetadataEnum.dlcmXml])) {
      metadataXml = metadata[MetadataEnum.dataciteXml];
    } else {
      metadataXml = metadata[MetadataEnum.dlcmXml];
      xpath = "/" + xpath;
    }
    if (isNullOrUndefined(metadataXml)) {
      console.error("metadataXml provided undefined");
      return "";
    }
    const xmlDoc = this.parse(metadataXml);
    const result = xmlDoc.evaluate(xpath, xmlDoc, this.prefixNSResolver, XPathResult.STRING_TYPE, null);
    return result.stringValue;
  }

  static getTitle(searchScope: SearchScopeEnum, metadata: Metadata): string {
    const xpath = "/d:resource/d:titles/d:title";
    return this.getInformation(xpath, searchScope, metadata);
  }

  static getOrganizationalUnitResId(metadata: Metadata): string {
    return metadata[MetadataEnum.aipOrganizationalUnit];
  }

  static getPublicationYear(searchScope: SearchScopeEnum, metadata: Metadata): string {
    const xpath = "/d:resource/d:publicationYear";
    return this.getInformation(xpath, searchScope, metadata);
  }

  static getAccessLevel(searchScope: SearchScopeEnum, metadata: Metadata): string {
    const xpath = `/d:resource/d:rightsList/d:rights[@rightsURI="${DLCM_URI_FINAL_ACCESS_LEVEL}"]`;
    return this.getInformation(xpath, searchScope, metadata);
  }

  static getEmbargoAccessLevel(searchScope: SearchScopeEnum, metadata: Metadata): string {
    const xpath = `/d:resource/d:rightsList/d:rights[@rightsURI="${DLCM_URI_EMBARGO_ACCESS_LEVEL}"]`;
    return this.getInformation(xpath, searchScope, metadata);
  }

  static getLicenses(searchScope: SearchScopeEnum, metadata: Metadata): string {
    const xpath = `/d:resource/d:rightsList/d:rights[@rightsURI!="${DLCM_URI_FINAL_ACCESS_LEVEL}"]`;
    return this.getInformation(xpath, searchScope, metadata);
  }

  static getDescription(searchScope: SearchScopeEnum, metadata: Metadata): string {
    const xpath = "/d:resource/d:descriptions/d:description";
    return this.getInformation(xpath, searchScope, metadata);
  }

  static getDOI(searchScope: SearchScopeEnum, metadata: Metadata): string {
    const xpath = "/d:resource/d:identifier[@identifierType='DOI']";
    return this.getInformation(xpath, searchScope, metadata);
  }

  static getAuthors(searchScope: SearchScopeEnum, metadata: Metadata): string[] {
    const xpath = "/d:resource/d:creators/d:creator/d:creatorName";
    const results = [];
    const xmlDoc = this.parse(metadata[MetadataEnum.dataciteXml]);
    const result = xmlDoc.evaluate(xpath, xmlDoc, this.prefixNSResolver, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
    for (let i = 0, length = result.snapshotLength; i < length; ++i) {
      results.push(result.snapshotItem(i).textContent);
    }
    return results;
  }

  static getType(searchScope: SearchScopeEnum, metadata: Metadata): string {
    const xpath = "/d:resource/d:resourceType";
    return this.getInformation(xpath, searchScope, metadata);
  }

  static getEmbargoEndDate(searchScope: SearchScopeEnum, metadata: Metadata): Date | null {
    return this.getDate("Available", searchScope, metadata);
  }

  static getAcceptedDate(searchScope: SearchScopeEnum, metadata: Metadata): Date | null {
    return this.getDate("Accepted", searchScope, metadata);
  }

  static getSubmittedDate(searchScope: SearchScopeEnum, metadata: Metadata): Date | null {
    return this.getDate("Submitted", searchScope, metadata);
  }

  static getSize(metadata: Metadata): number | null {
    return metadata[MetadataEnum.aipSize];
  }

  static getFileNumber(metadata: Metadata): string | null {
    if (isNullOrUndefined(metadata[MetadataEnum.aipFiles])) {
      return null;
    }
    return metadata[MetadataEnum.aipFiles].toString();
  }

  private static getDate(name: string, searchScope: SearchScopeEnum, metadata: Metadata): Date | null {
    const xpath = `/d:resource/d:dates/d:date[@dateType="${name}"]`;
    const result = this.getInformation(xpath, searchScope, metadata);
    if (isNullOrUndefined(result)) {
      return null;
    }
    const date = new Date(result);
    if (date.toString() === "Invalid Date") {
      return null;
    }
    return date;
  }

  static getDatacite(searchScope: SearchScopeEnum, metadata: Metadata): string {
    if (searchScope === SearchScopeEnum.public) {
      return metadata[MetadataEnum.dataciteXml];
    } else {
      return metadata[MetadataEnum.dlcmXml];
    }
  }
}
